@@ -1,8 +1,9 @@
 #
-# demo.tcl,v 1.97 2005/10/08 14:24:22 jenglish Exp
+# demo.tcl,v 1.100 2006/04/08 00:48:39 jenglish Exp
 #
 # Tile widget set -- widget demo
 #
+package require Tk
 
 eval destroy [winfo children .]		;# in case script is re-sourced
 
@@ -411,26 +412,21 @@ $l.t insert end $txt
 $r.t insert end $txt
 
 ### Scales and sliders pane.
-# (Also: partial test for #1263510)
 #
 proc scales.pane {scales} {
-    #ttk::frame $scales
-    set scales [ttk::frame .scales]
+    ttk::frame $scales
 
     ttk::paned $scales.pw -orient horizontal
     set l [ttk::labelframe $scales.styled -text "Themed" -padding 6]
     set r [labelframe $scales.orig -text "Standard" -padx 6 -pady 6]
 
     ttk::scale $l.scale -orient horizontal -from 0 -to 100 -variable ::V(SCALE)
-    ttk::scale $l.vscale -orient vertical -from 0 -to 100 -variable ::V(VSCALE)
+    ttk::scale $l.vscale -orient vertical -from 100 -to 0 -variable ::V(VSCALE)
     ttk::progressbar $l.progress -orient horizontal -maximum 100 
     ttk::progressbar $l.vprogress -orient vertical -maximum 100
     if {1} {
-        proc progress.inverted {w value} {
-            $w configure -value [expr {[$w cget -maximum] - $value}]
-        }
 	$l.scale configure -command [list $l.progress configure -value]
-	$l.vscale configure -command [list progress.inverted $l.vprogress]
+	$l.vscale configure -command [list $l.vprogress configure -value]
     } else {
 	# This would also work, but the Tk scale widgets 
 	# in the right hand pane cause some interference when 
@@ -483,7 +479,7 @@ proc scales.pane {scales} {
     grid rowconfigure $l 99 -weight 1
 
     scale $r.scale -orient horizontal -from 0 -to 100 -variable ::V(SCALE)
-    scale $r.vscale -orient vertical -from 0 -to 100 -variable ::V(VSCALE)
+    scale $r.vscale -orient vertical -from 100 -to 0 -variable ::V(VSCALE)
     grid $r.scale -sticky news
     grid $r.vscale -sticky nws
 
