@@ -1,4 +1,4 @@
-/* altTheme.c,v 1.58 2005/10/07 16:21:40 jenglish Exp
+/* altTheme.c,v 1.60 2006/04/06 20:02:51 jenglish Exp
  *
  * Copyright (c) 2003, Joe English
  *
@@ -450,7 +450,7 @@ Ttk_StateTable radiobutton_states[] =
 static char *radiobutton_pixels[] = {
     "FFFFAAAAFFFFFFFFFAAAAFFFFFFFFFAAAAFFFFFFFFFAAAAFFFFF",
     "FFAAEEEEAAFFFFFAAEEEEAAFFFFFAAEEEEAAFFFFFAAEEEEAAFFF",
-    "FAEEDDDDEEBFFFAEEDDDDEEBFFFAEEFFFFEEBFFFAEEFFFFEEBFF",
+    "FAEEDDDDECBFFFAEEDDDDECBFFFAEEFFFFECBFFFAEEFFFFECBFF",
     "FAEDDDDDDCBFFFAEDDDDDDCBFFFAEFFFFFFCBFFFAEFFFFFFCBFF",
     "AEDDDDDDDDCBFAEDDDGGDDDCBFAEFFFFFFFFCBFAEFFFHHFFFCBF",
     "AEDDDDDDDDCBFAEDDGGGGDDCBFAEFFFFFFFFCBFAEFFHHHHFFCBF",
@@ -958,6 +958,7 @@ typedef struct
     Tcl_Obj *thicknessObj;	/* Short axis dimension */
     Tcl_Obj *reliefObj;		/* Relief for this object */
     Tcl_Obj *borderObj;		/* Border / background color */
+    Tcl_Obj *borderColorObj;	/* Additional border color */
     Tcl_Obj *borderWidthObj;
     Tcl_Obj *orientObj;		/* Orientation of overall slider */
 } SliderElement;
@@ -974,6 +975,8 @@ static Ttk_ElementOptionSpec SliderElementOptions[] =
 	STR(BORDERWIDTH) },
     { "-background", TK_OPTION_BORDER, Tk_Offset(SliderElement,borderObj),
 	DEFAULT_BACKGROUND },
+    { "-bordercolor", TK_OPTION_COLOR, Tk_Offset(ThumbElement,borderColorObj),
+	"black" },
     { "-orient", TK_OPTION_ANY, Tk_Offset(SliderElement,orientObj),
 	"horizontal" },
     { NULL }
@@ -1009,44 +1012,17 @@ static void SliderElementDraw(
     Drawable d, Ttk_Box b, unsigned int state)
 {
     SliderElement *slider = elementRecord;
-    Tk_3DBorder border = NULL;
-    int relief, borderWidth, orient, n = 0;
-    int w = b.width, h = b.height;
-    XPoint pt[5];
+    Tk_3DBorder border = Tk_Get3DBorderFromObj(tkwin, slider->borderObj);
+    XColor *borderColor = Tk_GetColorFromObj(tkwin, slider->borderColorObj);
+    int relief = TK_RELIEF_RAISED, borderWidth = 2;
 
-    border = Tk_Get3DBorderFromObj(tkwin, slider->borderObj);
-    Ttk_GetOrientFromObj(NULL, slider->orientObj, &orient);
     Tk_GetPixelsFromObj(NULL, tkwin, slider->borderWidthObj, &borderWidth);
     Tk_GetReliefFromObj(NULL, slider->reliefObj, &relief);
 
-    if (relief == TK_RELIEF_FLAT) {
-	Tk_Fill3DRectangle(tkwin, d, border,
-	    b.x, b.y, b.width, b.height,
-	    borderWidth, relief);
-	return;
-    }
-
-    switch (orient) {
-	case TTK_ORIENT_VERTICAL:
-	    h = w/2;
-	    pt[n].x = b.x;             pt[n++].y = b.y;
-	    pt[n].x = b.x;             pt[n++].y = b.y + h;
-	    pt[n].x = b.x + 3*w/4;     pt[n++].y = b.y + h;
-	    pt[n].x = b.x + w;         pt[n++].y = b.y + h/2;
-	    pt[n].x = b.x + 3*w/4;     pt[n++].y = b.y;
-	    break;
-
-	case TTK_ORIENT_HORIZONTAL:
-	    w = h/2;
-	    pt[n].x = b.x;            pt[n++].y = b.y;
-	    pt[n].x = b.x;            pt[n++].y = b.y + 3*h/4;
-	    pt[n].x = b.x + w/2;      pt[n++].y = b.y + h;
-	    pt[n].x = b.x + w;        pt[n++].y = b.y + 3*h/4;
-	    pt[n].x = b.x + w;        pt[n++].y = b.y;
-	    break;
-    }
-
-    Tk_Fill3DPolygon(tkwin, d, border, pt, n, borderWidth, relief);
+    Tk_Fill3DRectangle(tkwin, d, border,
+	b.x, b.y, b.width, b.height,
+	borderWidth, TK_RELIEF_FLAT);
+    DrawBorder(tkwin, d, border, borderColor, b, borderWidth, relief);
 }
 
 static Ttk_ElementSpec SliderElementSpec =
