@@ -1,4 +1,4 @@
-/* button.c,v 1.45 2005/10/08 14:02:47 jenglish Exp
+/* button.c,v 1.47 2006/01/23 02:38:04 jenglish Exp
  * Copyright (c) 2003, Joe English
  *
  * Tile widget set: label, button, checkbutton, radiobutton, and 
@@ -9,7 +9,6 @@
 #include <tk.h>
 #include "tkTheme.h"
 #include "widget.h"
-#include "compat.h"
 
 /* Bit fields for OptionSpec mask field:
  */
@@ -311,7 +310,6 @@ typedef struct
     WidgetCore	core;
     BasePart	base;
     LabelPart	label;
-    LABEL_COMPAT_DECLS
 } Label;
 
 static Tk_OptionSpec LabelOptionSpecs[] =
@@ -329,23 +327,21 @@ static Tk_OptionSpec LabelOptionSpecs[] =
 	NULL, Tk_Offset(Label,label.borderWidthObj), -1,
 	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
     {TK_OPTION_RELIEF, "-relief", "relief", "Relief",
-	"flat", Tk_Offset(Label,label.reliefObj), -1,
+	NULL, Tk_Offset(Label,label.reliefObj), -1,
 	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
     {TK_OPTION_ANCHOR, "-anchor", "anchor", "Anchor",
-	"center", Tk_Offset(Label,label.anchorObj), -1, 
-	0, 0, GEOMETRY_CHANGED},
+	NULL, Tk_Offset(Label,label.anchorObj), -1, 
+	TK_OPTION_NULL_OK, 0, GEOMETRY_CHANGED},
     {TK_OPTION_JUSTIFY, "-justify", "justify", "Justify",
-	"center", Tk_Offset(Label, label.justifyObj), -1,
-	0,0,GEOMETRY_CHANGED },
+	NULL, Tk_Offset(Label, label.justifyObj), -1,
+	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
     {TK_OPTION_PIXELS, "-wraplength", "wrapLength", "WrapLength",
-	"0", Tk_Offset(Label, label.wrapLengthObj), -1,
-	0,0,GEOMETRY_CHANGED /*SB: SIZE_CHANGED*/ },
+	NULL, Tk_Offset(Label, label.wrapLengthObj), -1,
+	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED /*SB: SIZE_CHANGED*/ },
 
     WIDGET_INHERIT_OPTIONS(BaseOptionSpecs)
 };
 
-LABEL_COMPAT_OPTIONS
-
 static WidgetCommandSpec LabelCommands[] =
 {
     { "configure",	WidgetConfigureCommand },
@@ -359,7 +355,7 @@ WidgetSpec LabelWidgetSpec =
 {
     "TLabel",			/* className */
     sizeof(Label),		/* recordSize */
-    LabelCompatOptionSpecs,	/* optionSpecs */
+    LabelOptionSpecs,		/* optionSpecs */
     LabelCommands,		/* subcommands */
     BaseInitialize,		/* initializeProc */
     BaseCleanup,		/* cleanupProc */
@@ -387,7 +383,6 @@ typedef struct
     WidgetCore	core;
     BasePart	base;
     ButtonPart	button;
-    BUTTON_COMPAT_DECLS
 } Button;
 
 /*
@@ -406,8 +401,6 @@ static Tk_OptionSpec ButtonOptionSpecs[] =
     WIDGET_INHERIT_OPTIONS(BaseOptionSpecs)
 };
 
-BUTTON_COMPAT_OPTIONS
-
 static int ButtonConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 {
     Button *buttonPtr = recordPtr;
@@ -463,7 +456,7 @@ WidgetSpec ButtonWidgetSpec =
 {
     "TButton",			/* className */
     sizeof(Button),		/* recordSize */
-    ButtonCompatOptionSpecs,	/* optionSpecs */
+    ButtonOptionSpecs,		/* optionSpecs */
     ButtonCommands,		/* subcommands */
     BaseInitialize,		/* initializeProc */
     BaseCleanup,		/* cleanupProc */
@@ -494,7 +487,6 @@ typedef struct
     WidgetCore core;
     BasePart base;
     CheckbuttonPart checkbutton;
-    CHECKBUTTON_COMPAT_DECLS
 } Checkbutton;
 
 /*
@@ -516,8 +508,6 @@ static Tk_OptionSpec CheckbuttonOptionSpecs[] =
     WIDGET_INHERIT_OPTIONS(BaseOptionSpecs)
 };
 
-CHECKBUTTON_COMPAT_OPTIONS
-
 /*
  * Variable trace procedure for checkbutton -variable option
  */
@@ -657,7 +647,7 @@ WidgetSpec CheckbuttonWidgetSpec =
 {
     "TCheckbutton",		/* className */
     sizeof(Checkbutton),	/* recordSize */
-    CheckbuttonCompatOptionSpecs,	/* optionSpecs */
+    CheckbuttonOptionSpecs,	/* optionSpecs */
     CheckbuttonCommands,	/* subcommands */
     CheckbuttonInitialize,	/* initializeProc */
     CheckbuttonCleanup,		/* cleanupProc */
@@ -688,7 +678,6 @@ typedef struct
     WidgetCore core;
     BasePart base;
     RadiobuttonPart radiobutton;
-    RADIOBUTTON_COMPAT_DECLS
 } Radiobutton;
 
 /*
@@ -711,8 +700,6 @@ static Tk_OptionSpec RadiobuttonOptionSpecs[] =
     WIDGET_INHERIT_OPTIONS(BaseOptionSpecs)
 };
 
-RADIOBUTTON_COMPAT_OPTIONS
-
 /*
  * Variable trace procedure for radiobuttons.
  */
@@ -830,7 +817,7 @@ WidgetSpec RadiobuttonWidgetSpec =
 {
     "TRadiobutton",		/* className */
     sizeof(Radiobutton),	/* recordSize */
-    RadiobuttonCompatOptionSpecs,	/* optionSpecs */
+    RadiobuttonOptionSpecs,	/* optionSpecs */
     RadiobuttonCommands,	/* subcommands */
     BaseInitialize,		/* initializeProc */
     RadiobuttonCleanup,		/* cleanupProc */
@@ -857,7 +844,6 @@ typedef struct
     WidgetCore core;
     BasePart base;
     MenubuttonPart menubutton;
-    MENUBUTTON_COMPAT_DECLS
 } Menubutton;
 
 /*
@@ -879,8 +865,6 @@ static Tk_OptionSpec MenubuttonOptionSpecs[] =
     WIDGET_INHERIT_OPTIONS(BaseOptionSpecs)
 };
 
-MENUBUTTON_COMPAT_OPTIONS
-
 static WidgetCommandSpec MenubuttonCommands[] =
 {
     { "configure",	WidgetConfigureCommand },
@@ -894,7 +878,7 @@ WidgetSpec MenubuttonWidgetSpec =
 {
     "TMenubutton",		/* className */
     sizeof(Menubutton), 	/* recordSize */
-    MenubuttonCompatOptionSpecs,/* optionSpecs */
+    MenubuttonOptionSpecs, 	/* optionSpecs */
     MenubuttonCommands,  	/* subcommands */
     BaseInitialize,     	/* initializeProc */
     BaseCleanup,		/* cleanupProc */
