@@ -1,5 +1,5 @@
 /*
- * entry.c,v 1.35 2005/10/08 14:02:47 jenglish Exp
+ * entry.c,v 1.39 2006/04/17 15:56:05 jenglish Exp
  *
  * DERIVED FROM: tk/generic/tkEntry.c r1.35.
  *
@@ -16,16 +16,14 @@
 
 #include "tkTheme.h"
 #include "widget.h"
-#include "compat.h"
 
 /*
  * Extra bits for core.flags:
  */
 #define GOT_SELECTION		(WIDGET_USER_FLAG<<1)
-#define UPDATE_SCROLLBAR	(WIDGET_USER_FLAG<<2)
-#define SYNCING_VARIABLE	(WIDGET_USER_FLAG<<3)
-#define VALIDATING		(WIDGET_USER_FLAG<<4)
-#define VALIDATION_SET_VALUE	(WIDGET_USER_FLAG<<5)
+#define SYNCING_VARIABLE	(WIDGET_USER_FLAG<<2)
+#define VALIDATING		(WIDGET_USER_FLAG<<3)
+#define VALIDATION_SET_VALUE	(WIDGET_USER_FLAG<<4)
 
 /*
  * Definitions for -validate option values:
@@ -57,17 +55,13 @@ static const char *validateReasonStrings[] = {
  * Dependencies:
  *
  * textVariableTrace	: textVariableObj
- * avgWidth		: fontObj
  *
  * numBytes,numChars	: string
  * displayString	: numChars, showChar
  * layoutHeight,
  * layoutWidth,
  * textLayout		: fontObj, displayString
- * layoutX, layoutY	: textLayout, justify, leftIndex
- * leftIndex		: (may be modified based on [window size])
- * rightIndex		: textLayout, justify, leftIndex, [window size]
- * [scrollbar position]	: leftIndex, rightIndex, numChars
+ * layoutX, layoutY	: textLayout, justify, xscroll.first
  *
  * Invariants:
  *
@@ -100,10 +94,11 @@ typedef struct
     int numChars;		/* Length of string in characters. */
 
     int insertPos;		/* Insert index */
-    int leftIndex;		/* Index of left-most visible character */
     int selectFirst;		/* Index of start of selection, or -1 */
     int selectLast;		/* Index of end of selection, or -1 */
-    int selectAnchor;		/* Fixed end of selection */
+
+    Scrollable xscroll;		/* Current scroll position */
+    ScrollHandle xscrollHandle;
 
     /*
      * Options managed by Tk_SetOptions:
@@ -111,8 +106,6 @@ typedef struct
     Tcl_Obj *textVariableObj;	/* Name of linked variable */
     int exportSelection;	/* Tie internal selection to X selection? */
 
-    char *scrollCmd;		/* Scrollbar command prefix */
-
     VMODE validate;		/* Validation mode */
     char *validateCmd;		/* Validation script template */
     char *invalidCmd;		/* Invalid callback script template */
@@ -132,7 +125,6 @@ typedef struct
      * Derived resources:
      */
     Ttk_TraceHandle *textVariableTrace;
-    int avgWidth;		/* Average character width of text font */
 
     char *displayString;	/* String to use when displaying */
     Tk_TextLayout textLayout;	/* Cached text layout information. */
@@ -140,7 +132,6 @@ typedef struct
     int layoutHeight;		/* textLayout height */
 
     int layoutX, layoutY;	/* Origin for text layout. */
-    int rightIndex;	 	/* Index of rightmost visible character+1 */
 
 } EntryPart;
 
@@ -148,7 +139,6 @@ typedef struct
 {
     WidgetCore	core;
     EntryPart	entry;
-    ENTRY_COMPAT_DECLS
 } Entry;
 
 /*
@@ -156,6 +146,7 @@ typedef struct
  */
 #define STATE_CHANGED	 	(0x100)	/* -state option changed */
 #define TEXTVAR_CHANGED	 	(0x200)	/* -textvariable option changed */
+#define SCROLLCMD_CHANGED	(0x400)	/* -xscrollcommand option changed */
 
 /*
  * Default option values:
@@ -201,8 +192,8 @@ static Tk_OptionSpec EntryOptionSpecs[] =
 	DEF_ENTRY_WIDTH, Tk_Offset(Entry, entry.widthObj), -1,
 	0,0,GEOMETRY_CHANGED},
     {TK_OPTION_STRING, "-xscrollcommand", "xScrollCommand", "ScrollCommand",
-	NULL, -1, Tk_Offset(Entry, entry.scrollCmd),
-	TK_OPTION_NULL_OK, 0, 0},
+	NULL, -1, Tk_Offset(Entry, entry.xscroll.scrollCmd),
+	TK_OPTION_NULL_OK, 0, SCROLLCMD_CHANGED},
 
     /* EntryStyleData options:
      */
@@ -212,6 +203,8 @@ static Tk_OptionSpec EntryOptionSpecs[] =
     {TK_OPTION_COLOR, "-background", "windowColor", "WindowColor",
 	NULL, Tk_Offset(Entry, entry.styleData.backgroundObj), -1,
 	TK_OPTION_NULL_OK,0,0},
+
+#if UNDOCUMENTED_OPTIONS
     {TK_OPTION_COLOR, "-insertcolor", "insertColor", "TextColor",
 	NULL, Tk_Offset(Entry, entry.styleData.insertColorObj), -1,
 	TK_OPTION_NULL_OK,0,0},
@@ -227,12 +220,11 @@ static Tk_OptionSpec EntryOptionSpecs[] =
     {TK_OPTION_PIXELS, "-selectborderwidth","selectBorderWidth","BorderWidth",
 	NULL, Tk_Offset(Entry, entry.styleData.selBorderWidthObj), -1,
         TK_OPTION_NULL_OK, 0, 0},
+#endif
 
     WIDGET_INHERIT_OPTIONS(CoreOptionSpecs)
 };
 
-ENTRY_COMPAT_OPTIONS
-
 /*------------------------------------------------------------------------
  * +++ EntryStyleData management.
  * 	This is still more awkward than it should be;
@@ -331,8 +323,6 @@ static char *EntryDisplayString(const char *showChar, int numChars)
 /* EntryUpdateTextLayout --
  * 	Recompute textLayout, layoutWidth, and layoutHeight
  * 	from displayString and fontObj.
- *
- * See also: EntryPlaceTextLayout
  */
 static void EntryUpdateTextLayout(Entry *entryPtr)
 {
@@ -342,7 +332,6 @@ static void EntryUpdateTextLayout(Entry *entryPtr)
 	    entryPtr->entry.displayString, entryPtr->entry.numChars,
 	    0/*wraplength*/, entryPtr->entry.justify, TK_IGNORE_NEWLINES,
 	    &entryPtr->entry.layoutWidth, &entryPtr->entry.layoutHeight);
-    entryPtr->core.flags |= UPDATE_SCROLLBAR;
 }
 
 /* EntryEditable --
@@ -417,75 +406,6 @@ static void EntryOwnSelection(Entry *entryPtr)
     }
 }
 
-/*------------------------------------------------------------------------
- * +++ Scrollbar interface.
- */
-
-/* EntryVisibleRange --
- *	Return information about the range of the entry that is
- *	currently visible.
- *
- * Results:
- *	*firstPtr and *lastPtr are modified to hold fractions between
- *	0 and 1 identifying the range of characters visible in the
- *	entry.
- *
- * Notes:
- * 	Result may be incorrect if the widget has not been redisplayed
- * 	since leftIndex changed.
- */
-static void
-EntryVisibleRange(Entry *entryPtr, double *firstPtr, double *lastPtr)
-{
-    int numChars = entryPtr->entry.numChars;
-    if (numChars == 0) {
-	*firstPtr = 0.0;
-	*lastPtr = 1.0;
-    } else {
-	*firstPtr = (double)entryPtr->entry.leftIndex / numChars;
-	*lastPtr = (double)entryPtr->entry.rightIndex / numChars;
-    }
-}
-
-/* EntryUpdateScrollbar --
- *	Call the -scrollcommand callback to sync the scrollbar with
- *	the entry's visible range.
- *
- * 	Returns: Whatever the -scrollcommand does.
- */
-static int
-EntryUpdateScrollbar(Tcl_Interp *interp, Entry *entryPtr)
-{
-    char args[TCL_DOUBLE_SPACE * 2];
-    double first, last;
-    int code;
-
-    entryPtr->core.flags &= ~UPDATE_SCROLLBAR;
-    if (entryPtr->entry.scrollCmd == NULL)
-	return TCL_OK;
-
-    EntryVisibleRange(entryPtr, &first, &last);
-    sprintf(args, " %g %g", first, last);
-
-    code = Tcl_VarEval(interp, entryPtr->entry.scrollCmd, args, NULL);
-
-    if (WidgetDestroyed(&entryPtr->core))
-	return TCL_ERROR;
-
-    if (code != TCL_OK) {
-	/* Disable the -scrollcommand, add to stack trace:
-	 */
-	ckfree(entryPtr->entry.scrollCmd);
-	entryPtr->entry.scrollCmd = 0;
-
-	Tcl_AddErrorInfo(interp,
-		"\n    (horizontal scrolling command executed by ");
-	Tcl_AddErrorInfo(interp, Tk_PathName(entryPtr->core.tkwin));
-	Tcl_AddErrorInfo(interp, ")");
-    }
-    return code;
-}
-
 /*------------------------------------------------------------------------
  * +++ Validation.
  */
@@ -780,8 +700,8 @@ static int AdjustIndex(int i0, int index, int nChars)
 
 /* AdjustIndices --
  * 	Adjust all internal entry indexes to account for change.
- * 	Note that insertPos, selectFirst, and selectAnchor have "right gravity",
- * 	while leftIndex and selectLast have "left gravity".
+ * 	Note that insertPos, and selectFirst have "right gravity",
+ * 	while leftIndex (=xscroll.first) and selectLast have "left gravity".
  */
 static void AdjustIndices(Entry *entryPtr, int index, int nChars)
 {
@@ -790,9 +710,8 @@ static void AdjustIndices(Entry *entryPtr, int index, int nChars)
 
     e->insertPos    = AdjustIndex(e->insertPos, index, nChars);
     e->selectFirst  = AdjustIndex(e->selectFirst, index, nChars);
-    e->leftIndex    = AdjustIndex(e->leftIndex, index+g, nChars);
     e->selectLast   = AdjustIndex(e->selectLast, index+g, nChars);
-    e->selectAnchor = AdjustIndex(e->selectAnchor, index, nChars);
+    e->xscroll.first= AdjustIndex(e->xscroll.first, index+g, nChars);
 
     if (e->selectLast <= e->selectFirst)
 	e->selectFirst = e->selectLast = -1;
@@ -989,11 +908,9 @@ DeleteChars(
 
 /* EntryEventProc --
  *	Extra event handling for entry widgets:
- *
- *	+ Triggers validation on FocusIn and FocusOut events
- *	+ Schedules a scrollbar update when widget is resized
+ *	Triggers validation on FocusIn and FocusOut events.
  */
-#define EntryEventMask (FocusChangeMask|StructureNotifyMask)
+#define EntryEventMask (FocusChangeMask)
 static void
 EntryEventProc(ClientData clientData, XEvent *eventPtr)
 {
@@ -1005,9 +922,6 @@ EntryEventProc(ClientData clientData, XEvent *eventPtr)
 	    Tk_DeleteEventHandler(entryPtr->core.tkwin,
 		    EntryEventMask, EntryEventProc, clientData);
 	    break;
-	case ConfigureNotify:
-	    entryPtr->core.flags |= UPDATE_SCROLLBAR;
-	    break;
 	case FocusIn:
 	    EntryRevalidateBG(entryPtr, VALIDATE_FOCUSIN);
 	    break;
@@ -1041,14 +955,13 @@ EntryInitialize(Tcl_Interp *interp, void *recordPtr)
 
     EntryInitStyleDefaults(&entryPtr->entry.styleDefaults);
 
+    entryPtr->entry.xscrollHandle =
+	CreateScrollHandle(&entryPtr->core, &entryPtr->entry.xscroll);
+
     entryPtr->entry.insertPos		= 0;
-    entryPtr->entry.leftIndex		= 0;
-    entryPtr->entry.rightIndex		= 0;
     entryPtr->entry.selectFirst 	= -1;
     entryPtr->entry.selectLast		= -1;
 
-    entryPtr->entry.selectAnchor	= 0;	/* COMPAT */
-
     return TCL_OK;
 }
 
@@ -1060,6 +973,8 @@ EntryCleanup(void *recordPtr)
     if (entryPtr->entry.textVariableTrace)
 	Ttk_UntraceVariable(entryPtr->entry.textVariableTrace);
 
+    FreeScrollHandle(entryPtr->entry.xscrollHandle);
+
     EntryFreeStyleDefaults(&entryPtr->entry.styleDefaults);
 
     Tk_DeleteSelHandler(entryPtr->core.tkwin, XA_PRIMARY, XA_STRING);
@@ -1078,8 +993,6 @@ static int EntryConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
     Entry *entryPtr = recordPtr;
     Tcl_Obj *textVarName = entryPtr->entry.textVariableObj;
     Ttk_TraceHandle *vt = 0;
-    Tk_Window tkwin = entryPtr->core.tkwin;
-    Tk_Font font;
 
     if (mask & TEXTVAR_CHANGED) {
 	if (textVarName && *Tcl_GetString(textVarName)) {
@@ -1102,12 +1015,6 @@ static int EntryConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 	entryPtr->entry.textVariableTrace = vt;
     }
 
-    font = Tk_GetFontFromObj(tkwin, entryPtr->entry.fontObj);
-    entryPtr->entry.avgWidth = Tk_TextWidth(font, "0", 1);
-    if (entryPtr->entry.avgWidth == 0) {
-	entryPtr->entry.avgWidth = 1;
-    }
-
     /* Claim the selection, in case we've suddenly started exporting it.
      */
     if (entryPtr->entry.exportSelection && entryPtr->entry.selectFirst != -1) {
@@ -1120,6 +1027,12 @@ static int EntryConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 	CheckStateOption(&entryPtr->core, entryPtr->entry.stateObj);
     }
 
+    /* Force scrollbar update if needed:
+     */
+    if (mask & SCROLLCMD_CHANGED) {
+	ScrollbarUpdateRequired(entryPtr->entry.xscrollHandle);
+    }
+
     /* Recompute the displayString, in case showChar changed:
      */
     if (entryPtr->entry.displayString != entryPtr->entry.string)
@@ -1164,25 +1077,43 @@ EntryTextArea(Entry *entryPtr)
 {
     WidgetCore *corePtr = &entryPtr->core;
     Ttk_LayoutNode *node = Ttk_LayoutFindNode(corePtr->layout, "textarea");
-    return node
-	? Ttk_LayoutNodeParcel(node)
-	: Ttk_MakeBox(0,0,Tk_Width(corePtr->tkwin),Tk_Height(corePtr->tkwin))
-	;
+    return node ? Ttk_LayoutNodeParcel(node) : Ttk_WinBox(corePtr->tkwin);
 }
 
-/* EntryPlaceTextLayout -
- * 	Determine position of textLayout based on leftIndex, justify,
- * 	and display area (computed indirectly by EntryDoLayout).
+/* EntryCharPosition --
+ * 	Return the X coordinate of the specified character index.
+ * 	Precondition: textLayout and layoutX up-to-date.
+ */
+static int
+EntryCharPosition(Entry *entryPtr, int index)
+{
+    int xPos;
+    Tk_CharBbox(entryPtr->entry.textLayout, index, &xPos, NULL, NULL, NULL);
+    return xPos + entryPtr->entry.layoutX;
+}
+
+/* EntryDoLayout --
+ * 	Layout hook for entry widgets.
  *
- * 	Recalculates layoutX, layoutY, and rightIndex;
- * 	may adjust leftIndex to ensure the maximum #characters are onscreen.
+ * 	Determine position of textLayout based on xscroll.first, justify,
+ * 	and display area.
+ *
+ * 	Recalculates layoutX, layoutY, and rightIndex,
+ * 	and updates xscroll accordingly.
+ * 	May adjust xscroll.first to ensure the maximum #characters are onscreen.
  */
 static void
-EntryPlaceTextLayout(Entry *entryPtr)
+EntryDoLayout(void *recordPtr)
 {
-    Ttk_Box textarea = EntryTextArea(entryPtr);
-    int leftIndex = entryPtr->entry.leftIndex;
-    int rightIndex = entryPtr->entry.rightIndex;
+    Entry *entryPtr = recordPtr;
+    WidgetCore *corePtr = &entryPtr->core;
+    Tk_TextLayout textLayout = entryPtr->entry.textLayout;
+    int leftIndex = entryPtr->entry.xscroll.first;
+    int rightIndex;
+    Ttk_Box textarea;
+
+    Ttk_PlaceLayout(corePtr->layout,corePtr->state,Ttk_WinBox(corePtr->tkwin));
+    textarea = EntryTextArea(entryPtr);
 
     /* Center the text vertically within the available parcel:
      */
@@ -1210,79 +1141,24 @@ EntryPlaceTextLayout(Entry *entryPtr)
 	 * Limit leftIndex to leave at most one character's worth
 	 * of empty space on the right.
 	 */
-	int leftX;
 	int overflow = entryPtr->entry.layoutWidth - textarea.width;
-	int maxLeftIndex =
-	    1 + Tk_PointToChar(entryPtr->entry.textLayout, overflow, 0);
+	int maxLeftIndex = 1 + Tk_PointToChar(textLayout, overflow, 0);
+	int leftX;
 
-	if (leftIndex > maxLeftIndex)
+	if (leftIndex > maxLeftIndex) {
 	    leftIndex = maxLeftIndex;
+	}
 
 	/* Compute layoutX and rightIndex.
+	 * rightIndex is set to one past the last fully-visible character.
 	 */
-	Tk_CharBbox(entryPtr->entry.textLayout, leftIndex,
-		&leftX, NULL, NULL, NULL);
+	Tk_CharBbox(textLayout, leftIndex, &leftX, NULL, NULL, NULL);
+	rightIndex = Tk_PointToChar(textLayout, leftX + textarea.width, 0);
 	entryPtr->entry.layoutX = textarea.x - leftX;
-	rightIndex = Tk_PointToChar(entryPtr->entry.textLayout,
-		leftX + textarea.width, 0);
-	if (rightIndex < entryPtr->entry.numChars)
-	    ++rightIndex;
     }
 
-    if (   leftIndex != entryPtr->entry.leftIndex
-	|| rightIndex != entryPtr->entry.rightIndex)
-    {
-	entryPtr->entry.leftIndex = leftIndex;
-	entryPtr->entry.rightIndex = rightIndex;
-	entryPtr->core.flags |= UPDATE_SCROLLBAR;
-    }
-}
-
-/* EntryUpdateScrollbarBG --
- * 	Idle handler to update the scrollbar.
- */
-static void EntryUpdateScrollbarBG(ClientData clientData)
-{
-    Entry *entryPtr = (Entry *)clientData;
-    Tcl_Interp *interp = entryPtr->core.interp;
-    int code;
-
-    if (WidgetDestroyed(&entryPtr->core)) {
-	Tcl_Release(clientData);
-	return;
-    }
-
-    Tcl_Preserve((ClientData) interp);
-    code = EntryUpdateScrollbar(interp, entryPtr);
-    if (code == TCL_ERROR && !Tcl_InterpDeleted(interp)) {
-	Tcl_BackgroundError(interp);
-    }
-    Tcl_Release((ClientData) interp);
-    Tcl_Release(clientData);
-}
-
-/* EntryDoLayout --
- * 	Layout hook for entry widgets.
- */
-static void
-EntryDoLayout(void *recordPtr)
-{
-    Entry *entryPtr = recordPtr;
-    WidgetCore *corePtr = &entryPtr->core;
-
-    Ttk_PlaceLayout(corePtr->layout,corePtr->state,Ttk_WinBox(corePtr->tkwin));
-
-    EntryPlaceTextLayout(entryPtr);
-
-    /* Update the scrollbar if required, now that we have the position.
-     * It's not safe to enter the interpreter at this point, so schedule
-     * it as an idle handler.
-     */
-    if (entryPtr->core.flags & UPDATE_SCROLLBAR) {
-	entryPtr->core.flags &= ~UPDATE_SCROLLBAR;
-	Tcl_Preserve(recordPtr);
-	Tcl_DoWhenIdle(EntryUpdateScrollbarBG, recordPtr);
-    }
+    Scrolled(entryPtr->entry.xscrollHandle,
+	    leftIndex, rightIndex, entryPtr->entry.numChars);
 }
 
 /* EntryGetGC -- Helper routine.
@@ -1314,30 +1190,36 @@ static void EntryDisplay(void *clientData, Drawable d)
 {
     Entry *entryPtr = clientData;
     Tk_Window tkwin = entryPtr->core.tkwin;
+    int leftIndex = entryPtr->entry.xscroll.first,
+	rightIndex = entryPtr->entry.xscroll.last,
+	selFirst = entryPtr->entry.selectFirst,
+	selLast = entryPtr->entry.selectLast;
     EntryStyleData es;
     GC gc;
-    Ttk_Box textarea = EntryTextArea(entryPtr);
-    int selFirst = entryPtr->entry.selectFirst,
-	selLast = entryPtr->entry.selectLast;
     int showSelection, showCursor;
 
     EntryInitStyleData(entryPtr, &es);
 
+    showCursor = 
+	   (entryPtr->core.flags & CURSOR_ON) != 0
+	&& EntryEditable(entryPtr)
+	&& entryPtr->entry.insertPos >= leftIndex
+	&& entryPtr->entry.insertPos <= rightIndex
+	;
     showSelection =
 	   (entryPtr->core.state & TTK_STATE_DISABLED) == 0
 	&& selFirst > -1
-	&& selLast > entryPtr->entry.leftIndex
-	&& selFirst <= entryPtr->entry.rightIndex
+	&& selLast > leftIndex
+	&& selFirst <= rightIndex
 	;
-    showCursor = (entryPtr->core.flags & CURSOR_ON) && EntryEditable(entryPtr);
 
     /* Adjust selection range to keep in display bounds.
      */
     if (showSelection) {
-	if (selFirst < entryPtr->entry.leftIndex)
-	    selFirst = entryPtr->entry.leftIndex;
-	if (selLast > entryPtr->entry.rightIndex)
-	    selLast = entryPtr->entry.rightIndex;
+	if (selFirst < leftIndex)
+	    selFirst = leftIndex;
+	if (selLast > rightIndex)
+	    selLast = rightIndex;
     }
 
     /* Draw widget background & border
@@ -1348,54 +1230,41 @@ static void EntryDisplay(void *clientData, Drawable d)
      */
     if (showSelection && es.selBorderObj) {
 	Tk_3DBorder selBorder = Tk_Get3DBorderFromObj(tkwin, es.selBorderObj);
-	int bw = 1;
-	int selStartX, selEndX;
+	int selStartX = EntryCharPosition(entryPtr, selFirst);
+	int selEndX = EntryCharPosition(entryPtr, selLast);
+	int borderWidth = 1;
 
-	Tcl_GetIntFromObj(NULL, es.selBorderWidthObj, &bw);
-	Tk_CharBbox(entryPtr->entry.textLayout, selFirst,
-		    &selStartX, NULL, NULL, NULL);
-	selStartX += entryPtr->entry.layoutX;
-	Tk_CharBbox(entryPtr->entry.textLayout, selLast,
-		    &selEndX, NULL, NULL, NULL);
-	selEndX += entryPtr->entry.layoutX;
+	Tcl_GetIntFromObj(NULL, es.selBorderWidthObj, &borderWidth);
 
 	if (selBorder) {
 	    Tk_Fill3DRectangle(tkwin, d, selBorder,
-		selStartX - bw, entryPtr->entry.layoutY - bw,
-		selEndX - selStartX + 2*bw,
-		entryPtr->entry.layoutHeight + 2*bw,
-		bw, TK_RELIEF_RAISED);
+		selStartX - borderWidth, entryPtr->entry.layoutY - borderWidth,
+		selEndX - selStartX + 2*borderWidth,
+		entryPtr->entry.layoutHeight + 2*borderWidth,
+		borderWidth, TK_RELIEF_RAISED);
 	}
     }
 
     /* Draw cursor:
      */
     if (showCursor) {
-	int cursorX;
-	Tk_CharBbox(entryPtr->entry.textLayout, entryPtr->entry.insertPos,
-		&cursorX, NULL, NULL, NULL);
-	cursorX += entryPtr->entry.layoutX;
+	int cursorX = EntryCharPosition(entryPtr, entryPtr->entry.insertPos),
+	    cursorY = entryPtr->entry.layoutY,
+	    cursorHeight = entryPtr->entry.layoutHeight,
+	    cursorWidth = 1;
+
+	Tcl_GetIntFromObj(NULL,es.insertWidthObj,&cursorWidth);
+	if (cursorWidth <= 0) {
+	    cursorWidth = 1;
+	}
 
 	/* @@@ should: maybe: SetCaretPos even when blinked off */
-	/* @@@ should: maybe: don't SetCaretPos when out of bounds */
+	Tk_SetCaretPos(tkwin, cursorX, cursorY, cursorHeight);
 
-	Tk_SetCaretPos(tkwin, cursorX, entryPtr->entry.layoutY,
-		entryPtr->entry.layoutHeight);
-
-	if (textarea.x <= cursorX && cursorX <= textarea.x + textarea.width) {
-	    int insertWidth = 1, x = cursorX, y = entryPtr->entry.layoutY;
-
-	    gc = EntryGetGC(entryPtr, es.insertColorObj);
-	    Tcl_GetIntFromObj(NULL,es.insertWidthObj,&insertWidth);
-	    if (insertWidth <= 0)
-		insertWidth = 1;
-
-	    x -= insertWidth/2;
-	    XFillRectangle(Tk_Display(tkwin), d, gc, x, y, insertWidth,
-		entryPtr->entry.layoutHeight);
-
-	    Tk_FreeGC(Tk_Display(tkwin), gc);
-	}
+	gc = EntryGetGC(entryPtr, es.insertColorObj);
+	XFillRectangle(Tk_Display(tkwin), d, gc, 
+	    cursorX-cursorWidth/2, cursorY, cursorWidth, cursorHeight);
+	Tk_FreeGC(Tk_Display(tkwin), gc);
     }
 
     /* Draw the text:
@@ -1404,16 +1273,17 @@ static void EntryDisplay(void *clientData, Drawable d)
     Tk_DrawTextLayout(
 	Tk_Display(tkwin), d, gc, entryPtr->entry.textLayout,
 	entryPtr->entry.layoutX, entryPtr->entry.layoutY,
-	entryPtr->entry.leftIndex, entryPtr->entry.rightIndex);
+	leftIndex, rightIndex);
     Tk_FreeGC(Tk_Display(tkwin), gc);
 
     /* Overwrite the selected portion (if any) in the -selectforeground color:
      */
     if (showSelection) {
 	gc = EntryGetGC(entryPtr, es.selForegroundObj);
-	Tk_DrawTextLayout(Tk_Display(tkwin), d, gc, entryPtr->entry.textLayout,
-		entryPtr->entry.layoutX, entryPtr->entry.layoutY,
-		selFirst, selLast);
+	Tk_DrawTextLayout(
+	    Tk_Display(tkwin), d, gc, entryPtr->entry.textLayout,
+	    entryPtr->entry.layoutX, entryPtr->entry.layoutY,
+	    selFirst, selLast);
 	Tk_FreeGC(Tk_Display(tkwin), gc);
     }
 }
@@ -1449,11 +1319,9 @@ EntryIndex(
     } else if (strncmp(string, "insert", length) == 0) {
 	*indexPtr = entryPtr->entry.insertPos;
     } else if (strncmp(string, "left", length) == 0) {	/* for debugging */
-	*indexPtr = entryPtr->entry.leftIndex;
+	*indexPtr = entryPtr->entry.xscroll.first;
     } else if (strncmp(string, "right", length) == 0) {	/* for debugging */
-	*indexPtr = entryPtr->entry.rightIndex;
-    } else if (strncmp(string, "anchor", length) == 0) { /* COMPAT */
-	*indexPtr = entryPtr->entry.selectAnchor;
+	*indexPtr = entryPtr->entry.xscroll.last;
     } else if (strncmp(string, "sel.", 4) == 0) {
 	if (entryPtr->entry.selectFirst < 0) {
 	    Tcl_ResetResult(interp);
@@ -1483,8 +1351,8 @@ EntryIndex(
 	*indexPtr = Tk_PointToChar(entryPtr->entry.textLayout,
 		x - entryPtr->entry.layoutX, 0);
 
-	if (*indexPtr < entryPtr->entry.leftIndex) {
-	    *indexPtr = entryPtr->entry.leftIndex;
+	if (*indexPtr < entryPtr->entry.xscroll.first) {
+	    *indexPtr = entryPtr->entry.xscroll.first;
 	}
 
 	/*
@@ -1515,26 +1383,6 @@ badIndex:
     return TCL_ERROR;
 }
 
-/* EntryCompatCommand --
- * 	Compatibility subcommands.  See entry.tcl for details.
- */
-static int
-EntryCompatCommand(
-    Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
-{
-    int status;
-    Tcl_Obj *cmdName = Tcl_NewStringObj("::tile::entry::compat", -1);
-    Tcl_Obj *cmd = Tcl_NewListObj(objc, objv);
-
-    Tcl_ListObjReplace(interp, cmd, 0,0, 1,&cmdName);
-
-    Tcl_IncrRefCount(cmdName); Tcl_IncrRefCount(cmd);
-    status = Tcl_EvalObjEx(interp, cmd, 0);
-    Tcl_DecrRefCount(cmdName); Tcl_DecrRefCount(cmd);
-
-    return status;
-}
-
 /* $entry bbox $index --
  * 	Return the bounding box of the character at the specified index.
  */
@@ -1692,20 +1540,6 @@ static int EntrySelectionClearCommand(
     return TCL_OK;
 }
 
-/* $entry selection from $index (COMPAT)
- * 	Set the selection anchor.
- */
-static int EntrySelectionFromCommand(
-    Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
-{
-    Entry *entryPtr = recordPtr;
-    if (objc != 4) {
-	Tcl_WrongNumArgs(interp, 3, objv, "index");
-	return TCL_ERROR;
-    }
-    return EntryIndex(interp,entryPtr,objv[3],&entryPtr->entry.selectAnchor);
-}
-
 /* $entry selection present --
  * 	Returns 1 if any characters are selected, 0 otherwise.
  */
@@ -1763,10 +1597,6 @@ static int EntrySelectionCommand(
 	{ "clear", EntrySelectionClearCommand },
 	{ "present", EntrySelectionPresentCommand },
 	{ "range", EntrySelectionRangeCommand },
-
-	{ "adjust", EntryCompatCommand },
-	{ "from", EntrySelectionFromCommand },
-	{ "to", EntryCompatCommand },
 	{0,0}
     };
     return WidgetEnsembleCommand(
@@ -1812,67 +1642,13 @@ static int EntryValidateCommand(
     return TCL_OK;
 }
 
-/* $entry xview	-- return current view region
- * $entry xview $index -- set leftIndex
- * $entry xview moveto $fraction
- * $entry xview scroll $number $what -- scrollbar interface
+/* $entry xview	-- horizontal scrolling interface
  */
 static int EntryXViewCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
 {
     Entry *entryPtr = recordPtr;
-    int index;
-
-    if (objc == 2) {
-	double first, last;
-	char buf[TCL_DOUBLE_SPACE * 2];
-
-	EntryPlaceTextLayout(entryPtr);	/* Ensure up-to-date */
-	EntryVisibleRange(entryPtr, &first, &last);
-	sprintf(buf, "%g %g", first, last);
-	Tcl_SetResult(interp, buf, TCL_VOLATILE);
-	return TCL_OK;
-    } else if (objc == 3) {
-	if (EntryIndex(interp, entryPtr, objv[2], &index) != TCL_OK) {
-	    return TCL_ERROR;
-	}
-    } else {
-	double fraction;
-	int count;
-
-	index = entryPtr->entry.leftIndex;
-	switch (Tk_GetScrollInfoObj(interp, objc, objv, &fraction, &count)) {
-	    case TK_SCROLL_ERROR:
-		return TCL_ERROR;
-	    case TK_SCROLL_MOVETO:
-		index = (int) ((fraction * entryPtr->entry.numChars) + 0.5);
-		break;
-	    case TK_SCROLL_UNITS:
-		index += count;
-		break;
-	    case TK_SCROLL_PAGES: {
-		int charsPerPage = (
-		    EntryWidth(entryPtr) / entryPtr->entry.avgWidth)
-		    - 2;
-		if (charsPerPage < 1) {
-		    charsPerPage = 1;
-		}
-		index += count * charsPerPage;
-		break;
-	    }
-	}
-    }
-    if (index >= entryPtr->entry.numChars) {
-	index = entryPtr->entry.numChars - 1;
-    }
-    if (index < 0) {
-	index = 0;
-    }
-    entryPtr->entry.leftIndex = index;
-
-    EntryPlaceTextLayout(entryPtr);
-    TtkRedisplayWidget(&entryPtr->core);
-    return TCL_OK;
+    return ScrollviewCommand(interp, objc, objv, entryPtr->entry.xscrollHandle);
 }
 
 static WidgetCommandSpec EntryCommands[] =
@@ -1883,10 +1659,10 @@ static WidgetCommandSpec EntryCommands[] =
     { "delete", 	EntryDeleteCommand },
     { "get", 		EntryGetCommand },
     { "icursor", 	EntryICursorCommand },
+    { "identify",	WidgetIdentifyCommand },
     { "index", 		EntryIndexCommand },
     { "insert", 	EntryInsertCommand },
     { "instate",	WidgetInstateCommand },
-    { "scan", 		EntryCompatCommand },
     { "selection", 	EntrySelectionCommand },
     { "state",  	WidgetStateCommand },
     { "validate", 	EntryValidateCommand },
@@ -1902,7 +1678,7 @@ WidgetSpec EntryWidgetSpec =
 {
     "TEntry",			/* className */
     sizeof(Entry), 		/* recordSize */
-    EntryCompatOptionSpecs,	/* optionSpecs */
+    EntryOptionSpecs, 		/* optionSpecs */
     EntryCommands,  		/* subcommands */
     EntryInitialize,     	/* initializeProc */
     EntryCleanup,		/* cleanupProc */
