@@ -1,4 +1,4 @@
-/* frame.c,v 1.41 2005/10/08 14:02:47 jenglish Exp
+/* frame.c,v 1.48 2006/04/22 20:43:00 jenglish Exp
  * Copyright (c) 2004, Joe English
  *
  * Tile widget set: frame and labelframe widgets
@@ -8,7 +8,6 @@
 
 #include "tkTheme.h"
 #include "widget.h"
-#include "compat.h"
 #include "manager.h"
 
 /* ======================================================================
@@ -28,7 +27,6 @@ typedef struct
 {
     WidgetCore	core;
     FramePart	frame;
-    FRAME_COMPAT_DECLS
 } Frame;
 
 static Tk_OptionSpec FrameOptionSpecs[] =
@@ -52,8 +50,6 @@ static Tk_OptionSpec FrameOptionSpecs[] =
     WIDGET_INHERIT_OPTIONS(CoreOptionSpecs)
 };
 
-FRAME_COMPAT_OPTIONS
-
 static WidgetCommandSpec FrameCommands[] =
 {
     { "configure",	WidgetConfigureCommand },
@@ -111,7 +107,9 @@ static int FrameSize(void *recordPtr, int *widthPtr, int *heightPtr)
  *
  *	To avoid geometry blinking, we issue a geometry request
  *	in the Configure hook instead of the Size hook, and only
- *	if -width and/or -height is nonzero.
+ *	if -width and/or -height is nonzero and one of them
+ *	or the other size-related options (-borderwidth, -padding)
+ *	has been changed.
  */
 
 static int FrameConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
@@ -143,7 +141,7 @@ static int FrameConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 	return TCL_ERROR;
     }
 
-    if (width > 0 || height > 0) {
+    if ((width > 0 || height > 0) && (mask & GEOMETRY_CHANGED)) {
 	Tk_GeometryRequest(framePtr->core.tkwin, width, height);
     }
 
@@ -155,7 +153,7 @@ WidgetSpec FrameWidgetSpec =
 {
     "TFrame",			/* className */
     sizeof(Frame),		/* recordSize */
-    FrameCompatOptionSpecs,	/* optionSpecs */
+    FrameOptionSpecs,		/* optionSpecs */
     FrameCommands,		/* subcommands */
     NullInitialize,		/* initializeProc */
     NullCleanup,		/* cleanupProc */
@@ -171,53 +169,62 @@ WidgetSpec FrameWidgetSpec =
  * +++ Labelframe widget:
  */
 
-#define LABELFRAME_SPACE	4
 #define DEFAULT_LABELINSET	8
 #define DEFAULT_BORDERWIDTH	2
 
-typedef enum {
-    LABELANCHOR_E, LABELANCHOR_EN, LABELANCHOR_ES,
-    LABELANCHOR_N, LABELANCHOR_NE, LABELANCHOR_NW,
-    LABELANCHOR_S, LABELANCHOR_SE, LABELANCHOR_SW,
-    LABELANCHOR_W, LABELANCHOR_WN, LABELANCHOR_WS
-} LabelAnchor;
-
-static const char *labelAnchorStrings[] = {
-    "e", "en", "es", "n", "ne", "nw", "s", "se", "sw", "w", "wn", "ws", NULL
-};
-
-/* LabelAnchorSide, LabelAnchorSticky --
- * 	Returns the side (resp. sticky flags) corresponding
- * 	to a LabelAnchor value.
- */
-static Ttk_Side LabelAnchorSide(LabelAnchor labelAnchor)
+int TtkGetLabelAnchorFromObj(
+    Tcl_Interp *interp, Tcl_Obj *objPtr, Ttk_PositionSpec *anchorPtr)
 {
-    switch (labelAnchor) {
-	default:
-	case LABELANCHOR_W: case LABELANCHOR_WN: case LABELANCHOR_WS:
-	    return TTK_SIDE_LEFT;
-	case LABELANCHOR_E: case LABELANCHOR_EN: case LABELANCHOR_ES:
-	    return TTK_SIDE_RIGHT;
-	case LABELANCHOR_N: case LABELANCHOR_NE: case LABELANCHOR_NW:
-	    return TTK_SIDE_TOP;
-	case LABELANCHOR_S: case LABELANCHOR_SE: case LABELANCHOR_SW:
-	    return TTK_SIDE_BOTTOM;
+    const char *string = Tcl_GetString(objPtr);
+    char c = *string++;
+    Ttk_PositionSpec flags = 0;
+
+    /* First character determines side:
+     */
+    switch (c) {
+	case 'w' : flags = TTK_PACK_LEFT; 	break;
+	case 'e' : flags = TTK_PACK_RIGHT; 	break;
+	case 'n' : flags = TTK_PACK_TOP; 	break;
+	case 's' : flags = TTK_PACK_BOTTOM; 	break;
+	default  : goto error;
     }
+    
+    /* Remaining characters are as per -sticky:
+     */
+    while ((c = *string++) != '\0') {
+	switch (c) {
+	    case 'w' : flags |= TTK_STICK_W; break;
+	    case 'e' : flags |= TTK_STICK_E; break;
+	    case 'n' : flags |= TTK_STICK_N; break;
+	    case 's' : flags |= TTK_STICK_S; break;
+	    default  : goto error;
+	}
+    }
+
+    *anchorPtr = flags;
+    return TCL_OK;
+
+error:
+    if (interp) {
+	Tcl_ResetResult(interp);
+	Tcl_AppendResult(interp,
+	    "Bad label anchor specification ", Tcl_GetString(objPtr),
+	    NULL);
+    }
+    return TCL_ERROR;
 }
 
-static Ttk_Sticky LabelAnchorSticky(LabelAnchor labelAnchor)
+/* LabelAnchorSide --
+ * 	Returns the side corresponding to a LabelAnchor value.
+ */
+static Ttk_Side LabelAnchorSide(Ttk_PositionSpec flags)
 {
-    switch (labelAnchor) {
-	default:		return 0;
-	case LABELANCHOR_EN:
-	case LABELANCHOR_WN:	return TTK_STICK_N;
-	case LABELANCHOR_ES:
-	case LABELANCHOR_WS:	return TTK_STICK_S;
-	case LABELANCHOR_NE:
-	case LABELANCHOR_SE:	return TTK_STICK_E;
-	case LABELANCHOR_NW:
-	case LABELANCHOR_SW:	return TTK_STICK_W;
-    }
+    if (flags & TTK_PACK_LEFT) 		return TTK_SIDE_LEFT;
+    else if (flags & TTK_PACK_RIGHT)	return TTK_SIDE_RIGHT;
+    else if (flags & TTK_PACK_TOP)	return TTK_SIDE_TOP;
+    else if (flags & TTK_PACK_BOTTOM)	return TTK_SIDE_BOTTOM;
+    /*NOTREACHED*/
+    return TTK_SIDE_TOP;
 }
 
 /*
@@ -229,8 +236,9 @@ typedef struct {
     Tcl_Obj 	*underlineObj;
     Tcl_Obj	*labelWidgetObj;
 
+    Ttk_Manager	*mgr;
     Tk_Window	labelWidget;	/* Set in configureProc */
-    Ttk_Box	slaveParcel;	/* Current position of slave widget */ 
+    Ttk_Box	labelParcel;	/* Set in layoutProc */
 } LabelframePart;
 
 typedef struct
@@ -238,14 +246,15 @@ typedef struct
     WidgetCore  	core;
     FramePart   	frame;
     LabelframePart	label;
-    LABELFRAME_COMPAT_DECLS
 } Labelframe;
 
+#define LABELWIDGET_CHANGED 0x100
+
 static Tk_OptionSpec LabelframeOptionSpecs[] =
 {
-    {TK_OPTION_STRING_TABLE, "-labelanchor", "labelAnchor", "LabelAnchor",
+    {TK_OPTION_STRING, "-labelanchor", "labelAnchor", "LabelAnchor",
 	"nw", Tk_Offset(Labelframe, label.labelAnchorObj),-1,
-        0, (ClientData) labelAnchorStrings, GEOMETRY_CHANGED},
+        0,0,GEOMETRY_CHANGED},
     {TK_OPTION_STRING, "-text", "text", "Text", "",
 	Tk_Offset(Labelframe,label.textObj), -1, 
 	0,0,GEOMETRY_CHANGED }, 
@@ -254,24 +263,21 @@ static Tk_OptionSpec LabelframeOptionSpecs[] =
 	0,0,0 },
     {TK_OPTION_WINDOW, "-labelwidget", "labelWidget", "LabelWidget", NULL,
 	Tk_Offset(Labelframe,label.labelWidgetObj), -1,
-	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
+	TK_OPTION_NULL_OK,0,LABELWIDGET_CHANGED|GEOMETRY_CHANGED },
 
     WIDGET_INHERIT_OPTIONS(FrameOptionSpecs)
 };
 
-LABELFRAME_COMPAT_OPTIONS
-
 /*
  * Labelframe style parameters:
  */
 typedef struct
 {
-    int 	borderWidth;	/* border width */
-    Ttk_Padding	padding;	/* internal padding */
-    LabelAnchor	labelAnchor;	/* corner/side to place label */
-    int 	labelOutside;	/* true=>place label outside border */
-    int 	labelInset;	/* distance between label and adjacent corner */
-    int 	labelSpace;	/* distance between label and border */
+    int 		borderWidth;	/* border width */
+    Ttk_Padding 	padding;	/* internal padding */
+    Ttk_PositionSpec	labelAnchor;	/* corner/side to place label */
+    Ttk_Padding		labelMargins;	/* extra space around label */
+    int 		labelOutside;	/* true=>place label outside border */
 } LabelframeStyle;
 
 static void LabelframeStyleOptions(Labelframe *lf, LabelframeStyle *style)
@@ -281,10 +287,10 @@ static void LabelframeStyleOptions(Labelframe *lf, LabelframeStyle *style)
 
     style->borderWidth = DEFAULT_BORDERWIDTH;
     style->padding = Ttk_UniformPadding(0);
-    style->labelAnchor = LABELANCHOR_NW;
+    style->labelAnchor = TTK_PACK_TOP | TTK_STICK_W;
+    style->labelMargins = 
+	Ttk_MakePadding(DEFAULT_LABELINSET,0,DEFAULT_LABELINSET,0);
     style->labelOutside = 0;
-    style->labelSpace = 1;
-    style->labelInset = DEFAULT_LABELINSET;
 
     if ((objPtr = Ttk_QueryOption(layout, "-borderwidth", 0)) != NULL) {
 	Tk_GetPixelsFromObj(NULL, lf->core.tkwin, objPtr, &style->borderWidth);
@@ -293,141 +299,36 @@ static void LabelframeStyleOptions(Labelframe *lf, LabelframeStyle *style)
 	Ttk_GetPaddingFromObj(NULL, lf->core.tkwin, objPtr, &style->padding);
     }
     if ((objPtr = Ttk_QueryOption(layout,"-labelanchor", 0)) != NULL) {
-	Tcl_GetIndexFromObj(NULL, objPtr, labelAnchorStrings, "label anchor",
-		TCL_EXACT, (int*)&style->labelAnchor);
+	TtkGetLabelAnchorFromObj(NULL, objPtr, &style->labelAnchor);
+    }
+    if ((objPtr = Ttk_QueryOption(layout,"-labelmargins", 0)) != NULL) {
+	Ttk_GetBorderFromObj(NULL, objPtr, &style->labelMargins);
     }
     if ((objPtr = Ttk_QueryOption(layout,"-labeloutside", 0)) != NULL) {
 	Tcl_GetBooleanFromObj(NULL, objPtr, &style->labelOutside);
     }
-    if ((objPtr = Ttk_QueryOption(layout,"-labelspace", 0)) != NULL) {
-	Tk_GetPixelsFromObj(NULL, lf->core.tkwin, objPtr, &style->labelSpace);
-    }
-
-    if ((objPtr = Ttk_QueryOption(layout,"-labelinset", 0)) != NULL) {
-	Tk_GetPixelsFromObj(NULL, lf->core.tkwin, objPtr, &style->labelInset);
-    }
 
     return;
 }
 
-/*
- * LabelframeSlaveEventHandler --
- * 	Tracks when slave is destroyed, notifies parent.
- */
-static void LabelframeGeometryLostSlaveProc(ClientData, Tk_Window);/*forward*/
-
-static void
-LabelframeSlaveEventHandler(ClientData clientData, XEvent *eventPtr)
-{
-    Labelframe *lframe = clientData;
-    if (eventPtr->type == DestroyNotify) {
-	LabelframeGeometryLostSlaveProc(clientData,lframe->label.labelWidget);
-    }
-}
-
-/*
- * Labelframe geometry manager procedures:
- * ClientData is the labelframe widget.
- */
-static void
-LabelframeGeometryRequestProc(ClientData clientData, Tk_Window slave)
-{
-    Labelframe *lframePtr = clientData;
-    TtkResizeWidget(&lframePtr->core);
-}
-
-static void
-LabelframeGeometryLostSlaveProc(ClientData clientData, Tk_Window slave)
-{
-    Labelframe *lframePtr = clientData;
-
-    Tcl_DecrRefCount(lframePtr->label.labelWidgetObj);
-    lframePtr->label.labelWidgetObj = 0;
-    lframePtr->label.labelWidget = 0;
-
-    /* See also: LabelframeUnregisterSlave */
-    Tk_DeleteEventHandler(
-	slave, StructureNotifyMask, LabelframeSlaveEventHandler, clientData);
-    Tk_UnmaintainGeometry(slave, lframePtr->core.tkwin);
-
-    TtkResizeWidget(&lframePtr->core);
-}
-
-static Tk_GeomMgr LabelframeGeometryManager = {
-    "labelframe",
-    LabelframeGeometryRequestProc,
-    LabelframeGeometryLostSlaveProc
-};
-
-static void
-LabelframeRegisterSlave(Labelframe *lframePtr, Tk_Window slave)
-{
-    Tk_CreateEventHandler(
-	slave, StructureNotifyMask, LabelframeSlaveEventHandler, lframePtr);
-    Tk_ManageGeometry(slave, &LabelframeGeometryManager, lframePtr);
-    lframePtr->label.slaveParcel = Ttk_MakeBox(-1,-1,-1,-1);
-    /* Tk_MaintainGeometry() called later */
-}
-
-static void
-LabelframeUnregisterSlave(Labelframe *lframePtr, Tk_Window slave)
-{
-    Tk_ManageGeometry(slave, NULL, 0);
-
-    /* See also: LabelframeGeometryLostSlaveProc */
-    Tk_DeleteEventHandler(
-	slave, StructureNotifyMask, LabelframeSlaveEventHandler, lframePtr);
-    Tk_UnmaintainGeometry(slave, lframePtr->core.tkwin);
-
-    Tk_UnmapWindow(slave);
-}
-
-/* LabelframePlaceSlave --
- * 	Sets the position and size of the labelwidget.  
- */
-static void
-LabelframePlaceSlave(Labelframe *lframePtr, Ttk_Box parcel)
-{
-    Ttk_Box current = lframePtr->label.slaveParcel;
-
-    if (   current.x == parcel.x
-    	&& current.y == parcel.y
-    	&& current.width == parcel.width
-    	&& current.height == parcel.height
-    ) {
-    	/* Already in the right place - do nothing */
-	return;
-    }
-
-    Tk_MaintainGeometry(
-	lframePtr->label.labelWidget, lframePtr->core.tkwin,
-	parcel.x,parcel.y, parcel.width,parcel.height);
-
-    lframePtr->label.slaveParcel = parcel;
-}
-
 /* LabelframeLabelSize --
  * 	Extract the requested width and height of the labelframe's label:
  * 	taken from the label widget if specified, otherwise the text label.
  */
-
 static void 
 LabelframeLabelSize(Labelframe *lframePtr, int *widthPtr, int *heightPtr)
 {
     WidgetCore *corePtr = &lframePtr->core;
     Tk_Window labelWidget = lframePtr->label.labelWidget;
+    Ttk_LayoutNode *textNode = Ttk_LayoutFindNode(corePtr->layout, "text");
 
     if (labelWidget) {
 	*widthPtr = Tk_ReqWidth(labelWidget);
 	*heightPtr = Tk_ReqHeight(labelWidget);
+    } else if (textNode) { 
+	Ttk_LayoutNodeReqSize(corePtr->layout, textNode, widthPtr, heightPtr);
     } else {
-	Ttk_LayoutNode *textNode = Ttk_LayoutFindNode(corePtr->layout, "text");
-	if (textNode) { 
-	    Ttk_LayoutNodeReqSize(
-		corePtr->layout, textNode, widthPtr, heightPtr);
-	} else {
-	    *widthPtr = *heightPtr = 0;
-	}
+	*widthPtr = *heightPtr = 0;
     }
 }
 
@@ -454,12 +355,8 @@ static int LabelframeSize(void *recordPtr, int *widthPtr, int *heightPtr)
     /* Adjust margins based on label size and position:
      */
     LabelframeLabelSize(lframePtr, &labelWidth, &labelHeight);
-
-    /* Adjust for -labeloutside / -labelspace:
-    */
-    if (style.labelOutside) {
-	labelHeight += style.labelSpace;
-    }
+    labelWidth += Ttk_PaddingWidth(style.labelMargins);
+    labelHeight += Ttk_PaddingHeight(style.labelMargins);
 
     switch (LabelAnchorSide(style.labelAnchor)) {
 	case TTK_SIDE_LEFT:	margins.left   += labelWidth;	break;
@@ -473,12 +370,18 @@ static int LabelframeSize(void *recordPtr, int *widthPtr, int *heightPtr)
     /* Request minimum size based on border width and label size:
      */
     Tk_SetMinimumRequestSize(corePtr->tkwin, 
-	    labelWidth + 2*style.borderWidth + LABELFRAME_SPACE,
-	    labelHeight + 2*style.borderWidth + LABELFRAME_SPACE);
+	    labelWidth + 2*style.borderWidth,
+	    labelHeight + 2*style.borderWidth);
 
     return 0;
 }
 
+/*
+ * LabelframeDoLayout --
+ * 	Labelframe layout hook.  
+ *
+ * Side effects: Computes labelParcel.
+ */
 
 static void LabelframeDoLayout(void *recordPtr)
 {
@@ -502,14 +405,13 @@ static void LabelframeDoLayout(void *recordPtr)
      */
     LabelframeStyleOptions(lframePtr, &style);
     LabelframeLabelSize(lframePtr, &lw, &lh);
-    labelParcel = Ttk_PlaceBox(
-	    &borderParcel, lw, lh,
-	    LabelAnchorSide(style.labelAnchor),
-	    LabelAnchorSticky(style.labelAnchor));
+    lw += Ttk_PaddingWidth(style.labelMargins);
+    lh += Ttk_PaddingHeight(style.labelMargins);
+
+    labelParcel = Ttk_PadBox(
+	Ttk_PositionBox(&borderParcel, lw, lh, style.labelAnchor),
+	style.labelMargins);
 
-    /*
-     * Place label on top of border, unless -labeloutside is set:
-     */
     if (!style.labelOutside) {
 	/* Move border edge so it's over label:
 	*/
@@ -519,52 +421,133 @@ static void LabelframeDoLayout(void *recordPtr)
 	    case TTK_SIDE_TOP:  	borderParcel.y -= lh / 2;
 	    case TTK_SIDE_BOTTOM:	borderParcel.height += lh / 2;	break;
 	}
-    } else {
-	/* Add extra space between label and border:
-	 */
-	borderParcel.y += style.labelSpace;
-	borderParcel.height -= style.labelSpace;
-    }
-
-    /* Move label away from corner:
-    */
-    switch (style.labelAnchor) {
-	default:		break; /*no-op*/
-	case LABELANCHOR_NW:
-	case LABELANCHOR_SW:	labelParcel.x += style.labelInset; break;
-	case LABELANCHOR_NE:
-	case LABELANCHOR_SE:	labelParcel.x -= style.labelInset; break;
-	case LABELANCHOR_EN:
-	case LABELANCHOR_WN:	labelParcel.y += style.labelInset; break;
-	case LABELANCHOR_ES:
-	case LABELANCHOR_WS:	labelParcel.y -= style.labelInset; break;
     }
 
     /* 
      * Place border and label:
      */
-    if (borderNode)
-	Ttk_LayoutNodeSetParcel(borderNode, borderParcel);
-    if (lframePtr->label.labelWidget)
-	LabelframePlaceSlave(lframePtr, labelParcel);
-    if (textNode)
-	Ttk_LayoutNodeSetParcel(textNode, labelParcel);
+    if (borderNode) {
+	Ttk_PlaceLayoutNode(corePtr->layout, borderNode, borderParcel);
+    }
+    if (textNode) {
+	Ttk_PlaceLayoutNode(corePtr->layout, textNode, labelParcel);
+    }
+    /* labelWidget placed in LabelframePlaceSlaves GM hook */
+    lframePtr->label.labelParcel = labelParcel;
 }
 
-static void LabelframeCleanup(void *recordPtr)
+/* LabelframePlaceSlaves --
+ * 	Sets the position and size of the labelwidget.  
+ */
+static void LabelframePlaceSlaves(void *recordPtr)
 {
-    Labelframe *lframePtr = recordPtr;
+    Labelframe *lframe = recordPtr;
 
-    if (lframePtr->label.labelWidget) {
-	LabelframeUnregisterSlave(lframePtr,lframePtr->label.labelWidget);
-	lframePtr->label.labelWidget = 0;
+    if (Ttk_NumberSlaves(lframe->label.mgr) == 1) {
+	Ttk_Box b;
+	LabelframeDoLayout(recordPtr);
+	b = lframe->label.labelParcel;
+	/* ASSERT: slave #0 is lframe->label.labelWidget */
+	Ttk_PlaceSlave(lframe->label.mgr, 0, b.x,b.y,b.width,b.height);
     }
 }
 
+/* Labelframe geometry manager:
+ */
+static void LabelAdded(Ttk_Manager *mgr, int slaveIndex) { /* no-op */ }
+static void LabelRemoved(Ttk_Manager *mgr, int slaveIndex) { /* no-op */ }
+static int LabelConfigured(
+    Tcl_Interp *interp, Ttk_Manager *mgr, Ttk_Slave *slave, unsigned mask)
+    { return TCL_OK; }
+
+/* LabelframeLostSlave --
+ * 	Called when the labelWidget slave is involuntarily lost;
+ * 	unset the -labelwidget option.
+ * Notes:
+ * 	Do this here instead of in the SlaveRemoved hook, 
+ * 	since the latter is also called when the widget voluntarily
+ * 	forgets the slave.  The latter happens in the ConfigureProc
+ * 	at a time when the widget is in an inconsistent state.
+ */
+static void LabelframeLostSlave(ClientData clientData, Tk_Window slaveWindow)
+{
+    Ttk_Slave *slave = clientData;
+    Labelframe *lframePtr = slave->manager->managerData;
+
+    Tcl_DecrRefCount(lframePtr->label.labelWidgetObj);
+    lframePtr->label.labelWidgetObj = 0;
+    lframePtr->label.labelWidget = 0;
+    Ttk_LostSlaveProc(clientData, slaveWindow);
+}
+
+static Tk_OptionSpec LabelOptionSpecs[] = {
+    {TK_OPTION_END, 0,0,0, NULL, -1,-1, 0, 0,0}
+};
+
+static Ttk_ManagerSpec LabelframeManagerSpec = 
+{
+    { "labelframe", Ttk_GeometryRequestProc, LabelframeLostSlave },
+    LabelOptionSpecs, 0,
+
+    LabelframeSize,
+    LabelframePlaceSlaves,
+
+    LabelAdded,
+    LabelRemoved,
+    LabelConfigured
+};
+
+/* LabelframeInitialize --
+ * 	Initialization hook.
+ */
+static int LabelframeInitialize(Tcl_Interp *interp, void *recordPtr)
+{
+    Labelframe *lframe = recordPtr;
+
+    lframe->label.mgr = Ttk_CreateManager(
+	&LabelframeManagerSpec, lframe, lframe->core.tkwin);
+    lframe->label.labelWidget = 0;
+    lframe->label.labelParcel = Ttk_MakeBox(-1,-1,-1,-1);
+
+    return TCL_OK;
+}
+
+/* LabelframeCleanup --
+ * 	Cleanup hook.
+ */
+static void LabelframeCleanup(void *recordPtr)
+{
+    Labelframe *lframe = recordPtr;
+    Ttk_DeleteManager(lframe->label.mgr);
+}
+
+/* RaiseLabelWidget --
+ * 	Raise the -labelwidget to ensure that the labelframe doesn't 
+ * 	obscure it (if it's not a direct child), or bring it to
+ * 	the top of the stacking order (if it is).
+ */
+static void RaiseLabelWidget(Labelframe *lframe)
+{
+    Tk_Window parent = Tk_Parent(lframe->label.labelWidget);
+    Tk_Window sibling = NULL;
+    Tk_Window w = lframe->core.tkwin;
+
+    while (w && w != parent) {
+	sibling = w;
+	w = Tk_Parent(w);
+    }
+
+    Tk_RestackWindow(lframe->label.labelWidget, Above, sibling);
+}
+
+/* LabelframeConfigure --
+ * 	Configuration hook.
+ */
 static int LabelframeConfigure(Tcl_Interp *interp,void *recordPtr,int mask)
 {
     Labelframe *lframePtr = recordPtr;
-    Tk_Window labelWidget=NULL, oldLabelWidget=lframePtr->label.labelWidget;
+    Tk_Window labelWidget = NULL;
+    Ttk_PositionSpec unused;
 
     /* Validate -labelwidget option:
      */
@@ -576,12 +559,18 @@ static int LabelframeConfigure(Tcl_Interp *interp,void *recordPtr,int mask)
 	    if (!labelWidget) {
 		return TCL_ERROR;
 	    }
-	    if (!Maintainable(interp, labelWidget, lframePtr->core.tkwin)) {
+	    if (!Ttk_Maintainable(interp, labelWidget, lframePtr->core.tkwin)) {
 		return TCL_ERROR;
 	    }
 	}
     }
 
+    if (TtkGetLabelAnchorFromObj(
+    	interp, lframePtr->label.labelAnchorObj, &unused) != TCL_OK)
+    {
+	return TCL_ERROR;
+    }
+
     /* Base class configuration:
      */
     if (FrameConfigure(interp, recordPtr, mask) != TCL_OK) {
@@ -590,14 +579,22 @@ static int LabelframeConfigure(Tcl_Interp *interp,void *recordPtr,int mask)
 
     /* Update -labelwidget changes, if any:
      */
-    if (labelWidget != oldLabelWidget) {
-	if (oldLabelWidget) {
-	    LabelframeUnregisterSlave(lframePtr, oldLabelWidget);
-	}
-	if (labelWidget) {
-	    LabelframeRegisterSlave(lframePtr, labelWidget);
+    if (mask & LABELWIDGET_CHANGED) {
+	if (Ttk_NumberSlaves(lframePtr->label.mgr) == 1) {
+	    Ttk_ForgetSlave(lframePtr->label.mgr, 0);
 	}
+
 	lframePtr->label.labelWidget = labelWidget;
+
+	if (labelWidget) {
+	    Ttk_AddSlave(interp, lframePtr->label.mgr, labelWidget, 0,  0,0);
+	    RaiseLabelWidget(lframePtr);
+	}
+    }
+
+    if (mask & GEOMETRY_CHANGED) {
+	Ttk_ManagerSizeChanged(lframePtr->label.mgr);
+	Ttk_ManagerLayoutChanged(lframePtr->label.mgr);
     }
 
     return TCL_OK;
@@ -608,9 +605,9 @@ WidgetSpec LabelframeWidgetSpec =
 {
     "TLabelframe",		/* className */
     sizeof(Labelframe),		/* recordSize */
-    LabelframeCompatOptionSpecs,/* optionSpecs */
+    LabelframeOptionSpecs, 	/* optionSpecs */
     FrameCommands,		/* subcommands */
-    NullInitialize,		/* initializeProc */
+    LabelframeInitialize,	/* initializeProc */
     LabelframeCleanup,		/* cleanupProc */
     LabelframeConfigure,	/* configureProc */
     NullPostConfigure,  	/* postConfigureProc */
