@@ -5,7 +5,7 @@
  *
  * Copyright (c) 2003 Joe English.  Freely redistributable.
  *
- * layout.c,v 1.65 2005/09/04 18:05:43 jenglish Exp
+ * layout.c,v 1.68 2006/04/16 19:28:55 jenglish Exp
  */
 
 #include <string.h>
@@ -193,12 +193,10 @@ Ttk_Box Ttk_StickBox(Ttk_Box parcel, int width, int height, unsigned sticky)
 }
 
 /*
- * Ttk_AnchorBox --
- * 	Place a box of size w * h in the specified parcel,
- * 	according to the specified anchor.
+ * AnchorToSticky --
+ * 	Convert a Tk_Anchor enum to a TTK_STICKY bitmask.
  */
-
-static unsigned AnchorToSticky(int anchor)
+static Ttk_Sticky AnchorToSticky(Tk_Anchor anchor)
 {
     switch (anchor)
     {
@@ -215,7 +213,12 @@ static unsigned AnchorToSticky(int anchor)
     }
 }
 
-Ttk_Box Ttk_AnchorBox(Ttk_Box parcel, int width, int height, int anchor)
+/*
+ * Ttk_AnchorBox --
+ * 	Place a box of size w * h in the specified parcel,
+ * 	according to the specified anchor.
+ */
+Ttk_Box Ttk_AnchorBox(Ttk_Box parcel, int width, int height, Tk_Anchor anchor)
 {
     return Ttk_StickBox(parcel, width, height, AnchorToSticky(anchor));
 }
@@ -231,6 +234,25 @@ Ttk_Box Ttk_PlaceBox(
 	    Ttk_PackBox(cavity, width, height, side), width, height, sticky);
 }
 
+/*
+ * Ttk_PositionBox --
+ * 	Pack and stick a box according to PositionSpec flags.
+ */
+TTKAPI Ttk_Box
+Ttk_PositionBox(Ttk_Box *cavity, int width, int height, Ttk_PositionSpec flags)
+{
+    Ttk_Box parcel;
+
+	 if (flags & TTK_EXPAND)	parcel = *cavity;
+    else if (flags & TTK_PACK_TOP)	parcel = packTop(cavity, height);
+    else if (flags & TTK_PACK_LEFT)	parcel = packLeft(cavity, width);
+    else if (flags & TTK_PACK_BOTTOM)	parcel = packBottom(cavity, height);
+    else if (flags & TTK_PACK_RIGHT)	parcel = packRight(cavity, width);
+    else				parcel = *cavity;
+
+    return Ttk_StickBox(parcel, width, height, flags);
+}
+
 /*
  * TTKInitPadding --
  * 	Common factor of Ttk_GetPaddingFromObj and Ttk_GetBorderFromObj.
@@ -720,7 +742,6 @@ Tcl_Obj *Ttk_UnparseLayoutTemplate(Ttk_TemplateNode *node)
 
 	/* Back-compute -side.  <<NOTE-PACKSIDE>>
 	 * @@@ NOTES: Ick.
-	 * @@@ CHECK: ? '-expand true' equivalent to omitted -side?
 	 */
 	if (flags & TTK_EXPAND) {
 	    APPENDSTR("-expand");
@@ -739,8 +760,8 @@ Tcl_Obj *Ttk_UnparseLayoutTemplate(Ttk_TemplateNode *node)
 	    }
 	}
 
-	/* Default -sticky is "nsew", so always include this
-	 * even if no sticky bits are set.
+	/* In Ttk_ParseLayoutTemplate, default -sticky is "nsew",
+	 * so always include this even if no sticky bits are set.
 	 */
 	APPENDSTR("-sticky");
 	APPENDOBJ(Ttk_NewStickyObj(flags & _TTK_MASK_STICK));
@@ -826,22 +847,33 @@ Ttk_Layout Ttk_CreateLayout(
     return TTKNewLayout(style, recordPtr, optionTable, tkwin, bgnode);
 }
 
-/* Ttk_CreateSubLayout --
+/* Ttk_CreateSublayout --
  * 	Creates a new sublayout.
+ *
+ * 	Sublayouts are used to draw subparts of a compound widget.
+ *	They use the same Tk_Window, but a different option table 
+ *	and data record.
  */
 Ttk_Layout
-Ttk_CreateSubLayout(
+Ttk_CreateSublayout(
     Tcl_Interp *interp,
     Ttk_Theme themePtr,
-    const char *styleName,
-    void *recordPtr,
-    Tk_OptionTable optionTable,
-    Tk_Window tkwin)
+    Ttk_Layout parentLayout,
+    const char *baseName,
+    Tk_OptionTable optionTable)
 {
-    Ttk_Style style = Ttk_GetStyle(themePtr, styleName);
-    Ttk_LayoutTemplate layoutTemplate =
-	Ttk_FindLayoutTemplate(themePtr,styleName);
-    Ttk_LayoutNode *root;
+    Tcl_DString buf;
+    const char *styleName;
+    Ttk_Style style;
+    Ttk_LayoutTemplate layoutTemplate;
+
+    Tcl_DStringInit(&buf);
+    Tcl_DStringAppend(&buf, Ttk_StyleName(parentLayout->style), -1);
+    Tcl_DStringAppend(&buf, baseName, -1);
+    styleName = Tcl_DStringValue(&buf);
+
+    style = Ttk_GetStyle(themePtr, styleName);
+    layoutTemplate = Ttk_FindLayoutTemplate(themePtr, styleName);
 
     if (!layoutTemplate) {
 	Tcl_ResetResult(interp);
@@ -849,12 +881,15 @@ Ttk_CreateSubLayout(
 	return 0;
     }
 
-    root = Ttk_InstantiateLayout(themePtr, layoutTemplate);
-    return TTKNewLayout(style, recordPtr, optionTable, tkwin, root);
+    Tcl_DStringFree(&buf);
+
+    return TTKNewLayout(
+	    style, 0, optionTable, parentLayout->tkwin, 
+	    Ttk_InstantiateLayout(themePtr, layoutTemplate));
 }
 
 /* Ttk_RebindSublayout --
- * 	Rebind sublayout to new data source.
+ * 	Bind sublayout to new data source.
  */
 void Ttk_RebindSublayout(Ttk_Layout layout, void *recordPtr)
 {
@@ -975,46 +1010,25 @@ void Ttk_LayoutNodeReqSize(	/* @@@ Rename this */
 
 /* Ttk_PlaceNodeList --
  *	Compute parcel for each node in a layout tree
- *	according to packing specification and overall size.
+ *	according to position specification and overall size.
  */
 static void Ttk_PlaceNodeList(
     Ttk_Layout layout, Ttk_LayoutNode *node, Ttk_State state, Ttk_Box cavity)
 {
     for (; node; node = node->next)
     {
-	int elementWidth, elementHeight;
+	int width, height;
 	Ttk_Padding padding;
-	Ttk_Box parcel;
 
-	/* @@@ Cache this instead? */
-	Ttk_NodeSize(layout, node, state,
-	    &elementWidth, &elementHeight, &padding);
-
-	/*
-	 * Compute parcel:
+	/* Compute node size: (@@@ cache this instead?)
 	 */
-	if (node->flags & TTK_EXPAND) {
-	    parcel = cavity;
-	} else if (node->flags & TTK_PACK_TOP) {
-	    parcel = packTop(&cavity, elementHeight);
-	} else if (node->flags & TTK_PACK_LEFT) {
-	    parcel = packLeft(&cavity, elementWidth);
-	} else if (node->flags & TTK_PACK_BOTTOM) {
-	    parcel = packBottom(&cavity, elementHeight);
-	} else if (node->flags & TTK_PACK_RIGHT) {
-	    parcel = packRight(&cavity, elementWidth);
-	} else {
-	    parcel = cavity;
-	}
+	Ttk_NodeSize(layout, node, state, &width, &height, &padding);
 
-	/*
-	 * Place element within parcel:
+	/* Compute parcel:
 	 */
-	node->parcel = Ttk_StickBox(
-		parcel, elementWidth, elementHeight, node->flags);
+	node->parcel = Ttk_PositionBox(&cavity, width, height, node->flags);
 
-	/*
-	 * Place child nodes:
+	/* Place child nodes:
 	 */
 	if (node->child) {
 	    Ttk_Box childBox = Ttk_PadBox(node->parcel, padding);
