@@ -1,4 +1,4 @@
-/* notebook.c,v 1.74 2005/10/08 14:02:47 jenglish Exp
+/* notebook.c,v 1.85 2006/05/06 17:59:28 jenglish Exp
  * Copyright (c) 2004, Joe English
  *
  * NOTE-ACTIVE: activeTabIndex is not always correct (it's  
@@ -132,27 +132,34 @@ static Tk_OptionSpec NotebookOptionSpecs[] =
 };
 
 /* Notebook style options:
+ * <<NOTE-POSITIONSPEC>> -- this isn't fully supported yet, just enough
+ * to make (Jaguar-style) Aqua tab views work.
  */
 typedef struct 
 {
-    Ttk_Padding 	padding;	/* External padding */
-    Ttk_Padding 	expandTab;	/* Amount to expand selected tab */
+    Ttk_PositionSpec	tabPosition;	/* Where to place tabs */
+    Ttk_Padding 	tabMargins;	/* Margins around tab row */
     int 		minTabWidth;	/* Minimum tab width */
+    Ttk_Padding 	padding;	/* External padding */
 } NotebookStyle;
 
 static void NotebookStyleOptions(Notebook *nb, NotebookStyle *nbstyle)
 {
     Tcl_Obj *objPtr;
 
-    nbstyle->expandTab = Ttk_UniformPadding(0);
-    if ((objPtr = Ttk_QueryOption(nb->core.layout, "-expandtab", 0)) != 0) {
-	Ttk_GetPaddingFromObj(NULL,nb->core.tkwin,objPtr,&nbstyle->expandTab);
+    nbstyle->tabPosition = TTK_PACK_TOP | TTK_STICK_W;
+    if ((objPtr = Ttk_QueryOption(nb->core.layout, "-tabposition", 0)) != 0) {
+	TtkGetLabelAnchorFromObj(NULL, objPtr, &nbstyle->tabPosition);
+    }
+
+    nbstyle->tabMargins = Ttk_UniformPadding(0);
+    if ((objPtr = Ttk_QueryOption(nb->core.layout, "-tabmargins", 0)) != 0) {
+	Ttk_GetBorderFromObj(NULL, objPtr, &nbstyle->tabMargins);
     }
 
     nbstyle->padding = Ttk_UniformPadding(0);
-    if (nb->notebook.paddingObj) {
-	Ttk_GetPaddingFromObj(
-	    NULL,nb->core.tkwin,nb->notebook.paddingObj,&nbstyle->padding);
+    if ((objPtr = Ttk_QueryOption(nb->core.layout, "-padding", 0)) != 0) {
+	Ttk_GetPaddingFromObj(NULL,nb->core.tkwin,objPtr,&nbstyle->padding);
     }
 
     nbstyle->minTabWidth = DEFAULT_MIN_TAB_WIDTH;
@@ -206,7 +213,7 @@ static void ActivateTab(Notebook *nb, int index)
 static Ttk_State TabState(Notebook *nb, int index)
 {
     Ttk_State state = nb->core.state;
-    Tab *tab = nb->notebook.mgr->slaves[index]->slaveData;
+    Tab *tab = Ttk_SlaveData(nb->notebook.mgr, index);
 
     if (index == nb->notebook.currentIndex) {
 	state |= TTK_STATE_SELECTED;
@@ -313,7 +320,8 @@ static int NotebookSize(void *clientData, int *widthPtr, int *heightPtr)
     /* Tab row:
      */
     TabrowSize(nb, &tabrowWidth, &tabrowHeight);
-    tabrowHeight += nbstyle.expandTab.top;
+    tabrowHeight += Ttk_PaddingHeight(nbstyle.tabMargins);
+    tabrowWidth += Ttk_PaddingWidth(nbstyle.tabMargins);
 
     /* Account for exterior and interior padding:
      */
@@ -357,7 +365,7 @@ static void SqueezeTabs(
     int i;
 
     for (i = 0; i < nTabs; ++i) {
-	Tab *tab = nb->notebook.mgr->slaves[i]->slaveData;
+	Tab *tab = Ttk_SlaveData(nb->notebook.mgr,i);
 	int shrink = (shrinkage/nTabs) + (i < (shrinkage%nTabs)) + extra;
 	int shrinkability = MAX(0, tab->width - minTabWidth);
 	int delta = MIN(shrinkability, shrink);
@@ -366,6 +374,36 @@ static void SqueezeTabs(
     }
 }
 
+/* PlaceTabs --
+ * 	Compute all tab parcels.
+ */
+static void PlaceTabs(Notebook *nb, Ttk_Box tabrowBox)
+{
+    Ttk_Layout tabLayout = nb->notebook.tabLayout;
+    int nTabs = Ttk_NumberSlaves(nb->notebook.mgr);
+    int i;
+
+    for (i = 0; i < nTabs; ++i) {
+	Tab *tab = Ttk_SlaveData(nb->notebook.mgr, i);
+	Ttk_State tabState = TabState(nb, i);
+
+	if (tab->state != TAB_STATE_HIDDEN) {
+	    Ttk_Padding expand = Ttk_UniformPadding(0);
+	    Tcl_Obj *expandObj = Ttk_QueryOption(tabLayout,"-expand",tabState);
+
+	    if (expandObj) {
+		Ttk_GetBorderFromObj(NULL, expandObj, &expand);
+	    }
+
+	    tab->parcel = 
+		Ttk_ExpandBox(
+		    Ttk_PlaceBox(&tabrowBox,
+			tab->width, tab->height, TTK_SIDE_LEFT, TTK_STICK_S),
+		    expand);
+	}
+    }
+}
+
 /* NotebookDoLayout --
  *	Computes notebook layout and places tabs.
  *
@@ -379,10 +417,8 @@ static void NotebookDoLayout(void *recordPtr)
     Ttk_Box cavity = Ttk_WinBox(nbwin);
     int tabrowWidth = 0, tabrowHeight = 0;
     Ttk_LayoutNode *clientNode = Ttk_LayoutFindNode(nb->core.layout, "client");
-    int nTabs = Ttk_NumberSlaves(nb->notebook.mgr);
     Ttk_Box tabrowBox;
     NotebookStyle nbstyle;
-    int i;
 
     NotebookStyleOptions(nb, &nbstyle);
 
@@ -395,29 +431,19 @@ static void NotebookDoLayout(void *recordPtr)
     Ttk_PlaceLayout(nb->core.layout, nb->core.state, Ttk_WinBox(nbwin));
 
     /* Place tabs:
-     * Allow extra space on the top, left, and right (only)
-     * to account for expandTab.
      */
     TabrowSize(nb, &tabrowWidth, &tabrowHeight);
-    tabrowBox = Ttk_PackBox(&cavity,
-	tabrowWidth, tabrowHeight+nbstyle.expandTab.top, TTK_SIDE_TOP);
-    tabrowBox.x += nbstyle.expandTab.left;
-    tabrowBox.width -= Ttk_PaddingWidth(nbstyle.expandTab);
+    tabrowBox = Ttk_PadBox(
+		    Ttk_PositionBox(&cavity,
+			tabrowWidth + Ttk_PaddingWidth(nbstyle.tabMargins),
+			tabrowHeight + Ttk_PaddingHeight(nbstyle.tabMargins),
+			nbstyle.tabPosition),
+		    nbstyle.tabMargins);
 
     if (tabrowWidth > tabrowBox.width) {
 	SqueezeTabs(nb, tabrowWidth, tabrowBox.width, nbstyle.minTabWidth);
     }
-
-    for (i = 0; i < nTabs; ++i) {
-	Tab *tab = Ttk_SlaveData(nb->notebook.mgr, i);
-	if (tab->state != TAB_STATE_HIDDEN) {
-	    tab->parcel = Ttk_PlaceBox(&tabrowBox,
-		tab->width, tab->height, TTK_SIDE_LEFT, TTK_STICK_S);
-	    if (TabState(nb, i) & TTK_STATE_SELECTED) {
-	        tab->parcel = Ttk_ExpandBox(tab->parcel, nbstyle.expandTab);
-	    }
-	}
-    }
+    PlaceTabs(nb, tabrowBox);
 
     /* Layout for client area frame:
      */
@@ -475,7 +501,7 @@ static void SelectTab(Notebook *nb, int index)
 	return;
     }
 
-    if (tab->state == TAB_STATE_DISABLED) {
+    if (TabState(nb, index) & TTK_STATE_DISABLED) {
 	return;
     }
 
@@ -505,31 +531,51 @@ static void SelectTab(Notebook *nb, int index)
 static int NextTab(Notebook *nb, int index)
 {
     int nTabs = Ttk_NumberSlaves(nb->notebook.mgr);
-    if (nTabs != 0) {
-	int nextIndex = index;
-	Tab *tab;
-	do {
-	    nextIndex = (nextIndex + 1) % nTabs;
-	    tab = Ttk_SlaveData(nb->notebook.mgr, nextIndex);
-	    if (tab->state == TAB_STATE_NORMAL) {
-		return nextIndex;
-	    }
-	} while (nextIndex != index);
+    int nextIndex;
+   
+    /* Scan forward for following usable tab:
+     */
+    for (nextIndex = index + 1; nextIndex < nTabs; ++nextIndex) {
+	Tab *tab = Ttk_SlaveData(nb->notebook.mgr, nextIndex);
+	if (tab->state == TAB_STATE_NORMAL) {
+	    return nextIndex;
+	}
     }
 
+    /* Not found -- scan backwards.
+     */
+    for (nextIndex = index - 1; nextIndex >= 0; --nextIndex) {
+	Tab *tab = Ttk_SlaveData(nb->notebook.mgr, nextIndex);
+	if (tab->state == TAB_STATE_NORMAL) {
+	    return nextIndex;
+	}
+    }
+
+    /* Still nothing.  Give up.
+     */
     return -1;
 }
 
-/* SelectNextTab --
- * 	Select the next tab after the specified index that is
- * 	in the normal state.  If all other tabs are disabled or hidden,
- * 	does nothing.
+/* SelectNearestTab --
+ * 	Handles the case where the current tab is forgotten, hidden, 
+ * 	or destroyed. Select the next available tab; or, if there is none,
+ * 	leaves all tabs unselected.
  */
-static void SelectNextTab(Notebook *nb, int index) 
+static void SelectNearestTab(Notebook *nb) 
 {
-    int nextIndex = NextTab(nb, index);
-    if (nextIndex != -1) {
+    int nextIndex = NextTab(nb, nb->notebook.currentIndex);
+
+    if (nextIndex >= 0) {
 	SelectTab(nb, nextIndex);
+    } else {
+	/* No available tabs -- unmap current one.
+	 * ASSERT: this is safe to do even when the slave is being destroyed.
+	 */
+	if (nb->notebook.currentIndex >= 0) {
+	    Ttk_UnmapSlave(nb->notebook.mgr, nb->notebook.currentIndex);
+	    SendVirtualEvent(nb->core.tkwin, "NotebookTabChanged");
+	}
+	nb->notebook.currentIndex = -1;
     }
 }
 
@@ -538,14 +584,15 @@ static void SelectNextTab(Notebook *nb, int index)
 static void TabAdded(Ttk_Manager *mgr, int slaveIndex) { /* No-op */ }
 
 /* TabRemoved -- GM SlaveRemoved hook.
+ * 	Select the next tab if the current one is being removed.
+ * 	Adjust currentIndex to account for removed slave if needed.
  */
 static void TabRemoved(Ttk_Manager *mgr, int index)
 {
     Notebook *nb = mgr->managerData;
 
     if (index == nb->notebook.currentIndex) {
-	nb->notebook.currentIndex = -1;
-	SelectNextTab(nb, index);
+	SelectNearestTab(nb);
     }
 
     if (index < nb->notebook.currentIndex) {
@@ -678,7 +725,7 @@ static int GetTabIndex(
  * +++ Widget command routines.
  */
 
-/* $nb add window [ options ... ]
+/* $nb add window ?options ... ?
  */
 static int NotebookAddCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
@@ -717,6 +764,77 @@ static int NotebookAddCommand(
     return TCL_OK;
 }
 
+/* $nb insert $index $tab ?options...?
+ * 	Insert new tab, or move existing one.
+ */
+static int NotebookInsertCommand(
+    Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
+{
+    Notebook *nb = recordPtr;
+    int current = nb->notebook.currentIndex;
+    int srcIndex, destIndex;
+    int status = TCL_OK;
+
+    if (objc < 4) {
+	Tcl_WrongNumArgs(interp, 2,objv, "index slave ?options...?");
+	return TCL_ERROR;
+    }
+
+    if (!strcmp(Tcl_GetString(objv[2]), "end")) {
+	destIndex = Ttk_NumberSlaves(nb->notebook.mgr);
+    } else if (!Ttk_GetSlaveFromObj(
+		interp, nb->notebook.mgr, objv[2], &destIndex)) {
+	return TCL_ERROR;
+    }
+
+    if (!Ttk_GetSlaveFromObj(interp, nb->notebook.mgr, objv[3], &srcIndex)) {
+	/* Try adding new slave:
+	 */
+	Tk_Window slaveWindow = 
+	    Tk_NameToWindow(interp,Tcl_GetString(objv[3]),nb->core.tkwin);
+	if (!slaveWindow) {
+	    return TCL_ERROR;
+	}
+
+	if (Ttk_AddSlave(interp, nb->notebook.mgr, slaveWindow,
+		destIndex, objc - 4, objv + 4) != TCL_OK) 
+	{
+	    return TCL_ERROR;
+	}
+	if (nb->notebook.currentIndex <= destIndex) {
+	    ++nb->notebook.currentIndex;
+	}
+	return TCL_OK;
+    }
+
+    /* else - move existing slave: */
+
+    if (destIndex >= nb->notebook.mgr->nSlaves) {
+	destIndex  = nb->notebook.mgr->nSlaves - 1;
+    }
+    Ttk_ReorderSlave(nb->notebook.mgr, srcIndex, destIndex);
+
+    /* Adjust internal indexes:
+     */
+    nb->notebook.activeIndex = -1;
+    if (current == srcIndex) {
+	nb->notebook.currentIndex = destIndex;
+    } else if (destIndex <= current && current < srcIndex) {
+	++nb->notebook.currentIndex;
+    } else if (srcIndex < current && current <= destIndex) {
+	--nb->notebook.currentIndex;
+    }
+
+    if (objc > 4) {
+	status = Ttk_ConfigureSlave(interp, nb->notebook.mgr,
+	     nb->notebook.mgr->slaves[destIndex], objc-4,objv+4);
+    }
+
+    TtkRedisplayWidget(&nb->core);
+
+    return status;
+}
+
 /* $nb forget $item --
  * 	Removes the selected tab.
  */
@@ -724,19 +842,20 @@ static int NotebookForgetCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     Notebook *nb = recordPtr;
-    int index, status;
+    int index;
 
     if (objc != 3) {
 	Tcl_WrongNumArgs(interp, 2, objv, "tab");
 	return TCL_ERROR;
     }
 
-    status = GetTabIndex(interp, nb, objv[2], &index);
-    if (status == TCL_OK) {
-	Ttk_ForgetSlave(nb->notebook.mgr, index);
+    if (GetTabIndex(interp, nb, objv[2], &index) != TCL_OK) {
+	return TCL_ERROR;
     }
 
-    return status;
+    Ttk_ForgetSlave(nb->notebook.mgr, index);
+
+    return TCL_OK;
 }
 
 /* $nb index $item --
@@ -771,26 +890,31 @@ static int NotebookIndexCommand(
     return status;
 }
 
-/* $nb select $item --
- * 	Selects the specified tab.
+/* $nb select ?$item? --
+ * 	Select the specified tab, or return the widget path of
+ * 	the currently-selected pane.
  */
 static int NotebookSelectCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     Notebook *nb = recordPtr;
-    int index, status;
 
-    if (objc != 3) {
-	Tcl_WrongNumArgs(interp, 2, objv, "tab");
-	return TCL_ERROR;
-    }
-
-    status = GetTabIndex(interp, nb, objv[2], &index);
-    if (status == TCL_OK) {
-	SelectTab(nb, index);
-    }
-
-    return status;
+    if (objc == 2) {
+	if (nb->notebook.currentIndex >= 0) {
+	    Tk_Window pane = Ttk_SlaveWindow(
+		nb->notebook.mgr, nb->notebook.currentIndex);
+	    Tcl_SetObjResult(interp, Tcl_NewStringObj(Tk_PathName(pane), -1));
+	}
+	return TCL_OK;
+    } else if (objc == 3) {
+	int index, status = GetTabIndex(interp, nb, objv[2], &index);
+	if (status == TCL_OK) {
+	    SelectTab(nb, index);
+	}
+	return status;
+    } /*else*/
+    Tcl_WrongNumArgs(interp, 2, objv, "?tab?");
+    return TCL_ERROR;
 }
 
 /* $nb tabs --
@@ -811,7 +935,7 @@ static int NotebookTabsCommand(
 
     result = Tcl_NewListObj(0, NULL);
     for (i = 0; i < mgr->nSlaves; ++i) {
-	const char *pathName = Tk_PathName(mgr->slaves[i]->slaveWindow);
+	const char *pathName = Tk_PathName(Ttk_SlaveWindow(mgr,i));
 	Tcl_ListObjAppendElement(interp, result, Tcl_NewStringObj(pathName,-1));
     }
     Tcl_SetObjResult(interp, result);
@@ -858,7 +982,7 @@ static int NotebookTabCommand(
      * select the next nondisabled, unhidden one:
      */
     if (index == nb->notebook.currentIndex && tab->state != TAB_STATE_NORMAL) {
-	SelectNextTab(nb, index);
+	SelectNearestTab(nb);
     }
 
     TtkResizeWidget(&nb->core);
@@ -874,6 +998,7 @@ static WidgetCommandSpec NotebookCommands[] =
     { "cget",		WidgetCgetCommand },
     { "forget",		NotebookForgetCommand },
     { "index",		NotebookIndexCommand },
+    { "insert",  	NotebookInsertCommand },
     { "instate",	WidgetInstateCommand },
     { "select",		NotebookSelectCommand },
     { "state",  	WidgetStateCommand },
@@ -944,12 +1069,18 @@ static int NotebookConfigure(Tcl_Interp *interp, void *clientData, int mask)
  * 	GetLayout widget hook.
  */
 static Ttk_Layout NotebookGetLayout(
-	Tcl_Interp *interp, Ttk_Theme theme, void *recordPtr)
+    Tcl_Interp *interp, Ttk_Theme theme, void *recordPtr)
 {
     Notebook *nb = recordPtr;
-    Ttk_Layout tabLayout = Ttk_CreateSubLayout(
-	interp, theme, "TNotebook.Tab", 0, 
-	nb->notebook.tabOptionTable, nb->core.tkwin);
+    Ttk_Layout notebookLayout = WidgetGetLayout(interp, theme, recordPtr);
+    Ttk_Layout tabLayout;
+
+    if (!notebookLayout) {
+	return NULL;
+    }
+
+    tabLayout = Ttk_CreateSublayout(
+	interp, theme, notebookLayout, ".Tab",	nb->notebook.tabOptionTable);
 
     if (tabLayout) {
 	if (nb->notebook.tabLayout) {
@@ -958,8 +1089,7 @@ static Ttk_Layout NotebookGetLayout(
 	nb->notebook.tabLayout = tabLayout;
     }
 
-    return Ttk_CreateLayout(interp, theme, "TNotebook", 
-	recordPtr, nb->core.optionTable, nb->core.tkwin);
+    return notebookLayout;
 }
 
 /* +++ Display routines.
