@@ -1,4 +1,4 @@
-/* paned.c,v 1.12 2005/10/19 20:30:49 jenglish Exp
+/* paned.c,v 1.13 2006/04/06 19:34:11 jenglish Exp
  *
  * Copyright (c) 2005, Joe English.  Freely redistributable.
  *
@@ -445,10 +445,11 @@ static Ttk_Layout PanedGetLayout(
     Tcl_Interp *interp, Ttk_Theme themePtr, void *recordPtr)
 {
     Paned *pw = recordPtr;
+    Ttk_Layout panedLayout = WidgetGetLayout(interp, themePtr, recordPtr);
     int horizontal = pw->paned.orient == TTK_ORIENT_HORIZONTAL;
-    const char *layoutName = horizontal ? "Vertical.Sash" : "Horizontal.Sash";
-    Ttk_Layout sashLayout = Ttk_CreateSubLayout(interp,themePtr,layoutName,
-	recordPtr, pw->core.optionTable, pw->core.tkwin);
+    const char *layoutName = horizontal ? ".Vertical.Sash" : ".Horizontal.Sash";
+    Ttk_Layout sashLayout = Ttk_CreateSublayout(interp, themePtr, panedLayout,
+	    layoutName, pw->core.optionTable);
 
     if (sashLayout) {
 	int sashWidth, sashHeight;
@@ -469,7 +470,7 @@ static Ttk_Layout PanedGetLayout(
 	Ttk_ManagerSizeChanged(pw->paned.mgr);
     }
 
-    return WidgetGetLayout(interp, themePtr, recordPtr);
+    return panedLayout;
 }
 
 static void PanedDisplay(void *recordPtr, Drawable d)
