@@ -1,4 +1,4 @@
-/* progress.c,v 1.10 2005/10/08 14:02:47 jenglish Exp
+/* progress.c,v 1.13 2006/01/31 07:30:24 jenglish Exp
  *
  * Copyright (c) Joe English, Pat Thoyts, Michael Kirkham
  *
@@ -325,9 +325,9 @@ static void ProgressbarIndeterminateLayout(
     }
 
     if (orient == TTK_ORIENT_HORIZONTAL) {
-	pbarBox.x = parcel.x + fraction * (parcel.width-pbarBox.width);
+	pbarBox.x = parcel.x + (int)(fraction * (parcel.width-pbarBox.width));
     } else {
-	pbarBox.y = parcel.y + fraction * (parcel.height-pbarBox.height);
+	pbarBox.y = parcel.y + (int)(fraction * (parcel.height-pbarBox.height));
     }
     Ttk_PlaceLayoutNode(pb->core.layout, pbarNode, pbarBox);
 }
@@ -449,7 +449,7 @@ static int ProgressbarStepCommand(
 }
 
 /* $sb start|stop ?args? --
- * Change [$sb $cmd ...] to [tile::progressbar::$cmd ...] 
+ * Change [$sb $cmd ...] to [ttk::progressbar::$cmd ...] 
  * and pass to interpreter.
  */
 static int ProgressbarStartStopCommand(
@@ -476,14 +476,14 @@ static int ProgressbarStartCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     return ProgressbarStartStopCommand(
-	interp, "tile::progressbar::start", objc, objv);
+	interp, "::ttk::progressbar::start", objc, objv);
 }
 
 static int ProgressbarStopCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     return ProgressbarStartStopCommand(
-	interp, "tile::progressbar::stop", objc, objv);
+	interp, "::ttk::progressbar::stop", objc, objv);
 }
 
 static WidgetCommandSpec ProgressbarCommands[] =
@@ -522,12 +522,12 @@ static WidgetSpec ProgressbarWidgetSpec =
  */
 TTK_BEGIN_LAYOUT(VerticalProgressbarLayout)
     TTK_GROUP("Vertical.Progressbar.trough", TTK_FILL_BOTH,
-	TTK_NODE("Vertical.Progressbar.pbar", TTK_PACK_BOTTOM))
+	TTK_NODE("Vertical.Progressbar.pbar", TTK_PACK_BOTTOM|TTK_FILL_X))
 TTK_END_LAYOUT
 
 TTK_BEGIN_LAYOUT(HorizontalProgressbarLayout)
     TTK_GROUP("Horizontal.Progressbar.trough", TTK_FILL_BOTH,
-	TTK_NODE("Horizontal.Progressbar.pbar", TTK_PACK_LEFT))
+	TTK_NODE("Horizontal.Progressbar.pbar", TTK_PACK_LEFT|TTK_FILL_Y))
 TTK_END_LAYOUT
 
 /*
