@@ -1,4 +1,4 @@
-/* scale.c,v 1.45 2005/10/08 14:02:47 jenglish Exp
+/* scale.c,v 1.47 2005/11/11 20:52:13 jenglish Exp
  * Copyright (C) 2004 Pat Thoyts <patthoyts@users.sourceforge.net>
  *
  * Tile widget set: scale widget.
@@ -12,9 +12,8 @@
 
 #define DEF_SCALE_LENGTH "100"
 
-#ifndef MAX
 #define MAX(a,b) ((a) > (b) ? (a) : (b))
-#endif
+#define MIN(a,b) ((a) < (b) ? (a) : (b))
 
 /*
  * Scale widget record
@@ -109,6 +108,16 @@ static int ScaleInitialize(Tcl_Interp *interp, void *recordPtr)
     return TCL_OK;
 }
 
+static void ScaleCleanup(void *recordPtr)
+{
+    Scale *scale = recordPtr;
+
+    if (scale->scale.variableTrace) {
+	Ttk_UntraceVariable(scale->scale.variableTrace);
+	scale->scale.variableTrace = 0;
+    }
+}
+
 /* ScaleConfigure --
  * 	Configuration hook.
  */
@@ -228,44 +237,19 @@ static double ScaleFraction(Scale *scalePtr, double value)
     Tcl_GetDoubleFromObj(NULL, scalePtr->scale.fromObj, &from);
     Tcl_GetDoubleFromObj(NULL, scalePtr->scale.toObj, &to);
 
-    if (from == to) 
+    if (from == to) {
 	return 1.0;
+    }
 
     fraction = (value - from) / (to - from);
 
     return fraction < 0 ? 0 : fraction > 1 ? 1 : fraction;
 }
 
-/*
- * Scale 'identify' method:
- * 	$widget identify $x $y
- *
- * Returns: name of element at $x, $y
-*/
-static int
-ScaleIdentifyCommand(
-    Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
-{
-    Scale *scalePtr = recordPtr;
-    Ttk_LayoutNode *node;
-    int x, y;
-
-    if (objc != 4) {
-	Tcl_WrongNumArgs(interp, 2, objv, "x y");
-	return TCL_ERROR;
-    }
-
-    if (Tcl_GetIntFromObj(interp, objv[2], &x) != TCL_OK
-	|| Tcl_GetIntFromObj(interp, objv[3], &y) != TCL_OK)
-	return TCL_ERROR;
-
-    node = Ttk_LayoutIdentify(scalePtr->core.layout, x, y); 
-    if (node) {
-	Tcl_SetObjResult(interp,Tcl_NewStringObj(Ttk_LayoutNodeName(node),-1));
-    }
-    return TCL_OK;
-}
-
+/* $scale get ?x y? --
+ * 	Returns the current value of the scale widget, or if $x and 
+ * 	$y are specified, the value represented by point @x,y.
+ */
 static int
 ScaleGetCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
@@ -292,33 +276,41 @@ ScaleGetCommand(
     return r;
 }
 
+/* $scale set $newValue
+ */
 static int
 ScaleSetCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     Scale *scalePtr = recordPtr;
-    double value, min, max;
-    int r = TCL_OK;
+    double from = 0.0, to = 1.0, value;
+    int result = TCL_OK;
 
     if (objc != 3) {
 	Tcl_WrongNumArgs(interp, 1, objv, "set value");
 	return TCL_ERROR;
     }
 
-    r = Tcl_GetDoubleFromObj(interp, objv[2], &value);
-    if (r == TCL_OK)
-	r = Tcl_GetDoubleFromObj(interp, scalePtr->scale.fromObj, &min);
-    if (r == TCL_OK)
-	r = Tcl_GetDoubleFromObj(interp, scalePtr->scale.toObj, &max);
+    if (Tcl_GetDoubleFromObj(interp, objv[2], &value) != TCL_OK) {
+	return TCL_ERROR;
+    }
 
-    if (r != TCL_OK) 
-	return r;
+    if (scalePtr->core.state & TTK_STATE_DISABLED) {
+	return TCL_OK;
+    }
 
-    if (scalePtr->core.state & TTK_STATE_DISABLED) 
-	return r;
+    /* ASSERT: fromObj and toObj are valid doubles.
+     */
+    Tcl_GetDoubleFromObj(interp, scalePtr->scale.fromObj, &from);
+    Tcl_GetDoubleFromObj(interp, scalePtr->scale.toObj, &to);
 
-    if (value < min) value = min;
-    if (value > max) value = max;
+    /* Limit new value to between 'from' and 'to':
+     */
+    if (from < to) {
+	value = value < from ? from : value > to ? to : value;
+    } else {
+	value = value < to ? to : value > from ? from : value;
+    }
 
     /*
      * Set value:
@@ -342,16 +334,16 @@ ScaleSetCommand(
     /*
      * Invoke -command, if any:
      */
-    if (r == TCL_OK && scalePtr->scale.commandObj != NULL) {
+    if (scalePtr->scale.commandObj != NULL) {
 	Tcl_Obj *cmdObj = Tcl_DuplicateObj(scalePtr->scale.commandObj);
 	Tcl_IncrRefCount(cmdObj);
 	Tcl_AppendToObj(cmdObj, " ", 1);
 	Tcl_AppendObjToObj(cmdObj, scalePtr->scale.valueObj);
-	r = Tcl_EvalObjEx(interp, cmdObj, TCL_EVAL_GLOBAL);
+	result = Tcl_EvalObjEx(interp, cmdObj, TCL_EVAL_GLOBAL);
 	Tcl_DecrRefCount(cmdObj);
     }
 
-    return r;
+    return result;
 }
 
 static int
@@ -486,7 +478,7 @@ static WidgetCommandSpec ScaleCommands[] =
     { "cget",        WidgetCgetCommand },
     { "state",       WidgetStateCommand },
     { "instate",     WidgetInstateCommand },
-    { "identify",    ScaleIdentifyCommand },
+    { "identify",    WidgetIdentifyCommand },
     { "set",         ScaleSetCommand },
     { "get",         ScaleGetCommand },
     { "coords",      ScaleCoordsCommand },
@@ -500,7 +492,7 @@ WidgetSpec ScaleWidgetSpec =
     ScaleOptionSpecs,		/* option specs */
     ScaleCommands,		/* widget commands */
     ScaleInitialize,		/* initialization proc */
-    NullCleanup,		/* cleanup proc */
+    ScaleCleanup,		/* cleanup proc */
     ScaleConfigure,		/* configure proc */
     ScalePostConfigure,		/* postConfigure */
     ScaleGetLayout, 		/* getLayoutProc */
