@@ -1,4 +1,4 @@
-/* scroll.c,v 1.3 2005/10/08 14:02:47 jenglish Exp
+/* scroll.c,v 1.4 2006/04/06 19:52:25 jenglish Exp
  *
  * Copyright 2004, Joe English
  *
@@ -13,8 +13,26 @@
  *      + 'total' depends on widget contents;
  *      + 'last' depends on first, total, and widget size.
  *
- * Once the widget knows 'total' and 'last' (typically in the LayoutProc),
- * call Scrolled(h,first,last,total) to synchronize the scrollbar.
+ * Choreography (typical usage):
+ *
+ * 	1. User adjusts scrollbar, scrollbar widget calls its -command
+ * 	2. Scrollbar -command invokes the scrollee [xy]view widget method
+ * 	3. ScrollviewCommand calls ScrollTo(), which updates 
+ * 	   'first' and schedules a redisplay.
+ * 	4. Once the scrollee knows 'total' and 'last' (typically in 
+ * 	   the LayoutProc), call Scrolled(h,first,last,total) to 
+ * 	   synchronize the scrollbar.
+ * 	5. The scrollee -[xy]scrollcommand is called (in an idle callback)
+ * 	6. Which calls the scrollbar 'set' method and redisplays the scrollbar.
+ *
+ * If the scrollee has internal scrolling (e.g., a 'see' method), 
+ * it should ScrollTo() directly (step 2).
+ *
+ * If the widget value changes, it should call Scrolled() (step 4).
+ * (This usually happens automatically when the widget is redisplayed).
+ *
+ * If the scrollee's -[xy]scrollcommand changes, it should call
+ * ScrollbarUpdateRequired, which will invoke step (5) (@@@ Fix this)
  */
 
 #include <tk.h>
@@ -115,7 +133,6 @@ static void UpdateScrollbarBG(ClientData clientData)
     Tcl_Release(clientData);
 }
 
-
 /* Scrolled --
  * 	Update scroll info, schedule scrollbar update.
  */
@@ -202,6 +219,16 @@ int ScrollviewCommand(
 	    }
 	}
     }
+
+    ScrollTo(h, newFirst);
+
+    return TCL_OK;
+}
+
+void ScrollTo(ScrollHandle h, int newFirst)
+{
+    Scrollable *s = h->scrollPtr;
+
     if (newFirst >= s->total)
 	newFirst = s->total - 1;
     if (newFirst > s->first && s->last >= s->total) /* don't scroll past end */
@@ -213,8 +240,5 @@ int ScrollviewCommand(
 	s->first = newFirst;
 	TtkRedisplayWidget(h->corePtr);
     }
-
-    return TCL_OK;
 }
 
-
