@@ -1,4 +1,4 @@
-/* scrollbar.c,v 1.43 2005/10/08 14:02:47 jenglish Exp
+/* scrollbar.c,v 1.44 2005/11/10 18:18:25 jenglish Exp
  * Copyright (c) 2003, Joe English
  * Tile widget set: scrollbar widget implementation.
  */
@@ -8,7 +8,6 @@
 
 #include "tkTheme.h"
 #include "widget.h"
-#include "compat.h"
 
 /*------------------------------------------------------------------------
  * +++ Scrollbar widget record.
@@ -31,7 +30,6 @@ typedef struct
 {
     WidgetCore core;
     ScrollbarPart scrollbar;
-    SCROLLBAR_COMPAT_DECLS
 } Scrollbar;
 
 static Tk_OptionSpec ScrollbarOptionSpecs[] =
@@ -47,8 +45,6 @@ static Tk_OptionSpec ScrollbarOptionSpecs[] =
     WIDGET_INHERIT_OPTIONS(CoreOptionSpecs)
 };
 
-SCROLLBAR_COMPAT_OPTIONS
-
 /*------------------------------------------------------------------------
  * +++ Widget hooks.
  */
@@ -303,7 +299,7 @@ WidgetSpec ScrollbarWidgetSpec =
 {
     "TScrollbar",		/* className */
     sizeof(Scrollbar),		/* recordSize */
-    ScrollbarCompatOptionSpecs,	/* optionSpecs */
+    ScrollbarOptionSpecs,	/* optionSpecs */
     ScrollbarCommands,		/* subcommands */
     ScrollbarInitialize,	/* initializeProc */
     NullCleanup,		/* cleanupProc */
