@@ -1,4 +1,4 @@
-/* tkElements.c,v 1.75 2005/10/07 16:21:40 jenglish Exp
+/* tkElements.c,v 1.77 2005/12/12 20:18:19 jenglish Exp
  *
  * Copyright (c) 2003, Joe English
  *
@@ -1252,19 +1252,17 @@ TTK_BEGIN_LAYOUT(ButtonLayout)
 TTK_END_LAYOUT
 
 TTK_BEGIN_LAYOUT(CheckbuttonLayout)
-     TTK_GROUP("Checkbutton.border", TTK_FILL_BOTH|TTK_BORDER,
-	 TTK_GROUP("Checkbutton.padding", TTK_FILL_BOTH,
-	     TTK_NODE("Checkbutton.indicator", TTK_PACK_LEFT)
-	     TTK_GROUP("Checkbutton.focus", TTK_PACK_LEFT | TTK_STICK_W,
-		 TTK_NODE("Checkbutton.label", TTK_FILL_BOTH))))
+     TTK_GROUP("Checkbutton.padding", TTK_FILL_BOTH,
+	 TTK_NODE("Checkbutton.indicator", TTK_PACK_LEFT)
+	 TTK_GROUP("Checkbutton.focus", TTK_PACK_LEFT | TTK_STICK_W,
+	     TTK_NODE("Checkbutton.label", TTK_FILL_BOTH)))
 TTK_END_LAYOUT
 
 TTK_BEGIN_LAYOUT(RadiobuttonLayout)
-     TTK_GROUP("Radiobutton.border", TTK_FILL_BOTH|TTK_BORDER,
-	 TTK_GROUP("Radiobutton.padding", TTK_FILL_BOTH,
-	     TTK_NODE("Radiobutton.indicator", TTK_PACK_LEFT)
-	     TTK_GROUP("Radiobutton.focus", TTK_PACK_LEFT,
-		 TTK_NODE("Radiobutton.label", TTK_FILL_BOTH))))
+     TTK_GROUP("Radiobutton.padding", TTK_FILL_BOTH,
+	 TTK_NODE("Radiobutton.indicator", TTK_PACK_LEFT)
+	 TTK_GROUP("Radiobutton.focus", TTK_PACK_LEFT,
+	     TTK_NODE("Radiobutton.label", TTK_FILL_BOTH)))
 TTK_END_LAYOUT
 
 TTK_BEGIN_LAYOUT(MenubuttonLayout)
@@ -1272,7 +1270,7 @@ TTK_BEGIN_LAYOUT(MenubuttonLayout)
 	TTK_GROUP("Menubutton.focus", TTK_FILL_BOTH,
 	    TTK_NODE("Menubutton.indicator", TTK_PACK_RIGHT)
 	    TTK_GROUP("Menubutton.padding", TTK_PACK_LEFT|TTK_EXPAND|TTK_FILL_X,
-	        TTK_NODE("Menubutton.label", 0))))
+	        TTK_NODE("Menubutton.label", TTK_PACK_LEFT))))
 TTK_END_LAYOUT
 
 TTK_BEGIN_LAYOUT(VerticalScrollbarLayout)
