@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * tkTheme.c,v 1.86 2005/09/14 17:24:28 jenglish Exp
+ * tkTheme.c,v 1.89 2006/04/06 19:34:11 jenglish Exp
  */
 
 #include <stdlib.h>
@@ -26,21 +26,31 @@ EXTERN CONST Tk_OptionSpec *TkGetOptionSpec (CONST char *name,
 
 /*------------------------------------------------------------------------
  * +++ Styles.
+ *
+ * Invariants:
+ * 	If styleName contains a dot, parentStyle->styleName is everything
+ * 	after the first dot; otherwise, parentStyle is the theme's root
+ * 	style ".".  The root style's parentStyle is NULL.
+ *
  */
 
 typedef struct Ttk_Style_
 {
+    const char		*styleName;	/* points to hash table key */
     Tcl_HashTable	settingsTable;	/* KEY: string; VALUE: StateMap */
     Tcl_HashTable	defaultsTable;	/* KEY: string; VALUE: resource */
+    Ttk_LayoutTemplate	layoutTemplate;	/* Layout template for style, or NULL */
     Ttk_Style		parentStyle;	/* Previous style in chain */
     Ttk_ResourceCache	cache;		/* Back-pointer to resource cache */
 } Style;
 
-static Style *NewStyle(void)
+static Style *NewStyle()
 {
     Style *stylePtr = (Style*)ckalloc(sizeof(Style));
 
+    stylePtr->styleName = NULL;
     stylePtr->parentStyle = NULL;
+    stylePtr->layoutTemplate = NULL;
     stylePtr->cache = NULL;
     Tcl_InitHashTable(&stylePtr->settingsTable, TCL_STRING_KEYS);
     Tcl_InitHashTable(&stylePtr->defaultsTable, TCL_STRING_KEYS);
@@ -69,6 +79,8 @@ static void FreeStyle(Style *stylePtr)
     }
     Tcl_DeleteHashTable(&stylePtr->defaultsTable);
 
+    Ttk_FreeLayoutTemplate(stylePtr->layoutTemplate);
+
     ckfree((char*)stylePtr);
 }
 
@@ -143,12 +155,6 @@ static const Tk_OptionSpec *TTKGetOptionSpec(
 	return 0;
     }
 
-    /* Ignore compatibility options:
-     */
-    if (optionSpec->flags & TK_OPTION_COMPAT) {
-	return 0;
-    }
-
     /* Grrr.  Ignore accidental mismatches caused by prefix-matching:
      */
     if (strcmp(optionSpec->optionName, optionName)) {
@@ -303,7 +309,6 @@ typedef struct Ttk_Theme_
     Ttk_Theme parentPtr;             	/* Parent theme. */
     Tcl_HashTable elementTable;	     	/* Map element names to ElementImpls */
     Tcl_HashTable styleTable;	     	/* Map style names to Styles */
-    Tcl_HashTable layoutTable;	     	/* Map layout names to LayoutTemplate */
     Ttk_Style rootStyle;		/* "." style, root of chain */
     Ttk_ThemeEnabledProc *enabledProc;	/* Function called by SetTheme */
     void *enabledData;              	/* ClientData for enabledProc */
@@ -322,20 +327,21 @@ static Theme *NewTheme(Ttk_ResourceCache cache, Ttk_Theme parent)
     themePtr->cache = cache;
     Tcl_InitHashTable(&themePtr->elementTable, TCL_STRING_KEYS);
     Tcl_InitHashTable(&themePtr->styleTable, TCL_STRING_KEYS);
-    Tcl_InitHashTable(&themePtr->layoutTable, TCL_STRING_KEYS);
 
     /*
      * Create root style "."
      */
     entryPtr = Tcl_CreateHashEntry(&themePtr->styleTable, ".", &unused);
     themePtr->rootStyle = NewStyle();
+    themePtr->rootStyle->styleName = 
+	Tcl_GetHashKey(&themePtr->styleTable, entryPtr);
     themePtr->rootStyle->cache = themePtr->cache;
     Tcl_SetHashValue(entryPtr, (ClientData)themePtr->rootStyle);
 
     return themePtr;
 }
 
-static void FreeTheme(Tcl_Interp *interp, Theme *themePtr)
+static void FreeTheme(Theme *themePtr)
 {
     Tcl_HashSearch search;
     Tcl_HashEntry *entryPtr;
@@ -362,18 +368,6 @@ static void FreeTheme(Tcl_Interp *interp, Theme *themePtr)
     }
     Tcl_DeleteHashTable(&themePtr->styleTable);
 
-    /*
-     * Free layout table:
-     */
-    entryPtr = Tcl_FirstHashEntry(&themePtr->layoutTable, &search);
-    while (entryPtr != NULL) {
-	Ttk_LayoutTemplate layoutTemplate =
-	    (Ttk_LayoutTemplate)Tcl_GetHashValue(entryPtr);
-	Ttk_FreeLayoutTemplate(layoutTemplate);
-	entryPtr = Tcl_NextHashEntry(&search);
-    }
-    Tcl_DeleteHashTable(&themePtr->layoutTable);
-
     /*
      * Free theme record:
      */
@@ -440,7 +434,7 @@ static void Ttk_StylePkgFree(ClientData clientData, Tcl_Interp *interp)
     entryPtr = Tcl_FirstHashEntry(&pkgPtr->themeTable, &search);
     while (entryPtr != NULL) {
 	themePtr = (Theme *) Tcl_GetHashValue(entryPtr);
-	FreeTheme(interp, themePtr);
+	FreeTheme(themePtr);
 	entryPtr = Tcl_NextHashEntry(&search);
     }
     Tcl_DeleteHashTable(&pkgPtr->themeTable);
@@ -474,6 +468,16 @@ static void Ttk_StylePkgFree(ClientData clientData, Tcl_Interp *interp)
     ckfree((char*)pkgPtr);
 }
 
+/*
+ * GetStylePackageData --
+ * 	Look up the package data registered with the interp.
+ */
+
+static StylePackageData *GetStylePackageData(Tcl_Interp *interp)
+{
+    return (StylePackageData*)Tcl_GetAssocData(interp, "StylePackage", NULL);
+}
+
 /*
  * Ttk_RegisterCleanup --
  *
@@ -486,8 +490,7 @@ static void Ttk_StylePkgFree(ClientData clientData, Tcl_Interp *interp)
 void Ttk_RegisterCleanup(
     Tcl_Interp *interp, ClientData clientData, Ttk_CleanupProc *cleanupProc)
 {
-    StylePackageData *pkgPtr = (StylePackageData *)
-	Tcl_GetAssocData(interp, "StylePackage", NULL);
+    StylePackageData *pkgPtr = GetStylePackageData(interp);
     Cleanup *cleanup = (Cleanup*)ckalloc(sizeof(*cleanup));
 
     cleanup->clientData = clientData;
@@ -546,8 +549,7 @@ Ttk_CreateTheme(
     const char *name,		/* Name of the theme to create. */
     Ttk_Theme parent) 		/* Parent/fallback theme, NULL for default */
 {
-    StylePackageData *pkgPtr = (StylePackageData *)
-	Tcl_GetAssocData(interp, "StylePackage", NULL);
+    StylePackageData *pkgPtr = GetStylePackageData(interp);
     Tcl_HashEntry *entryPtr;
     int newEntry;
     Theme *themePtr;
@@ -584,17 +586,13 @@ void Ttk_SetThemeEnabledProc(
 }
 
 /*
- * Ttk_GetTheme --
- *	Retrieve a registered theme by name
- *
- * Results:
- *	A pointer to the theme, or NULL if none found.
- *	If not found, leaves an error message in interp's result.
+ * LookupTheme --
+ *	Retrieve a registered theme by name.  If not found, 
+ *	returns NULL and leaves an error message in interp's result.
  */
 
-static Ttk_Theme
-LookupTheme(
-    Tcl_Interp *interp,		/* where to leave error messages (optional) */
+static Ttk_Theme LookupTheme(
+    Tcl_Interp *interp,		/* where to leave error messages */
     StylePackageData *pkgPtr,	/* style package master record */
     const char *name)		/* theme name */
 {
@@ -612,33 +610,24 @@ LookupTheme(
 
 /*
  * Ttk_GetTheme --
- *	Look up a Ttk_Theme by name.
- *
- * Results:
- *	The return value is a token for the named theme,
- *	NULL if not found.  If NULL is returned, an error
- *	message will be left in interp's result.
+ *	Public interface to LookupTheme.
  */
-
 Ttk_Theme Ttk_GetTheme(Tcl_Interp *interp, const char *themeName)
 {
-    StylePackageData *pkgPtr = (StylePackageData *)
-	Tcl_GetAssocData(interp, "StylePackage", NULL);
+    StylePackageData *pkgPtr = GetStylePackageData(interp);
 
     return LookupTheme(interp, pkgPtr, themeName);
 }
 
 Ttk_Theme Ttk_GetCurrentTheme(Tcl_Interp *interp)
 {
-    StylePackageData *pkgPtr = (StylePackageData *)
-	Tcl_GetAssocData(interp, "StylePackage", NULL);
+    StylePackageData *pkgPtr = GetStylePackageData(interp);
     return pkgPtr->currentTheme;
 }
 
 Ttk_Theme Ttk_GetDefaultTheme(Tcl_Interp *interp)
 {
-    StylePackageData *pkgPtr = (StylePackageData *)
-	Tcl_GetAssocData(interp, "StylePackage", NULL);
+    StylePackageData *pkgPtr = GetStylePackageData(interp);
     return pkgPtr->defaultTheme;
 }
 
@@ -648,8 +637,7 @@ Ttk_Theme Ttk_GetDefaultTheme(Tcl_Interp *interp)
  */
 int Ttk_UseTheme(Tcl_Interp *interp, Ttk_Theme  theme)
 {
-    StylePackageData *pkgPtr = (StylePackageData *)
-	Tcl_GetAssocData(interp, "StylePackage", NULL);
+    StylePackageData *pkgPtr = GetStylePackageData(interp);
 
     /*
      * Check if selected theme is enabled:
@@ -675,28 +663,24 @@ int Ttk_UseTheme(Tcl_Interp *interp, Ttk_Theme  theme)
 Ttk_ResourceCache
 Ttk_GetResourceCache(Tcl_Interp *interp)
 {
-    StylePackageData *pkgPtr = (StylePackageData *)
-	Tcl_GetAssocData(interp, "StylePackage", NULL);
+    StylePackageData *pkgPtr = GetStylePackageData(interp);
     return pkgPtr->cache;
 }
 
 /*
- * Register a new layout specification
+ * Register a new layout specification with a style.
+ * @@@ TODO: Make sure layoutName is not ".", root style must not have a layout
  */
 static void Ttk_RegisterLayoutTemplate(
-    Ttk_Theme themePtr,			/* Target theme */
+    Ttk_Theme theme,			/* Target theme */
     const char *layoutName,		/* Name of new layout */
     Ttk_LayoutTemplate layoutTemplate)	/* Template */
 {
-    Tcl_HashEntry *entryPtr;
-    int newEntry;
-
-    entryPtr=Tcl_CreateHashEntry(&themePtr->layoutTable,layoutName,&newEntry);
-    if (!newEntry) {
-	Ttk_LayoutTemplate oldTemplate = Tcl_GetHashValue(entryPtr);
-	Ttk_FreeLayoutTemplate(oldTemplate);
+    Ttk_Style style = Ttk_GetStyle(theme, layoutName);
+    if (style->layoutTemplate) {
+	Ttk_FreeLayoutTemplate(style->layoutTemplate);
     }
-    Tcl_SetHashValue(entryPtr, (ClientData)layoutTemplate);
+    style->layoutTemplate = layoutTemplate;
 }
 
 void Ttk_RegisterLayout(
@@ -728,6 +712,7 @@ Ttk_Style Ttk_GetStyle(Ttk_Theme themePtr, const char *styleName)
 	    stylePtr->parentStyle = themePtr->rootStyle;
 	}
 
+	stylePtr->styleName = Tcl_GetHashKey(&themePtr->styleTable, entryPtr);
 	stylePtr->cache = stylePtr->parentStyle->cache;
 	Tcl_SetHashValue(entryPtr, (ClientData)stylePtr);
 	return stylePtr;
@@ -743,28 +728,24 @@ Ttk_Style Ttk_GetStyle(Ttk_Theme themePtr, const char *styleName)
 Ttk_LayoutTemplate
 Ttk_FindLayoutTemplate(Ttk_Theme themePtr, const char *layoutName)
 {
-    const char *dot = layoutName;
-    Tcl_HashEntry *entryPtr =
-	Tcl_FindHashEntry(&themePtr->layoutTable, layoutName);
-
-    /* Look for named layout in current theme, from specific to generic:
-     */
-    while (!entryPtr && ((dot = strchr(dot, '.')) != NULL)) {
-	dot++;
-	entryPtr = Tcl_FindHashEntry(&themePtr->layoutTable, dot);
-    }
-    if (entryPtr) {
-	return (Ttk_LayoutTemplate)Tcl_GetHashValue(entryPtr);
-    }
-
-    /* Look for specific layout in parent theme:
-     */
-    if (themePtr->parentPtr) {
-        return Ttk_FindLayoutTemplate(themePtr->parentPtr, layoutName);
+    while (themePtr) {
+	Ttk_Style stylePtr = Ttk_GetStyle(themePtr, layoutName);
+	while (stylePtr) {
+	    if (stylePtr->layoutTemplate) {
+		return stylePtr->layoutTemplate;
+	    }
+	    stylePtr = stylePtr->parentStyle;
+	}
+	themePtr = themePtr->parentPtr;
     }
     return NULL;
 }
 
+const char *Ttk_StyleName(Ttk_Style stylePtr)
+{
+    return stylePtr->styleName;
+}
+
 /*
  * Ttk_GetElement --
  *	Look up an element implementation by name in a given theme.
@@ -825,8 +806,7 @@ int Ttk_RegisterElementFactory(
     Tcl_Interp *interp,	const char *name,
     Ttk_ElementFactory factory, void *clientData)
 {
-    StylePackageData *pkgPtr = (StylePackageData *)
-	Tcl_GetAssocData(interp, "StylePackage", NULL);
+    StylePackageData *pkgPtr = GetStylePackageData(interp);
     FactoryRec *recPtr = (FactoryRec*)ckalloc(sizeof(*recPtr));
     Tcl_HashEntry *entryPtr;
     int newEntry;
@@ -1191,7 +1171,7 @@ static Tcl_Obj* HashTableToDict(Tcl_HashTable *ht)
     return result;
 }
 
-/* style map $style ? -resource statemap ... ?
+/* + style map $style ? -resource statemap ... ?
  *
  * 	Note that resource names are unconstrained; the Style
  * 	doesn't know what resources individual elements may use.
@@ -1261,7 +1241,7 @@ usage:
     return TCL_OK;
 }
 
-/* style configure $style -option ?value...
+/* + style configure $style -option ?value...
  */
 static int StyleConfigureCmd(
     ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[])
@@ -1315,7 +1295,51 @@ usage:
     return TCL_OK;
 }
 
-/* style theme create name ?-parent $theme? ?-settings { script }?
+/* + style lookup $style -option ?statespec? ?defaultValue?
+ */
+static int StyleLookupCmd(
+    ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[])
+{
+    StylePackageData *pkgPtr = clientData;
+    Ttk_Theme theme = pkgPtr->currentTheme;
+    Ttk_Style style = NULL;
+    const char *optionName;
+    Ttk_State state = 0ul;
+    Tcl_Obj *result;
+
+    if (objc < 4 || objc > 6) {
+	Tcl_WrongNumArgs(interp, 2, objv, "style -option ?state? ?default?");
+	return TCL_ERROR;
+    }
+
+    style = Ttk_GetStyle(theme, Tcl_GetString(objv[2]));
+    if (!style) {
+	return TCL_ERROR;
+    }
+    optionName = Tcl_GetString(objv[3]);
+
+    if (objc >= 5) {
+	Ttk_StateSpec stateSpec;
+	/* @@@ SB: Ttk_GetStateFromObj(); 'offbits' spec is ignored */
+	if (Ttk_GetStateSpecFromObj(interp, objv[4], &stateSpec) != TCL_OK) {
+	    return TCL_ERROR;
+	}
+	state = stateSpec.onbits;
+    }
+
+    result = Ttk_QueryStyle(style, NULL,NULL, optionName, state);
+    if (result == NULL && objc >= 6) { /* Use caller-supplied fallback */
+	result = objv[5];
+    }
+
+    if (result) {
+	Tcl_SetObjResult(interp, result);
+    }
+
+    return TCL_OK;
+}
+
+/* + style theme create name ?-parent $theme? ?-settings { script }?
  */
 static int StyleThemeCreateCmd(
     ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[])
@@ -1378,7 +1402,7 @@ static int StyleThemeCreateCmd(
     }
 }
 
-/* style theme names --
+/* + style theme names --
  * 	Return list of registered themes.
  */
 static int StyleThemeNamesCmd(
@@ -1388,7 +1412,7 @@ static int StyleThemeNamesCmd(
     return EnumerateHashTable(interp, &pkgPtr->themeTable);
 }
 
-/* style theme settings $theme $script
+/* + style theme settings $theme $script
  *
  * 	Temporarily sets the current theme to $themeName,
  * 	evaluates $script, then restores the old theme.
@@ -1421,7 +1445,7 @@ StyleThemeSettingsCmd(
     return status;
 }
 
-/* style element create name type ? ...args ?
+/* + style element create name type ? ...args ?
  */
 static int StyleElementCreateCmd(
     ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[])
@@ -1452,7 +1476,7 @@ static int StyleElementCreateCmd(
 	    theme, elementName, objc - 5, objv + 5);
 }
 
-/* style element names --
+/* + style element names --
  * 	Return a list of elements defined in the current theme.
  */
 static int StyleElementNamesCmd(
@@ -1468,7 +1492,7 @@ static int StyleElementNamesCmd(
     return EnumerateHashTable(interp, &theme->elementTable);
 }
 
-/* style element options $element --
+/* + style element options $element --
  */
 static int StyleElementOptionsCmd(
     ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[])
@@ -1505,7 +1529,7 @@ static int StyleElementOptionsCmd(
     return TCL_ERROR;
 }
 
-/* style layout name ?spec?
+/* + style layout name ?spec?
  */
 static int StyleLayoutCmd(
     ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[])
@@ -1541,7 +1565,7 @@ static int StyleLayoutCmd(
     return TCL_OK;
 }
 
-/*  style theme use $theme --
+/* + style theme use $theme --
  *  	Sets the current theme to $theme
  */
 static int
@@ -1595,11 +1619,13 @@ struct Ensemble StyleElementEnsemble[] = {
 
 struct Ensemble StyleEnsemble[] = {
     { "configure", StyleConfigureCmd, 0 },
-    { "default", StyleConfigureCmd, 0 }, /* TEMP: for pre-0.7 compatibility */
     { "map", StyleMapCmd, 0 },
+    { "lookup", StyleLookupCmd, 0 },
     { "layout", StyleLayoutCmd, 0 },
     { "theme", 0, StyleThemeEnsemble },
     { "element", 0, StyleElementEnsemble },
+
+    { "default", StyleConfigureCmd, 0 }, /* TEMP: for pre-0.7 compatibility */
     { NULL, 0, 0 }
 };
 
@@ -1639,8 +1665,7 @@ StyleObjCmd(
  *	package on a per-interp basis.
  */
 
-void
-Ttk_StylePkgInit(Tcl_Interp *interp)
+void Ttk_StylePkgInit(Tcl_Interp *interp)
 {
     StylePackageData *pkgPtr = (StylePackageData *)
 	ckalloc(sizeof(StylePackageData));
