@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2003 Joe English.  Freely redistributable.
  *
- * tkTheme.h,v 1.110 2005/10/15 02:15:47 jenglish Exp
+ * tkTheme.h,v 1.117 2006/04/20 22:07:30 jenglish Exp
  */
 
 #ifndef TKTHEME_H
@@ -21,7 +21,7 @@ extern "C" {
 #	define TTKAPI DLLIMPORT
 #endif
 
-#define TILE_VERSION	"0.7.2"
+#define TILE_VERSION	"0.7.6"
 #define TILE_PATCHLEVEL	TILE_VERSION
 
 /*------------------------------------------------------------------------
@@ -145,13 +145,6 @@ typedef enum {
 
 typedef unsigned int Ttk_Sticky;
 
-TTKAPI Ttk_Box Ttk_PackBox(Ttk_Box *cavity, int w, int h, Ttk_Side side);
-TTKAPI Ttk_Box Ttk_StickBox(Ttk_Box parcel, int w, int h, Ttk_Sticky sticky);
-TTKAPI Ttk_Box Ttk_AnchorBox(Ttk_Box parcel, int w, int h, int anchor);
-TTKAPI Ttk_Box Ttk_PadBox(Ttk_Box b, Ttk_Padding p);
-TTKAPI Ttk_Box Ttk_ExpandBox(Ttk_Box b, Ttk_Padding p);
-TTKAPI Ttk_Box Ttk_PlaceBox(Ttk_Box *cavity, int w, int h, Ttk_Side side, Ttk_Sticky sticky);
-
 /*
  * -sticky bits for Ttk_StickBox:
  */
@@ -171,8 +164,11 @@ TTKAPI int Ttk_GetStickyFromObj(Tcl_Interp *, Tcl_Obj *, Ttk_Sticky *);
 TTKAPI Tcl_Obj *Ttk_NewStickyObj(Ttk_Sticky);
 
 /*
- * Extra bits for layout specifications:
+ * Extra bits for position specifications (combine -side and -sticky)
  */
+
+typedef unsigned int Ttk_PositionSpec;	/* See below */
+
 #define TTK_PACK_LEFT	(0x10)	/* pack at left of current parcel */
 #define TTK_PACK_RIGHT	(0x20)	/* pack at right of current parcel */
 #define TTK_PACK_TOP	(0x40)	/* pack at top of current parcel */
@@ -180,12 +176,25 @@ TTKAPI Tcl_Obj *Ttk_NewStickyObj(Ttk_Sticky);
 #define TTK_EXPAND	(0x100)	/* use entire parcel */
 #define TTK_BORDER	(0x200)	/* draw this element after children */
 #define TTK_UNIT	(0x400)	/* treat descendants as a part of element */
+
+/*
+ * Extra bits for layout specifications
+ */
 #define TTK_CHILDREN	(0x1000)/* for LayoutSpecs -- children follow */
 #define TTK_LAYOUT_END	(0x2000)/* for LayoutSpecs -- end of child list */
 
 #define _TTK_MASK_STICK (0x0F)	/* See Ttk_UnparseLayout() */
 #define _TTK_MASK_PACK	(0xF0)	/* See Ttk_UnparseLayout(), packStrings */
 
+
+TTKAPI Ttk_Box Ttk_PackBox(Ttk_Box *cavity, int w, int h, Ttk_Side side);
+TTKAPI Ttk_Box Ttk_StickBox(Ttk_Box parcel, int w, int h, Ttk_Sticky sticky);
+TTKAPI Ttk_Box Ttk_AnchorBox(Ttk_Box parcel, int w, int h, Tk_Anchor anchor);
+TTKAPI Ttk_Box Ttk_PadBox(Ttk_Box b, Ttk_Padding p);
+TTKAPI Ttk_Box Ttk_ExpandBox(Ttk_Box b, Ttk_Padding p);
+TTKAPI Ttk_Box Ttk_PlaceBox(Ttk_Box *cavity, int w,int h, Ttk_Side,Ttk_Sticky);
+TTKAPI Ttk_Box Ttk_PositionBox(Ttk_Box *cavity, int w, int h, Ttk_PositionSpec);
+
 /*------------------------------------------------------------------------
  * +++ Themes.
  */
@@ -223,13 +232,6 @@ typedef void (Ttk_ElementSizeProc)(void *clientData, void *elementRecord,
 typedef void (Ttk_ElementDrawProc)(void *clientData, void *elementRecord,
         Tk_Window tkwin, Drawable d, Ttk_Box b, Ttk_State state);
 
-/*
- * Flag bit for widget optionSpec->flags, indicating that
- * this is a compatibility resource and should be ignored
- * by the theme engine:
- */
-#define TK_OPTION_COMPAT	(TK_CONFIG_USER_BIT)
-
 typedef struct Ttk_ElementOptionSpec 
 {
     char *optionName;		/* Command-line name of the widget option */
@@ -302,9 +304,8 @@ extern Ttk_Layout Ttk_CreateLayout(
     Tcl_Interp *, Ttk_Theme, const char *name,
     void *recordPtr, Tk_OptionTable, Tk_Window tkwin);
 
-extern Ttk_Layout Ttk_CreateSubLayout(
-    Tcl_Interp *, Ttk_Theme, const char *name,
-    void *recordPtr, Tk_OptionTable, Tk_Window tkwin);
+extern Ttk_Layout Ttk_CreateSublayout(
+    Tcl_Interp *, Ttk_Theme, Ttk_Layout, const char *name, Tk_OptionTable);
 
 extern void Ttk_FreeLayout(Ttk_Layout);
 
