@@ -1,5 +1,5 @@
 /*
- * treeview.c,v 1.20 2005/10/08 14:02:47 jenglish Exp
+ * treeview.c,v 1.26 2006/04/06 19:52:25 jenglish Exp
  * Copyright (c) 2004, Joe English
  *
  * Tile widget set: treeview widget.
@@ -10,10 +10,6 @@
 #include "tkTheme.h"
 #include "widget.h"
 
-#define ITEM_LAYOUT		"Item"
-#define CELL_LAYOUT		"Cell"
-#define HEADING_LAYOUT		"Heading"
-#define ROW_LAYOUT		"Row"
 #define DEF_TREE_ROWS		"10"
 #define DEF_TREE_PADDING	"4"
 #define DEF_COLWIDTH		"200"
@@ -180,6 +176,7 @@ typedef struct {
      */
     Tcl_Obj 	*headingObj;		/* Heading label */
     Tcl_Obj	*headingImageObj;	/* Heading image */
+    Tcl_Obj 	*headingAnchorObj;	/* -anchor for heading label */
     Tcl_Obj	*headingCommandObj;	/* Command to execute */
     Tcl_Obj 	*headingStateObj;	/* @@@ testing ... */
     Ttk_State	headingState;		/* ... */
@@ -198,6 +195,7 @@ static void InitColumn(TreeColumn *column)
     column->headingState = 0;
     column->headingObj = 0;
     column->headingImageObj = 0;
+    column->headingAnchorObj = 0;
     column->headingStateObj = 0;
     column->headingCommandObj = 0;
 
@@ -211,6 +209,7 @@ static void FreeColumn(TreeColumn *column)	/* @@@ rename */
 
     if (column->headingObj) Tcl_DecrRefCount(column->headingObj);
     if (column->headingImageObj) Tcl_DecrRefCount(column->headingImageObj);
+    if (column->headingAnchorObj) Tcl_DecrRefCount(column->headingAnchorObj);
     if (column->headingStateObj) Tcl_DecrRefCount(column->headingStateObj);
     if (column->headingCommandObj) Tcl_DecrRefCount(column->headingCommandObj);
 
@@ -221,7 +220,7 @@ static Tk_OptionSpec ColumnOptionSpecs[] =
 {
     {TK_OPTION_INT, "-width", "width", "Width",
 	DEF_COLWIDTH, -1, Tk_Offset(TreeColumn,width),
-	0,0,0/*|GEOMETRY_CHANGED*/ },
+	0,0,GEOMETRY_CHANGED },
     {TK_OPTION_ANCHOR, "-anchor", "anchor", "Anchor",
 	"w", Tk_Offset(TreeColumn,anchorObj), -1,
 	0,0,0 },
@@ -239,6 +238,9 @@ static Tk_OptionSpec HeadingOptionSpecs[] =
     {TK_OPTION_STRING, "-image", "image", "Image",
 	"", Tk_Offset(TreeColumn,headingImageObj), -1,
 	0,0,0 },
+    {TK_OPTION_ANCHOR, "-anchor", "anchor", "Anchor",
+	"center", Tk_Offset(TreeColumn,headingAnchorObj), -1,
+	0,0,0 },
     {TK_OPTION_STRING, "-command", "", "",
 	"", Tk_Offset(TreeColumn,headingCommandObj), -1,
 	TK_OPTION_NULL_OK,0,0 },
@@ -269,9 +271,6 @@ typedef struct {
 
 static Tk_OptionSpec RowOptionSpecs[] =
 {
-    {TK_OPTION_STRING, "-unused", "unused", "Unused",
-	"", Tk_Offset(TreeRow,unusedObj), -1,
-	0,0,0 },
     {TK_OPTION_END, 0,0,0, NULL, -1,-1, 0,0,0}
 };
 
@@ -446,6 +445,17 @@ static Tcl_Obj *unshare(Tcl_Obj *objPtr)
     return objPtr;
 }
 
+/* @@@ Move this. */
+static Tcl_Obj *Ttk_NewBoxObj(Ttk_Box box)
+{
+    Tcl_Obj *result = Tcl_NewListObj(0,0);
+    Tcl_ListObjAppendElement(NULL, result, Tcl_NewIntObj(box.x));
+    Tcl_ListObjAppendElement(NULL, result, Tcl_NewIntObj(box.y));
+    Tcl_ListObjAppendElement(NULL, result, Tcl_NewIntObj(box.width));
+    Tcl_ListObjAppendElement(NULL, result, Tcl_NewIntObj(box.height));
+    return result;
+}
+
 /* DisplayLayout --
  * 	Rebind, place, and draw a layout + object combination.
  */
@@ -723,7 +733,8 @@ static int TreeviewInitialize(Tcl_Interp *interp, void *recordPtr)
 	Tk_CreateOptionTable(interp, DataCellOptionSpecs);
     tv->tree.headingOptionTable =
 	Tk_CreateOptionTable(interp, HeadingOptionSpecs);
-    tv->tree.rowOptionTable = Tk_CreateOptionTable(interp, RowOptionSpecs);
+    tv->tree.rowOptionTable =
+	Tk_CreateOptionTable(interp, RowOptionSpecs);
 
     tv->tree.itemLayout
 	= tv->tree.cellLayout
@@ -736,7 +747,14 @@ static int TreeviewInitialize(Tcl_Interp *interp, void *recordPtr)
     tv->tree.columns = NULL;
     tv->tree.displayColumns = NULL;
     tv->tree.showFlags = ~0;
+
     InitColumn(&tv->tree.column0);
+    Tk_InitOptions(
+	interp, (ClientData)(&tv->tree.column0),
+	tv->tree.columnOptionTable, tv->core.tkwin);
+    Tk_InitOptions(
+	interp, (ClientData)(&tv->tree.column0),
+	tv->tree.headingOptionTable, tv->core.tkwin);
 
     Tcl_InitHashTable(&tv->tree.items, TCL_STRING_KEYS);
     tv->tree.serial = 0;
@@ -898,7 +916,11 @@ static int ConfigureColumn(
 	goto error;
     }
 
-    if (mask & GEOMETRY_CHANGED) {
+    /* Propagate column width changes to overall widget request width,
+     * but only if the widget is currently unmapped, in order to prevent
+     * geometry jumping during interactive column resize.
+     */
+    if (mask & GEOMETRY_CHANGED && !Tk_IsMapped(tv->core.tkwin)) {
 	TtkResizeWidget(&tv->core);
     }
     TtkRedisplayWidget(&tv->core);
@@ -1072,24 +1094,68 @@ static int IdentifyDisplayColumn(Treeview *tv, int x, int *x1)
     return -1;
 }
 
+
+/* + SubtreeHeight --
+ * 	Returns the height of the visible subtree rooted at item.
+ */
+#define ItemHeight(item) (ROWHEIGHT)	/* TBFIXED */
+static int SubtreeHeight(TreeItem *item)
+{
+    int height = ItemHeight(item);
+    if (item->state & TTK_STATE_OPEN) {
+	TreeItem *child = item->children;
+	while (child) {
+	    height += SubtreeHeight(child);
+	    child = child->next;
+	}
+    }
+    return height;
+}
+
+/* + ItemYPosition --
+ * 	Returns Y position of specified item relative to root of tree,
+ * 	-1 if item is not viewable.
+ */
+static int ItemYPosition(Treeview *tv, TreeItem *p)
+{
+    TreeItem *root = tv->tree.root;
+    int ypos = 0;
+
+    for (;;) {
+	if (p->prev) {
+	    p = p->prev;
+	    ypos += SubtreeHeight(p);
+	} else {
+	    p = p->parent;
+	    if (!(p && (p->state & TTK_STATE_OPEN))) {	
+		/* detached or closed ancestor */
+		return -1;
+	    }
+	    if (p == root) {
+		return ypos;
+	    }
+	    ypos += ItemHeight(p);
+	}
+    }
+}
+
 /*------------------------------------------------------------------------
  * +++ Display routines.
  */
 
-/* + GetSubLayout --
+/* + GetSublayout --
  * 	Utility routine; acquires a sublayout for items, cells, etc.
  */
-static Ttk_Layout GetSubLayout(
+static Ttk_Layout GetSublayout(
     Tcl_Interp *interp,
     Ttk_Theme themePtr,
+    Ttk_Layout parentLayout,
     const char *layoutName,
     Tk_OptionTable optionTable,
-    Tk_Window tkwin,
     Ttk_Layout *layoutPtr)
 {
-    void *recordPtr = 0;
-    Ttk_Layout newLayout = Ttk_CreateSubLayout(
-	    interp, themePtr, layoutName, recordPtr, optionTable, tkwin);
+    Ttk_Layout newLayout = Ttk_CreateSublayout(
+	    interp, themePtr, parentLayout, layoutName, optionTable);
 
     if (newLayout) {
 	if (*layoutPtr)
@@ -1106,22 +1172,22 @@ static Ttk_Layout TreeviewGetLayout(
     Tcl_Interp *interp, Ttk_Theme themePtr, void *recordPtr)
 {
     Treeview *tv = recordPtr;
-    Tk_Window tkwin = tv->core.tkwin;
+    Ttk_Layout treeLayout = WidgetGetLayout(interp, themePtr, recordPtr);
 
     if (!(
-	GetSubLayout(interp, themePtr, ITEM_LAYOUT,
-		tv->tree.itemOptionTable, tkwin, &tv->tree.itemLayout)
-     && GetSubLayout(interp, themePtr, CELL_LAYOUT,
-		tv->tree.cellOptionTable, tkwin, &tv->tree.cellLayout)
-     && GetSubLayout(interp, themePtr, HEADING_LAYOUT,
-	    tv->tree.headingOptionTable, tkwin, &tv->tree.headingLayout)
-     && GetSubLayout(interp, themePtr, ROW_LAYOUT,
-	    tv->tree.rowOptionTable, tkwin, &tv->tree.rowLayout)
+	GetSublayout(interp, themePtr, treeLayout, ".Item",
+		tv->tree.itemOptionTable, &tv->tree.itemLayout)
+     && GetSublayout(interp, themePtr, treeLayout, ".Cell",
+		tv->tree.cellOptionTable, &tv->tree.cellLayout)
+     && GetSublayout(interp, themePtr, treeLayout, ".Heading",
+	    tv->tree.headingOptionTable, &tv->tree.headingLayout)
+     && GetSublayout(interp, themePtr, treeLayout, ".Row",
+	    tv->tree.rowOptionTable, &tv->tree.rowLayout)
     )) {
 	return 0;
     }
 
-    return WidgetGetLayout(interp, themePtr, recordPtr);
+    return treeLayout;
 }
 
 /* + TreeviewDoLayout --
@@ -1453,6 +1519,20 @@ static int RowNumber(Treeview *tv, TreeItem *item)
     return -1;
 }
 
+/* + ItemDepth -- return the depth of a tree item.
+ * 	The depth of an item is equal to the number of proper ancestors,
+ * 	not counting the root node.  
+ */
+static int ItemDepth(TreeItem *item)
+{
+    int depth = 0;
+    while (item->parent) {
+	++depth;
+	item = item->parent;
+    }
+    return depth-1;
+}
+
 /*------------------------------------------------------------------------
  * +++ Widget commands -- item inquiry.
  */
@@ -1662,23 +1742,88 @@ static int TreeviewExistsCommand(
     return TCL_OK;
 }
 
-/* + $tv identify x y --
- * 	Identify the element or component at position x,y.
+/* + $tv bbox $itemid ?$column? --
+ * 	Return bounding box [x y width height] of specified item.
+ */
+static int TreeviewBBoxCommand(
+    Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
+{
+    Treeview *tv = recordPtr;
+    TreeItem *item = 0;
+    TreeColumn *column = 0;
+    int ypos;
+    Ttk_Box bbox;
+ 
+    if (objc < 3 || objc > 4) {
+	Tcl_WrongNumArgs(interp, 2, objv, "itemid ?column");
+	return TCL_ERROR;
+    }
+
+    item = FindItem(interp, tv, objv[2]);
+    if (!item) {
+	return TCL_ERROR;
+    }
+    if (objc >=4 && (column = FindColumn(interp,tv,objv[3])) == NULL) {
+	return TCL_ERROR;
+    }
+
+    /* Compute bounding box of item:
+     * ALTERNATE: (RowNumber(tv, item) - tv->tree.yscroll.first) * ROWHEIGHT;
+     */
+    ypos = ItemYPosition(tv, item) - (ROWHEIGHT * tv->tree.yscroll.first);
+    if (ypos < 0 || ypos > tv->tree.treeArea.height) { 
+	/* not viewable, or off-screen */
+	return TCL_OK;
+    }
+
+    bbox = tv->tree.treeArea;
+    bbox.y += ypos;
+    bbox.height = ROWHEIGHT;
+
+    /* If column has been specified, compute bounding box of cell
+     */
+    if (column) {
+	int xpos = 0, i = (tv->tree.showFlags & SHOW_TREE) ? 0 : 1;
+	while (i < tv->tree.nDisplayColumns) {
+	    if (tv->tree.displayColumns[i] == column) {
+		break;
+	    }
+	    xpos += tv->tree.displayColumns[i]->width;
+	    ++i;
+	}
+	if (i == tv->tree.nDisplayColumns) { /* specified column unviewable */
+	    return TCL_OK;
+	}
+	bbox.x += xpos;
+	bbox.width = column->width;
+
+	/* Special case for tree column -- account for indentation:
+	 * (@@@ NOTE: doesn't account for tree indicator or image;
+	 *  @@@ this may or may not be the right thing.)
+	 */
+	if (column == &tv->tree.column0) {
+	    int indent = INDENT * ItemDepth(item);
+	    bbox.x += indent;
+	    bbox.width -= indent;
+	}
+    }
+
+    Tcl_SetObjResult(interp, Ttk_NewBoxObj(bbox));
+    return TCL_OK;
+}
+
+/* + $tv identify $x $y -- (obsolescent)
+ * 	Implements the old, horrible, 2-argument form of [$tv identify].
  *
  * Returns: one of
  * 	heading #n
  * 	cell itemid #n
  * 	item itemid element
  * 	row itemid
- * 	nothing
- *
- * Bugs:
- * 	This is a hairball.
  */
-static int TreeviewIdentifyCommand(
-    Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
+static int TreeviewHorribleIdentify(
+    Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], Treeview *tv)
 {
-    Treeview *tv = recordPtr;
     const char *what = "nothing", *detail = NULL;
     TreeItem *item = 0;
     Tcl_Obj *result;
@@ -1686,10 +1831,7 @@ static int TreeviewIdentifyCommand(
     char dcolbuf[16];
     int x, y, x1;
 
-    if (objc != 4) {
-	Tcl_WrongNumArgs(interp, 2, objv, "x y");
-	return TCL_ERROR;
-    }
+    /* ASSERT: objc == 4 */
 
     if (   Tcl_GetIntFromObj(interp, objv[2], &x) != TCL_OK
 	|| Tcl_GetIntFromObj(interp, objv[3], &y) != TCL_OK
@@ -1744,6 +1886,63 @@ done:
     return TCL_OK;
 }
 
+/* + $tv identify $component $x $y --
+ * 	Identify the component at position x,y.
+ */
+
+static int TreeviewIdentifyCommand(
+    Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
+{
+    static const char *componentStrings[] =
+	{ "row", "column", NULL };
+    enum { I_ROW, I_COLUMN };
+
+    Treeview *tv = recordPtr;
+    int component, x, y;
+
+    if (objc == 4) {	/* Old form */
+	return TreeviewHorribleIdentify(interp, objc, objv, tv);
+    } else if (objc != 5) {
+	Tcl_WrongNumArgs(interp, 2, objv, "component x y");
+	return TCL_ERROR;
+    }
+
+    if (   Tcl_GetIndexFromObj(interp, objv[2], 
+		componentStrings, "componen", TCL_EXACT, &component) != TCL_OK
+        || Tcl_GetIntFromObj(interp, objv[3], &x) != TCL_OK
+	|| Tcl_GetIntFromObj(interp, objv[4], &y) != TCL_OK
+    ) {
+	return TCL_ERROR;
+    }
+
+    switch (component) 
+    {
+	case I_ROW :
+	{
+	    Ttk_Box itemBox;
+	    TreeItem *item = IdentifyItem(tv, y, &itemBox);
+	    if (item) {
+		Tcl_SetObjResult(interp, ItemID(tv, item));
+	    }
+	    break;
+	}
+
+	case I_COLUMN :
+	{
+	    int x1;
+	    int column = IdentifyDisplayColumn(tv, x, &x1);
+
+	    if (column >= 0) {
+		char dcolbuf[16];
+		sprintf(dcolbuf, "#%d", column);
+		Tcl_SetObjResult(interp, Tcl_NewStringObj(dcolbuf, -1));
+	    }
+	    break;
+	}
+    }
+    return TCL_OK;
+}
+
 /*------------------------------------------------------------------------
  * +++ Widget commands -- item and column configuration.
  */
@@ -2194,14 +2393,12 @@ static int TreeviewSeeCommand(
      */
     rowNumber = RowNumber(tv, item);
     if (rowNumber < tv->tree.yscroll.first) {
-	tv->tree.yscroll.first = rowNumber;
+	ScrollTo(tv->tree.yscrollHandle, rowNumber);
     } else if (rowNumber >= tv->tree.yscroll.last) {
-	tv->tree.yscroll.first += (1+rowNumber - tv->tree.yscroll.last);
-    } else {
-	return TCL_OK;
+	ScrollTo(tv->tree.yscrollHandle,
+	    tv->tree.yscroll.first + (1+rowNumber - tv->tree.yscroll.last));
     }
 
-    TtkRedisplayWidget(&tv->core);
     return TCL_OK;
 }
 
@@ -2307,14 +2504,12 @@ static int TreeviewSelectionCommand(
     return TCL_OK;
 }
 
-
-
 /*------------------------------------------------------------------------
  * +++ Widget commands record.
  */
-
 static WidgetCommandSpec TreeviewCommands[] =
 {
+    { "bbox",  		TreeviewBBoxCommand },
     { "children",	TreeviewChildrenCommand },
     { "cget",		WidgetCgetCommand },
     { "column", 	TreeviewColumnCommand },
@@ -2388,7 +2583,7 @@ TTK_BEGIN_LAYOUT(HeadingLayout)
     TTK_NODE("Treeheading.cell", TTK_FILL_BOTH)
     TTK_GROUP("Treeheading.border", TTK_FILL_BOTH,
 	TTK_NODE("Treeheading.image", TTK_PACK_RIGHT)
-	TTK_NODE("Treeheading.text", 0))
+	TTK_NODE("Treeheading.text", TTK_FILL_X))
 TTK_END_LAYOUT
 
 TTK_BEGIN_LAYOUT(RowLayout)
@@ -2526,10 +2721,10 @@ DLLEXPORT int Treeview_Init(Tcl_Interp *interp)
     Ttk_RegisterElement(interp, theme, "Treeheading.cell", &RowElementSpec, 0);
 
     Ttk_RegisterLayout(theme, TreeviewWidgetSpec.className, TreeviewLayout);
-    Ttk_RegisterLayout(theme, ITEM_LAYOUT, ItemLayout);
-    Ttk_RegisterLayout(theme, CELL_LAYOUT, CellLayout);
-    Ttk_RegisterLayout(theme, HEADING_LAYOUT, HeadingLayout);
-    Ttk_RegisterLayout(theme, ROW_LAYOUT, RowLayout);
+    Ttk_RegisterLayout(theme, "Item", ItemLayout);
+    Ttk_RegisterLayout(theme, "Cell", CellLayout);
+    Ttk_RegisterLayout(theme, "Heading", HeadingLayout);
+    Ttk_RegisterLayout(theme, "Row", RowLayout);
 
     Tcl_PkgProvide(interp, "treeview", TILE_VERSION);
 
