@@ -1,4 +1,4 @@
-/* widget.h,v 1.47 2005/10/08 14:02:47 jenglish Exp
+/* widget.h,v 1.49 2006/04/22 20:43:00 jenglish Exp
  * Copyright (c) 2003, Joe English
  *
  * Helper routines for widget implementations.
@@ -207,6 +207,7 @@ extern void FreeScrollHandle(ScrollHandle);
 extern int ScrollviewCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], ScrollHandle);
 
+extern void ScrollTo(ScrollHandle, int newFirst);
 extern void Scrolled(ScrollHandle, int first, int last, int total);
 extern void ScrollbarUpdateRequired(ScrollHandle);
 
@@ -223,6 +224,11 @@ extern const char *TTKOrientStrings[];
 extern const char *TTKCompoundStrings[];
 extern const char *TTKDefaultStrings[];
 
+/* 
+ * ... other option types...
+ */
+extern int TtkGetLabelAnchorFromObj(Tcl_Interp*,Tcl_Obj*,Ttk_PositionSpec *);
+
 /*
  * Package initialiation routines:
  */
