@@ -1,5 +1,5 @@
 #
-# altTheme.tcl,v 1.30 2005/09/29 15:34:54 jenglish Exp
+# altTheme.tcl,v 1.34 2006/04/16 18:45:24 jenglish Exp
 #
 # Tile widget set: Alternate theme
 #
@@ -41,8 +41,8 @@ namespace eval tile::theme::alt {
 	    {active !disabled} 	raised
 	} -highlightcolor {alternate black}
 
-	style configure TCheckbutton -indicatorcolor "#ffffff"
-	style configure TRadiobutton -indicatorcolor "#ffffff"
+	style configure TCheckbutton -indicatorcolor "#ffffff" -padding 2
+	style configure TRadiobutton -indicatorcolor "#ffffff" -padding 2
 	style map TCheckbutton -indicatorcolor \
 	    [list  disabled $colors(-frame)  pressed $colors(-frame)]
 	style map TRadiobutton -indicatorcolor \
@@ -50,9 +50,10 @@ namespace eval tile::theme::alt {
 
 	style configure TMenubutton -width -11 -padding "3 3" -relief raised
 
-	style configure TEntry -relief sunken -borderwidth 1 -padding 1
+	style configure TEntry -padding 1
 	style map TEntry -fieldbackground \
 		[list readonly $colors(-frame) disabled $colors(-frame)]
+	style configure TCombobox -padding 1
 	style map TCombobox -fieldbackground \
 		[list readonly $colors(-frame) disabled $colors(-frame)]
 
@@ -66,13 +67,17 @@ namespace eval tile::theme::alt {
 
 	style configure TLabelframe -relief groove -borderwidth 2
 
-	style configure TNotebook -expandtab {2 2 1 0}
+	style configure TNotebook -tabmargins {2 2 1 0}
 	style configure TNotebook.Tab \
 	    -padding {4 2} -background $colors(-darker)
-	style map TNotebook.Tab -background [list selected $colors(-frame)]
+	style map TNotebook.Tab \
+	    -background [list selected $colors(-frame)] \
+	    -expand [list selected {2 2 1 0}] \
+	    ;
 
-	style configure TScale -groovewidth 4 -troughrelief sunken
-	style map TScale -sliderrelief {pressed sunken  {} raised}
+	style configure TScale \
+	    -groovewidth 4 -troughrelief sunken \
+	    -sliderwidth raised -borderwidth 2
 	style configure TProgressbar \
 	    -background $colors(-selectbg) -borderwidth 0
     }
