@@ -1,5 +1,5 @@
 #
-# aquaTheme.tcl,v 1.14 2005/09/29 15:34:54 jenglish Exp
+# aquaTheme.tcl,v 1.16 2006/04/22 20:43:00 jenglish Exp
 #
 # Tile widget set: Aqua theme (OSX native look and feel)
 #
@@ -34,6 +34,8 @@ namespace eval tile {
 
 	style configure TButton -padding {0 2} -width -6
 	style configure Toolbutton -padding 4
+	# See Apple HIG figs 14-63, 14-65
+	style configure TNotebook -tabposition n -padding {20 12}
 	style configure TNotebook.Tab -padding {10 2 10 2}
 
 	# Enable animation for ttk::progressbar widget:
@@ -58,10 +60,10 @@ namespace eval tile {
 	}
 	#
 	# For Aqua, labelframe labels should appear outside the border,
-	# with a 14 pixel inset and 4 pixels spacing 
+	# with a 14 pixel inset and 4 pixels spacing between border and label
 	# (ref: Apple Human Interface Guidelines / Controls / Grouping Controls)
 	#
     	style configure TLabelframe \
-		-labeloutside true -labelinset 14 -labelspace 4
+		-labeloutside true -labelmargins {14 0 14 4}
     }
 }
