@@ -1,5 +1,5 @@
 #
-# button.tcl,v 1.9 2005/06/26 22:07:39 jenglish Exp
+# button.tcl,v 1.10 2006/01/16 15:29:47 jenglish Exp
 #
 # Bindings for Buttons, Checkbuttons, and Radiobuttons.
 #
@@ -18,12 +18,12 @@
 # the pressed state if the button is currently active.
 #
 
-namespace eval tile { namespace eval button { } }
+namespace eval ttk { namespace eval button { } }
 
 bind TButton <Enter>		{ %W state active }
 bind TButton <Leave>		{ %W state !active }
-bind TButton <Key-space>	{ tile::button::activate %W }
-bind TButton <<Invoke>> 	{ tile::button::activate %W }
+bind TButton <Key-space>	{ ttk::button::activate %W }
+bind TButton <<Invoke>> 	{ ttk::button::activate %W }
 
 bind TButton <ButtonPress-1> \
     { %W instate !disabled { tile::clickToFocus %W; %W state pressed } }
@@ -41,8 +41,8 @@ tile::CopyBindings TButton TRadiobutton
 
 # ...plus a few more:
 
-bind TRadiobutton <KeyPress-Up> 	{ tile::button::RadioTraverse %W -1 }
-bind TRadiobutton <KeyPress-Down> 	{ tile::button::RadioTraverse %W +1 }
+bind TRadiobutton <KeyPress-Up> 	{ ttk::button::RadioTraverse %W -1 }
+bind TRadiobutton <KeyPress-Down> 	{ ttk::button::RadioTraverse %W +1 }
 
 # bind TCheckbutton <KeyPress-plus> { %W select }
 # bind TCheckbutton <KeyPress-minus> { %W deselect }
@@ -51,7 +51,7 @@ bind TRadiobutton <KeyPress-Down> 	{ tile::button::RadioTraverse %W +1 }
 #	Simulate a button press: temporarily set the state to 'pressed',
 #	then invoke the button.
 #
-proc tile::button::activate {w} {
+proc ttk::button::activate {w} {
     set oldState [$w state pressed]
     update idletasks; after 100
     $w state $oldState
@@ -64,7 +64,7 @@ proc tile::button::activate {w} {
 #	the same parent and -variable; this is a pretty good heuristic
 #	that works most of the time.
 #
-proc tile::button::RadioTraverse {w dir} {
+proc ttk::button::RadioTraverse {w dir} {
     set group [list]
     foreach sibling [winfo children [winfo parent $w]] {
     	if {   [winfo class $sibling] eq "TRadiobutton"
