@@ -1,5 +1,5 @@
 #
-# combobox.tcl,v 1.28 2005/09/29 02:32:06 jenglish Exp
+# combobox.tcl,v 1.32 2006/01/16 15:29:47 jenglish Exp
 #
 # Tile widget set: combobox bindings.
 #
@@ -9,7 +9,7 @@
 # combobox values with the listbox values.
 #
 
-namespace eval tile::combobox {
+namespace eval ttk::combobox {
     variable Values	;# Values($cb) is -listvariable of listbox widget
 
     variable State
@@ -23,32 +23,32 @@ namespace eval tile::combobox {
 
 tile::CopyBindings TEntry TCombobox
 
-bind TCombobox <KeyPress-Down> 		{ tile::combobox::Post %W }
-bind TCombobox <KeyPress-Escape> 	{ tile::combobox::Unpost %W }
+bind TCombobox <KeyPress-Down> 		{ ttk::combobox::Post %W }
+bind TCombobox <KeyPress-Escape> 	{ ttk::combobox::Unpost %W }
 
-bind TCombobox <ButtonPress-1> 		{ tile::combobox::Press "" %W %x %y }
-bind TCombobox <Shift-ButtonPress-1>	{ tile::combobox::Press "s" %W %x %y }
-bind TCombobox <Double-ButtonPress-1> 	{ tile::combobox::Press "2" %W %x %y }
-bind TCombobox <Triple-ButtonPress-1> 	{ tile::combobox::Press "3" %W %x %y }
-bind TCombobox <B1-Motion>		{ tile::combobox::Drag %W %x }
+bind TCombobox <ButtonPress-1> 		{ ttk::combobox::Press "" %W %x %y }
+bind TCombobox <Shift-ButtonPress-1>	{ ttk::combobox::Press "s" %W %x %y }
+bind TCombobox <Double-ButtonPress-1> 	{ ttk::combobox::Press "2" %W %x %y }
+bind TCombobox <Triple-ButtonPress-1> 	{ ttk::combobox::Press "3" %W %x %y }
+bind TCombobox <B1-Motion>		{ ttk::combobox::Drag %W %x }
 
-bind TCombobox <MouseWheel> 	{ tile::combobox::Scroll %W [expr {%D/-120}] }
+bind TCombobox <MouseWheel> 	{ ttk::combobox::Scroll %W [expr {%D/-120}] }
 if {[tk windowingsystem] eq "x11"} {
-    bind TCombobox <ButtonPress-4>	{ tile::combobox::Scroll %W -1 }
-    bind TCombobox <ButtonPress-5>	{ tile::combobox::Scroll %W  1 }
+    bind TCombobox <ButtonPress-4>	{ ttk::combobox::Scroll %W -1 }
+    bind TCombobox <ButtonPress-5>	{ ttk::combobox::Scroll %W  1 }
 }
 
-bind TCombobox <<TraverseIn>> 		{ tile::combobox::TraverseIn %W }
+bind TCombobox <<TraverseIn>> 		{ ttk::combobox::TraverseIn %W }
 
 ### Combobox listbox bindings.
 #
 bind ComboboxListbox <ButtonPress-1> 	{ focus %W ; continue }
-bind ComboboxListbox <ButtonRelease-1>	{ tile::combobox::LBSelected %W }
-bind ComboboxListbox <KeyPress-Return>	{ tile::combobox::LBSelected %W }
-bind ComboboxListbox <KeyPress-Escape>  { tile::combobox::LBCancel %W }
-bind ComboboxListbox <KeyPress-Tab>	{ tile::combobox::LBTab %W next }
-bind ComboboxListbox <<PrevWindow>>	{ tile::combobox::LBTab %W prev }
-bind ComboboxListbox <Destroy>		{ tile::combobox::LBCleanup %W }
+bind ComboboxListbox <ButtonRelease-1>	{ ttk::combobox::LBSelected %W }
+bind ComboboxListbox <KeyPress-Return>	{ ttk::combobox::LBSelected %W }
+bind ComboboxListbox <KeyPress-Escape>  { ttk::combobox::LBCancel %W }
+bind ComboboxListbox <KeyPress-Tab>	{ ttk::combobox::LBTab %W next }
+bind ComboboxListbox <<PrevWindow>>	{ ttk::combobox::LBTab %W prev }
+bind ComboboxListbox <Destroy>		{ ttk::combobox::LBCleanup %W }
 # Default behavior is to follow selection on mouseover
 bind ComboboxListbox <Motion> {
     %W selection clear 0 end
@@ -57,11 +57,25 @@ bind ComboboxListbox <Motion> {
 }
 
 # The combobox has a global grab active when the listbox is posted,
-# but on Windows that doesn't prevent the user from interacting
-# with other applications. The listbox gets a <FocusOut> event
-# when this happens.  Don't know how reliable this is:
+# but on Windows and OSX that doesn't prevent the user from interacting
+# with other applications.  We need to popdown the listbox when this happens.
 #
-bind ComboboxListbox <FocusOut>		{ tile::combobox::LBCancel %W }
+# On OSX, the listbox gets a <Deactivate> event.  This doesn't happen 
+# on Windows or X11, but it does get a <FocusOut> event.  However on OSX
+# in Tk 8.5, the listbox gets spurious <FocusOut> events when the listbox 
+# is posted (see #1349811).
+#
+# The following seems to work:
+#
+
+switch -- [tk windowingsystem] {
+    win32 {
+	bind ComboboxListbox <FocusOut>		{ ttk::combobox::LBCancel %W }
+    }
+    aqua {
+	bind ComboboxListbox <Deactivate>	{ ttk::combobox::LBCancel %W }
+    }
+}
 
 ### Option database settings.
 #
@@ -83,7 +97,7 @@ option add *TCombobox*Listbox.font TkTextFont
 #	Either post/unpost the listbox, or perform Entry widget binding,
 #	depending on widget state and location of button press.
 #
-proc tile::combobox::Press {mode w x y} {
+proc ttk::combobox::Press {mode w x y} {
     variable State
     set State(entryPress) [expr {
 	   [$w instate {!readonly !disabled}]
@@ -93,11 +107,11 @@ proc tile::combobox::Press {mode w x y} {
     if {$State(entryPress)} {
 	focus $w
 	switch -- $mode {
-	    s 	{ tile::entry::Shift-Press $w $x 	; # Shift }
-	    2	{ tile::entry::Select $w $x word 	; # Double click}
-	    3	{ tile::entry::Select $w $x line 	; # Triple click }
+	    s 	{ ttk::entry::Shift-Press $w $x 	; # Shift }
+	    2	{ ttk::entry::Select $w $x word 	; # Double click}
+	    3	{ ttk::entry::Select $w $x line 	; # Triple click }
 	    ""	-
-	    default { tile::entry::Press $w $x }
+	    default { ttk::entry::Press $w $x }
 	}
     } else {
 	TogglePost $w
@@ -109,17 +123,17 @@ proc tile::combobox::Press {mode w x y} {
 #	If the initial ButtonPress event was handled by Entry binding,
 #	perform Entry widget drag binding; otherwise nothing.
 #
-proc tile::combobox::Drag {w x}  {
+proc ttk::combobox::Drag {w x}  {
     variable State
     if {$State(entryPress)} {
-	tile::entry::Drag $w $x
+	ttk::entry::Drag $w $x
     }
 }
 
 ## TraverseIn -- receive focus due to keyboard navigation
 #	For editable comboboxes, set the selection and insert cursor.
 #
-proc tile::combobox::TraverseIn {w} {
+proc ttk::combobox::TraverseIn {w} {
     $w instate {!readonly !disabled} { 
 	$w selection range 0 end
 	$w icursor end
@@ -129,7 +143,7 @@ proc tile::combobox::TraverseIn {w} {
 ## SelectEntry $cb $index -- 
 #	Set the combobox selection in response to a user action.
 #
-proc tile::combobox::SelectEntry {cb index} {
+proc ttk::combobox::SelectEntry {cb index} {
     $cb current $index
     $cb selection range 0 end
     $cb icursor end
@@ -138,7 +152,7 @@ proc tile::combobox::SelectEntry {cb index} {
 
 ## Scroll -- Mousewheel binding
 #
-proc tile::combobox::Scroll {cb dir} {
+proc ttk::combobox::Scroll {cb dir} {
     $cb instate disabled { return }
     set max [llength [$cb cget -values]]
     set current [$cb current]
@@ -152,7 +166,7 @@ proc tile::combobox::Scroll {cb dir} {
 #	Set the combobox value to the currently-selected listbox value
 #	and unpost the listbox.
 #
-proc tile::combobox::LBSelected {lb} {
+proc ttk::combobox::LBSelected {lb} {
     set cb [LBMaster $lb]
     set selection [$lb curselection]
     Unpost $cb
@@ -165,7 +179,7 @@ proc tile::combobox::LBSelected {lb} {
 ## LBCancel --
 #	Unpost the listbox.
 #
-proc tile::combobox::LBCancel {lb} {
+proc ttk::combobox::LBCancel {lb} {
     Unpost [LBMaster $lb]
 }
 
@@ -173,7 +187,7 @@ proc tile::combobox::LBCancel {lb} {
 #	Tab key binding for combobox listbox:  
 #	Set the selection, and navigate to next/prev widget.
 #
-proc tile::combobox::LBTab {lb dir} {
+proc ttk::combobox::LBTab {lb dir} {
     set cb [LBMaster $lb]
     switch -- $dir {
 	next	{ set newFocus [tk_focusNext $cb] }
@@ -193,7 +207,7 @@ proc tile::combobox::LBTab {lb dir} {
 #	Returns the popdown shell widget associated with a combobox,
 #	creating it if necessary.
 #
-proc tile::combobox::PopdownShell {cb} {
+proc ttk::combobox::PopdownShell {cb} {
     if {![winfo exists $cb.popdown]} {
 	set popdown [toplevel $cb.popdown -relief solid -bd 1]
 	wm withdraw $popdown
@@ -210,7 +224,7 @@ proc tile::combobox::PopdownShell {cb} {
 		-command [list $popdown.l yview]
 	}
 	listbox $popdown.l \
-	    -listvariable tile::combobox::Values($cb) \
+	    -listvariable ttk::combobox::Values($cb) \
 	    -yscrollcommand [list $popdown.sb set] \
 	    -exportselection false \
 	    -selectmode browse \
@@ -232,7 +246,7 @@ proc tile::combobox::PopdownShell {cb} {
 ## combobox::Post $cb --
 #	Pop down the associated listbox.
 #
-proc tile::combobox::Post {cb} {
+proc ttk::combobox::Post {cb} {
     variable State
     variable Values
 
@@ -306,7 +320,7 @@ proc tile::combobox::Post {cb} {
 ## combobox::Unpost $cb --
 #	Unpost the listbox, restore focus to combobox widget.
 #
-proc tile::combobox::Unpost {cb} {
+proc ttk::combobox::Unpost {cb} {
     $cb state !pressed
     tile::releaseGrab $cb
     if {[winfo exists $cb.popdown]} {
@@ -318,14 +332,14 @@ proc tile::combobox::Unpost {cb} {
 ## combobox::TogglePost $cb --
 #	Post the listbox if unposted, unpost otherwise.
 #
-proc tile::combobox::TogglePost {cb} {
+proc ttk::combobox::TogglePost {cb} {
     if {[$cb instate pressed]} { Unpost $cb } { Post $cb }
 }
 
 ## LBMaster $lb --
 #	Return the combobox main widget that owns the listbox.
 #
-proc tile::combobox::LBMaster {lb} {
+proc ttk::combobox::LBMaster {lb} {
     winfo parent [winfo parent $lb]
 }
 
@@ -338,7 +352,7 @@ proc tile::combobox::LBMaster {lb} {
 #	[winfo parent] still works, fortunately.
 #
 
-proc tile::combobox::LBCleanup {lb} {
+proc ttk::combobox::LBCleanup {lb} {
     variable Values
     unset Values([LBMaster $lb])
 }
