@@ -1,5 +1,5 @@
 #
-# defaults.tcl,v 1.28 2005/09/29 15:34:54 jenglish Exp
+# defaults.tcl,v 1.30 2006/02/21 15:45:47 jenglish Exp
 #
 # Tile widget set: Default theme
 #
@@ -30,7 +30,7 @@ namespace eval tile {
 	    -selectborderwidth	1 \
 	    -selectbackground	$colors(-selectbg) \
 	    -selectforeground	$colors(-selectfg) \
-	    -insertwidth 	2 \
+	    -insertwidth 	1 \
 	    -indicatordiameter	10 \
 	    ;
 
@@ -44,12 +44,12 @@ namespace eval tile {
 	style map TButton -relief [list {!disabled pressed} sunken] 
 
 	style configure TCheckbutton \
-	    -indicatorcolor "#ffffff" -indicatorrelief sunken 
+	    -indicatorcolor "#ffffff" -indicatorrelief sunken -padding 1
 	style map TCheckbutton -indicatorcolor \
 	    [list pressed $colors(-activebg)  selected $colors(-indicator)]
 
 	style configure TRadiobutton \
-	    -indicatorcolor "#ffffff" -indicatorrelief sunken 
+	    -indicatorcolor "#ffffff" -indicatorrelief sunken -padding 1
 	style map TRadiobutton -indicatorcolor \
 	    [list pressed $colors(-activebg)  selected $colors(-indicator)]
 
@@ -59,9 +59,9 @@ namespace eval tile {
 	style map TEntry -fieldbackground \
 	    [list readonly $colors(-frame) disabled $colors(-frame)]
 
-	style configure TCombobox -arrowsize 12
+	style configure TCombobox -arrowsize 12 -padding 1
 	style map TCombobox -fieldbackground \
-		[list readonly $colors(-frame) disabled $colors(-frame)]
+	    [list readonly $colors(-frame) disabled $colors(-frame)]
 
 	style configure TLabelframe -relief groove -borderwidth 2
 
