@@ -2,24 +2,25 @@
 #
 # Bindings for the TScale widget
 #
-# scale.tcl,v 1.8 2004/08/25 23:58:11 hobbs Exp
+# scale.tcl,v 1.10 2006/01/16 15:29:47 jenglish Exp
 
-namespace eval tile::scale {
+namespace eval ttk::scale {
     variable State
     array set State  {
 	dragging 0
     }
 }
 
-bind TScale <ButtonPress-1>   { tile::scale::Press %W %x %y }
-bind TScale <B1-Motion>       { tile::scale::Drag %W %x %y }
-bind TScale <ButtonRelease-1> { tile::scale::Release %W %x %y }
+bind TScale <ButtonPress-1>   { ttk::scale::Press %W %x %y }
+bind TScale <B1-Motion>       { ttk::scale::Drag %W %x %y }
+bind TScale <ButtonRelease-1> { ttk::scale::Release %W %x %y }
 
-proc tile::scale::Press {w x y} {
+proc ttk::scale::Press {w x y} {
     variable State
     set State(dragging) 0
 
     switch -glob -- [$w identify $x $y] {
+	*track -
         *trough {
 	    if {[$w get $x $y] <= [$w get]} {
 		tile::Repeatedly Increment $w -1
@@ -34,20 +35,20 @@ proc tile::scale::Press {w x y} {
     }
 }
 
-proc tile::scale::Drag {w x y} {
+proc ttk::scale::Drag {w x y} {
     variable State
     if {$State(dragging)} {
 	$w set [$w get $x $y]
     }
 }
 
-proc tile::scale::Release {w x y} {
+proc ttk::scale::Release {w x y} {
     variable State
     set State(dragging) 0
     tile::CancelRepeat
 }
 
-proc tile::scale::Increment {w delta} {
+proc ttk::scale::Increment {w delta} {
     if {![winfo exists $w]} return
     $w set [expr {[$w get] + $delta}]
 }
