@@ -1,10 +1,10 @@
 #
-# treeview.tcl,v 1.14 2005/09/14 17:24:28 jenglish Exp
+# treeview.tcl,v 1.17 2006/03/13 15:50:15 jenglish Exp
 #
 # Tile widget set -- bindings for Treeview widget.
 #
 
-namespace eval ::tile::treeview {
+namespace eval ttk::treeview {
     variable State
 
     # Enter/Leave/Motion
@@ -26,6 +26,7 @@ namespace eval ::tile::treeview {
     set State(heading)  	{}
 
     # Provide [lassign] if not already present
+    # (@@@ TODO: check if this is still needed after horrible-identify purge)
     #
     if {![llength [info commands lassign]]} {
 	proc lassign {vals args} {
@@ -37,26 +38,26 @@ namespace eval ::tile::treeview {
 ### Widget bindings.
 #
 
-bind Treeview	<Motion> 		{ tile::treeview::Motion %W %x %y }
+bind Treeview	<Motion> 		{ ttk::treeview::Motion %W %x %y }
 bind Treeview	<B1-Leave>		{ #nothing }
-bind Treeview	<Leave>			{ tile::treeview::ActivateHeading {} {}}
-bind Treeview	<ButtonPress-1> 	{ tile::treeview::Press %W %x %y }
-bind Treeview	<Double-ButtonPress-1> 	{ tile::treeview::DoubleClick %W %x %y }
-bind Treeview	<ButtonRelease-1> 	{ tile::treeview::Release %W %x %y }
-bind Treeview	<B1-Motion> 		{ tile::treeview::Drag %W %x %y }
-bind Treeview 	<KeyPress-Up>    	{ tile::treeview::Keynav %W up }
-bind Treeview 	<KeyPress-Down>  	{ tile::treeview::Keynav %W down }
-bind Treeview 	<KeyPress-Right> 	{ tile::treeview::Keynav %W right }
-bind Treeview 	<KeyPress-Left>  	{ tile::treeview::Keynav %W left }
+bind Treeview	<Leave>			{ ttk::treeview::ActivateHeading {} {}}
+bind Treeview	<ButtonPress-1> 	{ ttk::treeview::Press %W %x %y }
+bind Treeview	<Double-ButtonPress-1> 	{ ttk::treeview::DoubleClick %W %x %y }
+bind Treeview	<ButtonRelease-1> 	{ ttk::treeview::Release %W %x %y }
+bind Treeview	<B1-Motion> 		{ ttk::treeview::Drag %W %x %y }
+bind Treeview 	<KeyPress-Up>    	{ ttk::treeview::Keynav %W up }
+bind Treeview 	<KeyPress-Down>  	{ ttk::treeview::Keynav %W down }
+bind Treeview 	<KeyPress-Right> 	{ ttk::treeview::Keynav %W right }
+bind Treeview 	<KeyPress-Left>  	{ ttk::treeview::Keynav %W left }
 bind Treeview	<KeyPress-Prior>	{ %W yview scroll -1 pages }
 bind Treeview	<KeyPress-Next> 	{ %W yview scroll  1 pages }
-bind Treeview	<KeyPress-Return>	{ tile::treeview::ToggleFocus %W }
-bind Treeview	<KeyPress-space>	{ tile::treeview::ToggleFocus %W }
+bind Treeview	<KeyPress-Return>	{ ttk::treeview::ToggleFocus %W }
+bind Treeview	<KeyPress-space>	{ ttk::treeview::ToggleFocus %W }
 
 bind Treeview	<Shift-ButtonPress-1> \
-		{ tile::treeview::Select %W %x %y add }
+		{ ttk::treeview::Select %W %x %y add }
 bind Treeview	<Control-ButtonPress-1> \
-		{ tile::treeview::Select %W %x %y toggle }
+		{ ttk::treeview::Select %W %x %y toggle }
 
 # Standard mousewheel bindings:
 #
@@ -73,7 +74,7 @@ if {[string equal "x11" [tk windowingsystem]]} {
 #
 # @@@ TODO: verify/rewrite up and down code.
 #
-proc tile::treeview::Keynav {w dir} {
+proc ttk::treeview::Keynav {w dir} {
     set focus [$w focus]
     if {$focus eq ""} { return }
 
@@ -119,7 +120,7 @@ proc tile::treeview::Keynav {w dir} {
 ## Motion -- pointer motion binding.
 #	Sets cursor, active element ...
 #
-proc tile::treeview::Motion {w x y} {
+proc ttk::treeview::Motion {w x y} {
     variable ::tile::Cursors
     variable State
 
@@ -140,7 +141,7 @@ proc tile::treeview::Motion {w x y} {
 
 ## ActivateHeading -- track active heading element
 #
-proc tile::treeview::ActivateHeading {w heading} {
+proc ttk::treeview::ActivateHeading {w heading} {
     variable State
 
     if {$w != $State(activeWidget) || $heading != $State(activeHeading)} {
@@ -157,28 +158,26 @@ proc tile::treeview::ActivateHeading {w heading} {
 
 ## Select -- adjust selection
 #
-proc tile::treeview::Select {w x y op} {
-    lassign [$w identify $x $y] what where detail
-    switch -- $what {
-	cell -
-	row  -
-	item { $w selection $op $where }
+proc ttk::treeview::Select {w x y op} {
+    set row [$w identify row $x $y] 
+    if {$row ne "" } {
+	$w selection $op [list $row]
     }
 }
 
 ## DoubleClick -- Double-ButtonPress-1 binding.  
 #
-proc tile::treeview::DoubleClick {w x y} {
-    lassign [$w identify $x $y] what where detail
-    switch -- $what {
-	item	{ Toggle $w $where }
-	default	{ Press $w $x $y ;# perform single-click action }
+proc ttk::treeview::DoubleClick {w x y} {
+    if {[set row [$w identify row $x $y]] ne ""} {
+	Toggle $w $row
+    } else {
+	Press $w $x $y ;# perform single-click action
     }
 }
 
 ## Press -- ButtonPress binding.
 #
-proc tile::treeview::Press {w x y} {
+proc ttk::treeview::Press {w x y} {
 
     lassign [$w identify $x $y] what where detail
 
@@ -198,7 +197,7 @@ proc tile::treeview::Press {w x y} {
 
 ## Drag -- B1-Motion binding
 #
-proc tile::treeview::Drag {w x y} {
+proc ttk::treeview::Drag {w x y} {
     variable State
     switch $State(pressMode) {
 	resize	{ resize.drag $w $x }
@@ -206,7 +205,7 @@ proc tile::treeview::Drag {w x y} {
     }
 }
 
-proc tile::treeview::Release {w x y} {
+proc ttk::treeview::Release {w x y} {
     variable State
     switch $State(pressMode) {
 	resize	{ resize.release $w $x }
@@ -220,7 +219,7 @@ proc tile::treeview::Release {w x y} {
 #
 # @@@ needs work.
 #
-proc tile::treeview::resize.press {w x column} {
+proc ttk::treeview::resize.press {w x column} {
     variable State
 
     set State(pressMode) "resize"
@@ -229,7 +228,7 @@ proc tile::treeview::resize.press {w x column} {
     set State(resizeWidth) [$w column $column -width]
 }
 
-proc tile::treeview::resize.drag {w x} {
+proc ttk::treeview::resize.drag {w x} {
     variable State
     set newWidth [expr {$State(resizeWidth) + $x - $State(pressX)}]
     if {$newWidth < $State(minWidth)} {
@@ -238,21 +237,21 @@ proc tile::treeview::resize.drag {w x} {
     $w column $State(resizeColumn) -width $newWidth
 }
 
-proc tile::treeview::resize.release {w x} {
+proc ttk::treeview::resize.release {w x} {
     # no-op
 }
 
 ### Heading activation.
 #
 
-proc tile::treeview::heading.press {w column} {
+proc ttk::treeview::heading.press {w column} {
     variable State
     set State(pressMode) "heading"
     set State(heading) $column
     $w heading $column state pressed
 }
 
-proc tile::treeview::heading.drag {w x y} {
+proc ttk::treeview::heading.drag {w x y} {
     variable State
     lassign [$w identify $x $y] what where detail
     if {$what eq "heading" && $where eq $State(heading)} {
@@ -262,7 +261,7 @@ proc tile::treeview::heading.drag {w x y} {
     }
 }
 
-proc tile::treeview::heading.release {w} {
+proc ttk::treeview::heading.release {w} {
     variable State
     if {[lsearch -exact [$w heading $State(heading) state] pressed] >= 0} {
 	after idle [$w heading $State(heading) -command]
@@ -276,13 +275,13 @@ proc tile::treeview::heading.release {w} {
 ## OpenItem, CloseItem -- Set the open state of an item, generate event
 #
 
-proc tile::treeview::OpenItem {w item} {
+proc ttk::treeview::OpenItem {w item} {
     $w focus $item
     event generate $w <<TreeviewOpen>>
     $w item $item -open true
 }
 
-proc tile::treeview::CloseItem {w item} {
+proc ttk::treeview::CloseItem {w item} {
     $w item $item -open false
     $w focus $item
     event generate $w <<TreeviewClose>>
@@ -290,7 +289,7 @@ proc tile::treeview::CloseItem {w item} {
 
 ## Toggle -- toggle opened/closed state of item
 #
-proc tile::treeview::Toggle {w item} {
+proc ttk::treeview::Toggle {w item} {
     if {[$w item $item -open]} {
 	CloseItem $w $item
     } else {
@@ -300,7 +299,7 @@ proc tile::treeview::Toggle {w item} {
 
 ## ToggleFocus -- toggle opened/closed state of focus item
 #
-proc tile::treeview::ToggleFocus {w} {
+proc ttk::treeview::ToggleFocus {w} {
     set item [$w focus]
     if {$item ne ""} {
     	Toggle $w $item
@@ -309,10 +308,10 @@ proc tile::treeview::ToggleFocus {w} {
 
 ## BrowseTo -- navigate to specified item; set focus and selection
 #
-proc tile::treeview::BrowseTo {w item} {
+proc ttk::treeview::BrowseTo {w item} {
     $w see $item
     $w focus $item
-    $w selection set $item
+    $w selection set [list $item]
 }
 
 ### Style settings for selected built-in themes:
@@ -320,7 +319,7 @@ proc tile::treeview::BrowseTo {w item} {
 #   Do this here instead of in the theme definitions since the details are
 #   likely to change; it's better to keep this all in one place for now.
 #
-namespace eval ::tile::treeview {
+namespace eval ::ttk::treeview {
 variable theme
 foreach theme [style theme names] {
     style theme settings $theme {
