@@ -1,5 +1,5 @@
 #
-# winTheme.tcl,v 1.27 2005/09/14 17:24:28 jenglish Exp
+# winTheme.tcl,v 1.32 2006/04/16 18:45:24 jenglish Exp
 #
 # Tile widget set: Windows Native theme
 #
@@ -8,27 +8,21 @@ namespace eval tile {
 
     style theme settings winnative {
 
-	variable WinGUIFont "{MS Sans Serif} 8"
-	if {$tcl_platform(platform) eq "windows"
-	    && $tcl_platform(osVersion) >= 5.0} {
-	    set WinGUIFont "Tahoma 8"
-	}
-
 	style configure "." \
 	    -background SystemButtonFace \
 	    -foreground SystemWindowText \
 	    -selectforeground SystemHighlightText \
 	    -selectbackground SystemHighlight \
 	    -troughcolor SystemScrollbar \
-	    -font $WinGUIFont \
+	    -font TkDefaultFont \
 	    ;
 
 	style map "." -foreground [list disabled SystemGrayText] ;
         style map "." -embossed [list disabled 1] ;
 
 	style configure TButton -width -11 -relief raised -shiftrelief 1
-	style configure TCheckbutton -padding "0 2"
-	style configure TRadiobutton -padding "0 2"
+	style configure TCheckbutton -padding "2 4"
+	style configure TRadiobutton -padding "2 4"
 	style configure TMenubutton -padding "8 4" -arrowsize 3 -relief raised
 
 	style map TButton -relief {{!disabled pressed} sunken}
@@ -38,8 +32,8 @@ namespace eval tile {
 	style map TEntry \
 	    -fieldbackground \
 	    	[list readonly SystemButtonFace disabled SystemButtonFace] \
-	    -selectbackground [list !focus "#c3c3c3"] \
-	    -selectforeground [list !focus "#000000"] \
+	    -selectbackground [list !focus SystemWindow] \
+	    -selectforeground [list !focus SystemWindowText] \
 	    ;
 
 	style configure TCombobox -padding 2
@@ -50,11 +44,11 @@ namespace eval tile {
 	style map Toolbutton -relief \
 	    {disabled flat selected sunken  pressed sunken  active raised}
 
-	style configure TScale -groovewidth 4 -sliderrelief raised
-	style map TScale -sliderrelief {pressed sunken}
+	style configure TScale -groovewidth 4
 
-	style configure TNotebook -expandtab {2 2 2 0}
+	style configure TNotebook -tabmargins {2 2 2 0}
 	style configure TNotebook.Tab -padding {3 1} -borderwidth 1
+	style map TNotebook.Tab -expand [list selected {2 2 2 0}]
 
         style configure TProgressbar -borderwidth 0 -background SystemHighlight
     }
