@@ -1,5 +1,5 @@
 #
-# xpTheme.tcl,v 1.27 2005/09/14 17:24:28 jenglish Exp
+# xpTheme.tcl,v 1.32 2006/04/17 03:29:45 jenglish Exp
 #
 # Tile widget set: XP Native theme
 #
@@ -9,16 +9,12 @@ namespace eval tile {
 
     style theme settings xpnative {
 
-	# NOTE: MS documentation says to use "Tahoma 8" in Windows 2000 / XP,
-	# although many MS programs still use "MS Sans Serif 8"
-	#
-	style configure . -font "Tahoma 8"
-
 	style configure . \
 	    -background SystemButtonFace \
 	    -foreground SystemWindowText \
 	    -selectforeground SystemHighlightText \
 	    -selectbackground SystemHighlight \
+	    -font TkDefaultFont \
 	    ;
 
 	style map "." \
@@ -26,17 +22,21 @@ namespace eval tile {
 	    ;
 
 	style configure TButton -padding {1 1} -width -11
+	style configure TRadiobutton -padding 2
+	style configure TCheckbutton -padding 2
 	style configure TMenubutton -padding {8 4}
 
-	style configure TNotebook -expandtab {2 2 2 2}
+	style configure TNotebook -tabmargins {2 2 2 0}
+	style map TNotebook.Tab \
+	    -expand [list selected {2 2 2 2}]
 
 	style configure TLabelframe -foreground "#0046d5"
 
 	# OR: -padding {3 3 3 6}, which some apps seem to use.
 	style configure TEntry -padding {2 2 2 4}
 	style map TEntry \
-	    -selectbackground [list !focus "#c3c3c3"] \
-	    -selectforeground [list !focus "#000000"] \
+	    -selectbackground [list !focus SystemWindow] \
+	    -selectforeground [list !focus SystemWindowText] \
 	    ;
 	style configure TCombobox -padding 2
 
