@@ -1,5 +1,5 @@
 /*
- * aquaTheme.c,v 1.32 2005/10/07 22:34:09 jenglish Exp
+ * aquaTheme.c,v 1.33 2005/11/21 06:55:57 jenglish Exp
  *
  * Tk theme engine for Mac OSX, using the Appearance Manager API.
  *
@@ -92,24 +92,13 @@ typedef struct {
     ThemeMetric 	heightMetric;
 } ThemeButtonParms;
 
-enum {
-    KTHEMEPushButton,
-    KTHEMECheckBox,
-    KTHEMERadioButton,
-    KTHEMEBevelButton,
-    KTHEMEPopupButton,
-    KTHEMEListHeaderButton,
-    KTHEMEDisclosureButton };
-
-static ThemeButtonParms ButtonParms[] = {
-    { kThemePushButton, NoThemeMetric },
-    { kThemeCheckBox, kThemeMetricCheckBoxHeight },
-    { kThemeRadioButton, kThemeMetricRadioButtonHeight },
-    { kThemeBevelButton, NoThemeMetric },
-    { kThemePopupButton, NoThemeMetric },
-    { kThemeListHeaderButton, kThemeMetricListHeaderHeight },
-    { kThemeDisclosureButton, kThemeMetricDisclosureTriangleHeight }
-};
+static ThemeButtonParms
+    PushButtonParms = { kThemePushButton, NoThemeMetric },
+    CheckBoxParms = { kThemeCheckBox, kThemeMetricCheckBoxHeight },
+    RadioButtonParms = { kThemeRadioButton, kThemeMetricRadioButtonHeight },
+    BevelButtonParms = { kThemeBevelButton, NoThemeMetric },
+    PopupButtonParms = { kThemePopupButton, NoThemeMetric },
+    ListHeaderParms = { kThemeListHeaderButton, kThemeMetricListHeaderHeight };
 
 static Ttk_StateTable ButtonValueTable[] = {
     { kThemeButtonMixed, TTK_STATE_ALTERNATE, 0 },
@@ -196,7 +185,7 @@ static void ButtonElementDraw(
 
     BEGIN_DRAWING(d)
     DrawThemeButton(&bounds, parms->kind, &info,
-	NULL/*prevInfo*/,NULL/*eraseProc*/,NULL/*labelProc*/,NULL/*userData*/);
+	NULL/*prevInfo*/,NULL/*eraseProc*/,NULL/*labelProc*/,0/*userData*/);
     END_DRAWING
 }
 
@@ -420,7 +409,7 @@ static void PopupArrowElementDraw(
 
     BEGIN_DRAWING(d)
     DrawThemeButton(&bounds, kThemeArrowButton, &info,
-	NULL/*prevInfo*/,NULL/*eraseProc*/,NULL/*labelProc*/,NULL/*userData*/);
+	NULL/*prevInfo*/,NULL/*eraseProc*/,NULL/*labelProc*/,0/*userData*/);
 
     bounds = BoxToRect(Ttk_PadBox(b, ButtonMargins));
     bounds.top += 2;
@@ -432,8 +421,7 @@ static void PopupArrowElementDraw(
     	kThemeArrowDown, 
 	kThemeArrow9pt,		/* ??? */
 	Ttk_StateTableLookup(ThemeStateTable, state),
-	NULL /*eraseProc*/,
-	NULL /*eraseData*/);
+	NULL /*eraseProc*/,0/*eraseData*/);
     END_DRAWING
 }
 
@@ -822,7 +810,7 @@ static void TreeHeaderElementDraw(
 
     BEGIN_DRAWING(d)
     DrawThemeButton(&bounds, parms->kind, &info,
-	NULL/*prevInfo*/,NULL/*eraseProc*/,NULL/*labelProc*/,NULL/*userData*/);
+	NULL/*prevInfo*/,NULL/*eraseProc*/,NULL/*labelProc*/,0/*userData*/);
     END_DRAWING
 }
 
@@ -904,17 +892,17 @@ int AquaTheme_Init(Tcl_Interp *interp)
 	&ToolbarBackgroundElementSpec, 0);
 
     Ttk_RegisterElementSpec(themePtr, "Button.button",
-    	&ButtonElementSpec, ButtonParms + KTHEMEPushButton);
+    	&ButtonElementSpec, &PushButtonParms);
     Ttk_RegisterElementSpec(themePtr, "Checkbutton.button",
-    	&ButtonElementSpec, ButtonParms + KTHEMECheckBox);
+    	&ButtonElementSpec, &CheckBoxParms);
     Ttk_RegisterElementSpec(themePtr, "Radiobutton.button",
-    	&ButtonElementSpec, ButtonParms + KTHEMERadioButton);
+    	&ButtonElementSpec, &RadioButtonParms);
     Ttk_RegisterElementSpec(themePtr, "Toolbutton.border",
-    	&ButtonElementSpec, ButtonParms + KTHEMEBevelButton);
+    	&ButtonElementSpec, &BevelButtonParms);
     Ttk_RegisterElementSpec(themePtr, "Menubutton.button",
-    	&ButtonElementSpec, ButtonParms + KTHEMEPopupButton);
+    	&ButtonElementSpec, &PopupButtonParms);
     Ttk_RegisterElementSpec(themePtr, "Treeheading.cell",
-	&TreeHeaderElementSpec, ButtonParms + KTHEMEListHeaderButton); 
+	&TreeHeaderElementSpec, &ListHeaderParms); 
 
     Ttk_RegisterElementSpec(themePtr, "Notebook.tab", &TabElementSpec, 0);
     Ttk_RegisterElementSpec(themePtr, "Notebook.client", &PaneElementSpec, 0);
