@@ -1,11 +1,14 @@
-# tvtest.tcl,v 1.3 2005/06/26 03:08:33 jenglish Exp
+# tvtest.tcl,v 1.6 2006/04/16 17:00:10 jenglish Exp
 # 
 # Sandbox test script for treeview testing 
 #
 
 package require tile
 
+source [file join [file dirname [info script]] testutils.tcl]
+
 set haveswaplist [expr {![catch {package require swaplist}]}]
+set haveinplace  [expr {![catch {package require tile::inplace}]}]
 
 option add *Toolbar*TCheckbutton.style Toolbutton
 option add *tearOff 0
@@ -28,7 +31,21 @@ grid rowconfigure $f 0 -weight 1
 #
 # Status bar:
 #
-ttk::label .l -textvariable message
+
+style configure Indicator.TLabel -relief sunken -borderwidth 1
+option add *Indicator.style Indicator.TLabel
+
+set s [ttk::frame .status -class Statusbar]
+
+pack \
+    [ttk::label $s.l -class Indicator -textvariable message -width 20] \
+    [ttk::label $s.lr -text  "Row:" ] \
+    [ttk::label $s.r -class Indicator -textvariable ::irow -width 10] \
+    [ttk::label $s.lc -text  "Col:" ] \
+    [ttk::label $s.c -class Indicator -textvariable ::icol -width 10] \
+    [ttk::label $s.lb -text  "BBox:" ] \
+    [ttk::label $s.b -class Indicator -textvariable ::bbox -width 16] \
+-side left -pady 2;
 
 #
 # View control:
@@ -85,14 +102,37 @@ $tv heading Y -text "YYY"
 $tv heading Z -text "ZZZ"
 
 pack .t -side top -expand false -fill x
-pack .l -side bottom -expand false -fill x
+pack .status -side bottom -expand false -fill x
 pack .f -side top -expand true -fill both -padx 6 -pady 6
 
 bind . <KeyPress-Escape> [list destroy .]
 
-bind $tv <Motion> { set ::message [%W identify %x %y] }
+bind $tv <Motion> { identify %W %x %y }
 bind $tv <KeyPress-Delete> { %W delete [%W selection] }
 
+# identify --
+#	Update status bar
+#
+proc identify {tv x y} {
+    # Old form:
+    set ::message [$tv identify $x $y]
+
+    # New form:
+    set ::irow [$tv identify row $x $y]
+    set ::icol [$tv identify column $x $y]
+
+    if {$::irow ne ""} {
+	if {$::icol ne ""} { 
+	    set ::bbox [$tv bbox $::irow $::icol] 
+	} else {
+	    set ::bbox [$tv bbox $::irow]
+	}
+    } else {
+	set ::bbox N/A
+    }
+}
+
+
 $tv insert {} end -id a -text "First node" -values [list a1 a2 a3]
 $tv insert {} end -id b -text "Second node" -values [list b1 b2 b3]
 $tv insert {} end -id c -text "Third node" -values [list c1 c2 c3]
@@ -178,4 +218,16 @@ proc selectColumns {tv} {
     }
 }
 
+# ...
+if {$haveinplace} {
 
+    bind $tv <ButtonPress-3> { clickToEdit %W %x %y }
+
+    proc clickToEdit {tv x y} {
+	set row [$tv identify row $x $y]
+	set column [$tv identify column $x $y]
+	if {$row ne "" && $column ne ""} {
+	    ttk::treeview::edit $tv $row $column
+	}
+    }
+}
