@@ -1,6 +1,6 @@
 /* winTheme.c - Copyright (C) 2004 Pat Thoyts <patthoyts@users.sf.net>
  *
- * winTheme.c,v 1.29 2005/03/26 20:07:08 jenglish Exp
+ * winTheme.c,v 1.33 2006/04/06 20:02:53 jenglish Exp
  */
 
 #ifdef _MSC_VER
@@ -41,19 +41,23 @@ static RECT BoxToRect(Ttk_Box b)
 /*
  * ReliefToEdge --
  * 	Convert a Tk "relief" value into an Windows "edge" value.
- * 	NB: Caller must check for RELIEF_FLAT, which does not correspond
- * 	to any Win32 enum.
+ * 	NB: Caller must check for RELIEF_FLAT and RELIEF_SOLID,
+ *	which must be handled specially.
+ *
+ *	Passing the BF_FLAT flag to DrawEdge() yields something similar
+ * 	to TK_RELIEF_SOLID. TK_RELIEF_FLAT can be implemented by not 
+ *	drawing anything.
  */
 static unsigned int ReliefToEdge(int relief)
 {
     switch (relief) {
-	case TK_RELIEF_FLAT:   return BDR_RAISEDOUTER;	/* @@@ */
 	case TK_RELIEF_RAISED: return EDGE_RAISED;
 	case TK_RELIEF_SUNKEN: return EDGE_SUNKEN;
+	case TK_RELIEF_RIDGE:  return EDGE_BUMP;
 	case TK_RELIEF_GROOVE: return EDGE_ETCHED;
 	case TK_RELIEF_SOLID:  return BDR_RAISEDOUTER;
-	case TK_RELIEF_RIDGE:  return EDGE_BUMP;
-	default:               return BDR_RAISEDOUTER;
+	default:
+	case TK_RELIEF_FLAT:   return BDR_RAISEDOUTER;
     }
 }
 
@@ -193,8 +197,9 @@ static void BorderElementDraw(
     Tk_GetReliefFromObj(NULL, border->reliefObj, &relief);
 
     if (relief != TK_RELIEF_FLAT) {
+	UINT xFlags = (relief == TK_RELIEF_SOLID) ? BF_FLAT : 0;
 	hdc = TkWinGetDrawableDC(Tk_Display(tkwin), d, &dcState);
-        DrawEdge(hdc, &rc, ReliefToEdge(relief), BF_RECT);
+	DrawEdge(hdc, &rc, ReliefToEdge(relief), BF_RECT | xFlags);
 	TkWinReleaseDrawableDC(d, hdc, &dcState);
     }
 }
@@ -494,13 +499,11 @@ static Ttk_ElementSpec TroughElementSpec =
 
 typedef struct
 {
-    Tcl_Obj *reliefObj;
     Tcl_Obj *orientObj;
 } ThumbElement;
 
 static Ttk_ElementOptionSpec ThumbElementOptions[] =
 {
-    { "-relief", TK_OPTION_RELIEF,Tk_Offset(ThumbElement,reliefObj),"raised" },
     { "-orient", TK_OPTION_ANY,Tk_Offset(ThumbElement,orientObj),"horizontal"},
     { NULL }
 };
@@ -526,19 +529,16 @@ static void ThumbElementDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     Drawable d, Ttk_Box b, unsigned int state)
 {
-    ThumbElement *thumbPtr = elementRecord;
     RECT rc = BoxToRect(b);
     TkWinDCState dcState;
     HDC hdc;
-    int relief = TK_RELIEF_FLAT;
 
     /* Windows doesn't show a thumb when the scrollbar is disabled */
     if (state & TTK_STATE_DISABLED)
 	return;
 
-    Tk_GetReliefFromObj(NULL, thumbPtr->reliefObj, &relief);
     hdc = TkWinGetDrawableDC(Tk_Display(tkwin), d, &dcState);
-    DrawEdge(hdc, &rc, ReliefToEdge(relief), BF_RECT | BF_MIDDLE);
+    DrawEdge(hdc, &rc, EDGE_RAISED, BF_RECT | BF_MIDDLE);
     TkWinReleaseDrawableDC(d, hdc, &dcState);
 }
 
@@ -558,14 +558,11 @@ static Ttk_ElementSpec ThumbElementSpec =
 
 typedef struct
 {
-    Tcl_Obj *reliefObj;
     Tcl_Obj *orientObj;  /* orientation of the slider widget */
 } SliderElement;
 
 static Ttk_ElementOptionSpec SliderElementOptions[] =
 {
-    { "-relief", TK_OPTION_RELIEF,Tk_Offset(SliderElement,reliefObj),
-      "raised" },
     { "-orient", TK_OPTION_ANY, Tk_Offset(SliderElement,orientObj),
       "horizontal" },
     { NULL }
@@ -575,10 +572,10 @@ static void SliderElementGeometry(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
-    ThumbElement *thumbPtr = elementRecord;
+    SliderElement *slider = elementRecord;
     int orient;
 
-    Ttk_GetOrientFromObj(NULL, thumbPtr->orientObj, &orient);
+    Ttk_GetOrientFromObj(NULL, slider->orientObj, &orient);
     if (orient == TTK_ORIENT_HORIZONTAL) {
 	*widthPtr = (GetSystemMetrics(SM_CXHTHUMB) / 2) | 1;
 	*heightPtr = GetSystemMetrics(SM_CYHSCROLL);
@@ -592,47 +589,12 @@ static void SliderElementDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     Drawable d, Ttk_Box b, unsigned int state)
 {
-    SliderElement *slider = elementRecord;
     RECT rc = BoxToRect(b);
-    RECT rct;
     TkWinDCState dcState;
     HDC hdc;
-    int orient, relief, edge;
 
-    Tk_GetReliefFromObj(NULL, slider->reliefObj, &relief);
-    Ttk_GetOrientFromObj(NULL, slider->orientObj, &orient);
     hdc = TkWinGetDrawableDC(Tk_Display(tkwin), d, &dcState);
-    edge = ReliefToEdge(relief);
-
-    if (orient == TTK_ORIENT_HORIZONTAL) {
-	int h = (rc.bottom - rc.top) / 4;
-	int w = (rc.right - rc.left) / 2;
-	CopyRect(&rct, &rc);
-	rct.bottom -= h;
-	DrawEdge(hdc, &rct, edge, BF_TOPLEFT | BF_MIDDLE);
-	DrawEdge(hdc, &rct, edge, BF_RIGHT);
-	rct.top += h * 3;
-	rct.bottom = rc.bottom;
-	rct.right -= w;
-	DrawEdge(hdc, &rct, edge, BF_DIAGONAL_ENDBOTTOMRIGHT);
-	rct.right = rc.right;
-	rct.left += w;
-	DrawEdge(hdc, &rct, edge, BF_DIAGONAL_ENDBOTTOMLEFT);
-    } else {
-	int h = (rc.bottom - rc.top) / 2;
-	int w = (rc.right - rc.left) / 4;
-	CopyRect(&rct, &rc);
-	rct.right -= w;
-	DrawEdge(hdc, &rct, edge, BF_TOPLEFT | BF_MIDDLE);
-	DrawEdge(hdc, &rct, edge, BF_BOTTOM);
-	rct.left += w * 3;
-	rct.right = rc.right;
-	rct.bottom -= h;
-	DrawEdge(hdc, &rct, edge, BF_DIAGONAL_ENDBOTTOMRIGHT);
-	rct.bottom = rc.bottom;
-	rct.top += h;
-	DrawEdge(hdc, &rct, edge, BF_DIAGONAL_ENDBOTTOMLEFT);
-    }
+    DrawEdge(hdc, &rc, EDGE_RAISED, BF_RECT | BF_MIDDLE);
     TkWinReleaseDrawableDC(d, hdc, &dcState);
 }
 
