@@ -1,5 +1,5 @@
 /*
- * xpTheme.c,v 1.81 2005/10/08 16:39:09 jenglish Exp
+ * xpTheme.c,v 1.82 2006/02/05 21:29:18 jenglish Exp
  *
  * Tk theme engine which uses the Windows XP "Visual Styles" API
  * Adapted from Georgios Petasis' XP theme patch.
@@ -922,6 +922,8 @@ static ElementInfo ElementInfoTable[] = {
     	TVP_GLYPH, tvpglyph_statemap, PAD(1,1,6,0), 0 },
     { "Treeheading.border", &GenericElementSpec, L"HEADER", 
     	HP_HEADERITEM, header_statemap, PAD(4,0,4,0),0 },
+    { "sizegrip", &GenericElementSpec, L"STATUS", 
+    	SP_GRIPPER, null_statemap, NOPAD,0 },
 
 #if BROKEN_TEXT_ELEMENT
     { "Labelframe.text", &TextElementSpec, L"BUTTON", 
